@@ -36,6 +36,8 @@ static void lptest_init(void);
 static void lptest_finish(void);
 #endif
 
+static void bgp_sync_label_manager(struct event *e);
+
 /*
  * Remember where pool data are kept
  */
@@ -75,6 +77,7 @@ struct lp_lcb {
 	mpls_label_t	label;		/* MPLS_LABEL_NONE = not allocated */
 	int		type;
 	void		*labelid;	/* unique ID */
+	vrf_id_t	vrf_id;
 	/*
 	 * callback for label allocation and loss
 	 *
@@ -95,6 +98,7 @@ struct lp_cbq_item {
 	int		type;
 	mpls_label_t	label;
 	void		*labelid;
+	vrf_id_t	vrf_id;
 	bool		allocated;	/* false = lost */
 };
 
@@ -103,6 +107,7 @@ static wq_item_status lp_cbq_docallback(struct work_queue *wq, void *data)
 	struct lp_cbq_item *lcbq = data;
 	int rc;
 	int debug = BGP_DEBUG(labelpool, LABELPOOL);
+	struct bgp *bgp = bgp_lookup_by_vrf_id(lcbq->vrf_id);
 
 	if (debug)
 		zlog_debug("%s: calling callback with labelid=%p label=%u allocated=%d",
@@ -115,6 +120,9 @@ static wq_item_status lp_cbq_docallback(struct work_queue *wq, void *data)
 		return WQ_SUCCESS;
 	}
 
+	if (!bgp)
+		return WQ_SUCCESS;
+
 	rc = (*(lcbq->cbfunc))(lcbq->label, lcbq->labelid, lcbq->allocated);
 
 	if (lcbq->allocated && rc) {
@@ -318,10 +326,8 @@ static mpls_label_t get_label_from_pool(void *labelid)
 /*
  * Success indicated by value of "label" field in returned LCB
  */
-static struct lp_lcb *lcb_alloc(
-	int	type,
-	void	*labelid,
-	int	(*cbfunc)(mpls_label_t label, void *labelid, bool allocated))
+static struct lp_lcb *lcb_alloc(int type, void *labelid, vrf_id_t vrf_id,
+				int (*cbfunc)(mpls_label_t label, void *labelid, bool allocated))
 {
 	/*
 	 * Set up label control block
@@ -332,6 +338,7 @@ static struct lp_lcb *lcb_alloc(
 	new->label = get_label_from_pool(labelid);
 	new->type = type;
 	new->labelid = labelid;
+	new->vrf_id = vrf_id;
 	new->cbfunc = cbfunc;
 
 	return new;
@@ -363,10 +370,8 @@ static struct lp_lcb *lcb_alloc(
  * Prior requests for a given labelid are detected so that requests and
  * assignments are not duplicated.
  */
-void bgp_lp_get(
-	int	type,
-	void	*labelid,
-	int	(*cbfunc)(mpls_label_t label, void *labelid, bool allocated))
+void bgp_lp_get(int type, void *labelid, vrf_id_t vrf_id,
+		int (*cbfunc)(mpls_label_t label, void *labelid, bool allocated))
 {
 	struct lp_lcb *lcb;
 	int requested = 0;
@@ -381,7 +386,7 @@ void bgp_lp_get(
 	if (!skiplist_search(lp->ledger, labelid, (void **)&lcb)) {
 		requested = 1;
 	} else {
-		lcb = lcb_alloc(type, labelid, cbfunc);
+		lcb = lcb_alloc(type, labelid, vrf_id, cbfunc);
 		if (debug)
 			zlog_debug("%s: inserting lcb=%p label=%u",
 				__func__, lcb, lcb->label);
@@ -411,6 +416,7 @@ void bgp_lp_get(
 		q->type = lcb->type;
 		q->label = lcb->label;
 		q->labelid = lcb->labelid;
+		q->vrf_id = lcb->vrf_id;
 		q->allocated = true;
 
 		/* if this is a LU request, lock node before queueing */
@@ -448,13 +454,16 @@ void bgp_lp_get(
 
 	if (lp_fifo_count(&lp->requests) > lp->pending_count) {
 		if (!bgp_zebra_request_label_range(MPLS_LABEL_BASE_ANY,
-						   lp->next_chunksize))
+						   lp->next_chunksize, true))
 			return;
 
 		lp->pending_count += lp->next_chunksize;
 		if ((lp->next_chunksize << 1) <= LP_CHUNK_SIZE_MAX)
 			lp->next_chunksize <<= 1;
 	}
+
+	event_add_timer(bm->master, bgp_sync_label_manager, NULL, 1,
+			&bm->t_bgp_sync_label_manager);
 }
 
 void bgp_lp_release(
@@ -494,8 +503,18 @@ void bgp_lp_release(
 				bf_release_index(chunk->allocated_map, index);
 				chunk->nfree += 1;
 				deallocated = true;
+				break;
 			}
 			assert(deallocated);
+			if (deallocated &&
+			    chunk->nfree == chunk->last - chunk->first + 1 &&
+			    lp_fifo_count(&lp->requests) == 0) {
+				bgp_zebra_release_label_range(chunk->first,
+							      chunk->last);
+				list_delete_node(lp->chunks, node);
+				lp_chunk_free(chunk);
+				lp->next_chunksize = LP_CHUNK_SIZE_MIN;
+			}
 		}
 	}
 }
@@ -547,6 +566,10 @@ static void bgp_sync_label_manager(struct event *e)
 				zlog_debug("%s: out of labels, await more",
 					   __func__);
 			}
+
+			lp_fifo_add_tail(&lp->requests, lf);
+			event_add_timer(bm->master, bgp_sync_label_manager,
+					NULL, 1, &bm->t_bgp_sync_label_manager);
 			break;
 		}
 
@@ -561,6 +584,7 @@ static void bgp_sync_label_manager(struct event *e)
 		q->type = lcb->type;
 		q->label = lcb->label;
 		q->labelid = lcb->labelid;
+		q->vrf_id = lcb->vrf_id;
 		q->allocated = true;
 
 		if (debug)
@@ -572,9 +596,6 @@ static void bgp_sync_label_manager(struct event *e)
 finishedrequest:
 		XFREE(MTYPE_BGP_LABEL_FIFO, lf);
 	}
-
-	event_add_timer(bm->master, bgp_sync_label_manager, NULL, 1,
-			&bm->t_bgp_sync_label_manager);
 }
 
 void bgp_lp_event_chunk(uint32_t first, uint32_t last)
@@ -642,7 +663,7 @@ void bgp_lp_event_zebra_up(void)
 	}
 
 	/* round up */
-	chunks_needed = (labels_needed / lp->next_chunksize) + 1;
+	chunks_needed = (labels_needed + lp->next_chunksize - 1) / lp->next_chunksize;
 	labels_needed = chunks_needed * lp->next_chunksize;
 
 	/*
@@ -650,10 +671,10 @@ void bgp_lp_event_zebra_up(void)
 	 */
 	list_delete_all_node(lp->chunks);
 
-	if (!bgp_zebra_request_label_range(MPLS_LABEL_BASE_ANY, labels_needed))
+	if (labels_needed && !bgp_zebra_request_label_range(MPLS_LABEL_BASE_ANY,
+							    labels_needed, true))
 		return;
-
-	lp->pending_count = labels_needed;
+	lp->pending_count += labels_needed;
 
 	/*
 	 * Invalidate any existing labels and requeue them as requests
@@ -677,6 +698,7 @@ void bgp_lp_event_zebra_up(void)
 				q->type = lcb->type;
 				q->label = lcb->label;
 				q->labelid = lcb->labelid;
+				q->vrf_id = lcb->vrf_id;
 				q->allocated = false;
 				check_bgp_lu_cb_lock(lcb);
 				work_queue_add(lp->callback_q, q);
