@@ -129,15 +129,19 @@ int bgp_path_info_nexthop_cmp(struct bgp_path_info *bpi1,
 					&bpi2->attr->mp_nexthop_global);
 				break;
 			case BGP_ATTR_NHLEN_IPV6_GLOBAL_AND_LL:
-				addr1 = (bpi1->attr->mp_nexthop_prefer_global)
+				addr1 = (CHECK_FLAG(bpi1->attr->nh_flags,
+						    BGP_ATTR_NH_MP_PREFER_GLOBAL))
 						? bpi1->attr->mp_nexthop_global
 						: bpi1->attr->mp_nexthop_local;
-				addr2 = (bpi2->attr->mp_nexthop_prefer_global)
+				addr2 = (CHECK_FLAG(bpi2->attr->nh_flags,
+						    BGP_ATTR_NH_MP_PREFER_GLOBAL))
 						? bpi2->attr->mp_nexthop_global
 						: bpi2->attr->mp_nexthop_local;
 
-				if (!bpi1->attr->mp_nexthop_prefer_global
-				    && !bpi2->attr->mp_nexthop_prefer_global)
+				if (!CHECK_FLAG(bpi1->attr->nh_flags,
+						BGP_ATTR_NH_MP_PREFER_GLOBAL) &&
+				    !CHECK_FLAG(bpi2->attr->nh_flags,
+						BGP_ATTR_NH_MP_PREFER_GLOBAL))
 					compare = !bgp_interface_same(
 						bpi1->peer->ifp,
 						bpi2->peer->ifp);
@@ -517,7 +521,7 @@ void bgp_path_info_mpath_update(struct bgp *bgp, struct bgp_dest *dest,
 				struct bgp_maxpaths_cfg *mpath_cfg)
 {
 	uint16_t maxpaths, mpath_count, old_mpath_count;
-	uint32_t bwval;
+	uint64_t bwval;
 	uint64_t cum_bw, old_cum_bw;
 	struct listnode *mp_node, *mp_next_node;
 	struct bgp_path_info *cur_mpath, *new_mpath, *next_mpath, *prev_mpath;
@@ -554,12 +558,11 @@ void bgp_path_info_mpath_update(struct bgp *bgp, struct bgp_dest *dest,
 	}
 
 	if (debug)
-		zlog_debug(
-			"%pRN(%s): starting mpath update, newbest %s num candidates %d old-mpath-count %d old-cum-bw %" PRIu64,
-			bgp_dest_to_rnode(dest), bgp->name_pretty,
-			new_best ? new_best->peer->host : "NONE",
-			mp_list ? listcount(mp_list) : 0, old_mpath_count,
-			old_cum_bw);
+		zlog_debug("%pBD(%s): starting mpath update, newbest %s num candidates %d old-mpath-count %d old-cum-bw %" PRIu64,
+			   dest, bgp->name_pretty,
+			   new_best ? new_best->peer->host : "NONE",
+			   mp_list ? listcount(mp_list) : 0, old_mpath_count,
+			   old_cum_bw);
 
 	/*
 	 * We perform an ordered walk through both lists in parallel.
@@ -590,11 +593,10 @@ void bgp_path_info_mpath_update(struct bgp *bgp, struct bgp_dest *dest,
 		tmp_info = mp_node ? listgetdata(mp_node) : NULL;
 
 		if (debug)
-			zlog_debug(
-				"%pRN(%s): comparing candidate %s with existing mpath %s",
-				bgp_dest_to_rnode(dest), bgp->name_pretty,
-				tmp_info ? tmp_info->peer->host : "NONE",
-				cur_mpath ? cur_mpath->peer->host : "NONE");
+			zlog_debug("%pBD(%s): comparing candidate %s with existing mpath %s",
+				   dest, bgp->name_pretty,
+				   tmp_info ? tmp_info->peer->host : "NONE",
+				   cur_mpath ? cur_mpath->peer->host : "NONE");
 
 		/*
 		 * If equal, the path was a multipath and is still a multipath.
@@ -611,8 +613,11 @@ void bgp_path_info_mpath_update(struct bgp *bgp, struct bgp_dest *dest,
 							    cur_mpath);
 				prev_mpath = cur_mpath;
 				mpath_count++;
-				if (ecommunity_linkbw_present(
-					    bgp_attr_get_ecommunity(
+				if (ecommunity_linkbw_present(bgp_attr_get_ecommunity(
+								      cur_mpath->attr),
+							      &bwval) ||
+				    ecommunity_linkbw_present(
+					    bgp_attr_get_ipv6_ecommunity(
 						    cur_mpath->attr),
 					    &bwval))
 					cum_bw += bwval;
@@ -622,10 +627,8 @@ void bgp_path_info_mpath_update(struct bgp *bgp, struct bgp_dest *dest,
 					bgp_path_info_path_with_addpath_rx_str(
 						cur_mpath, path_buf,
 						sizeof(path_buf));
-					zlog_debug(
-						"%pRN: %s is still multipath, cur count %d",
-						bgp_dest_to_rnode(dest),
-						path_buf, mpath_count);
+					zlog_debug("%pBD: %s is still multipath, cur count %d",
+						   dest, path_buf, mpath_count);
 				}
 			} else {
 				mpath_changed = 1;
@@ -633,12 +636,10 @@ void bgp_path_info_mpath_update(struct bgp *bgp, struct bgp_dest *dest,
 					bgp_path_info_path_with_addpath_rx_str(
 						cur_mpath, path_buf,
 						sizeof(path_buf));
-					zlog_debug(
-						"%pRN: remove mpath %s nexthop %pI4, cur count %d",
-						bgp_dest_to_rnode(dest),
-						path_buf,
-						&cur_mpath->attr->nexthop,
-						mpath_count);
+					zlog_debug("%pBD: remove mpath %s nexthop %pI4, cur count %d",
+						   dest, path_buf,
+						   &cur_mpath->attr->nexthop,
+						   mpath_count);
 				}
 			}
 			mp_node = mp_next_node;
@@ -663,10 +664,10 @@ void bgp_path_info_mpath_update(struct bgp *bgp, struct bgp_dest *dest,
 			if (debug) {
 				bgp_path_info_path_with_addpath_rx_str(
 					cur_mpath, path_buf, sizeof(path_buf));
-				zlog_debug(
-					"%pRN: remove mpath %s nexthop %pI4, cur count %d",
-					bgp_dest_to_rnode(dest), path_buf,
-					&cur_mpath->attr->nexthop, mpath_count);
+				zlog_debug("%pBD: remove mpath %s nexthop %pI4, cur count %d",
+					   dest, path_buf,
+					   &cur_mpath->attr->nexthop,
+					   mpath_count);
 			}
 			cur_mpath = next_mpath;
 		} else {
@@ -702,8 +703,11 @@ void bgp_path_info_mpath_update(struct bgp *bgp, struct bgp_dest *dest,
 				prev_mpath = new_mpath;
 				mpath_changed = 1;
 				mpath_count++;
-				if (ecommunity_linkbw_present(
-					    bgp_attr_get_ecommunity(
+				if (ecommunity_linkbw_present(bgp_attr_get_ecommunity(
+								      new_mpath->attr),
+							      &bwval) ||
+				    ecommunity_linkbw_present(
+					    bgp_attr_get_ipv6_ecommunity(
 						    new_mpath->attr),
 					    &bwval))
 					cum_bw += bwval;
@@ -713,12 +717,10 @@ void bgp_path_info_mpath_update(struct bgp *bgp, struct bgp_dest *dest,
 					bgp_path_info_path_with_addpath_rx_str(
 						new_mpath, path_buf,
 						sizeof(path_buf));
-					zlog_debug(
-						"%pRN: add mpath %s nexthop %pI4, cur count %d",
-						bgp_dest_to_rnode(dest),
-						path_buf,
-						&new_mpath->attr->nexthop,
-						mpath_count);
+					zlog_debug("%pBD: add mpath %s nexthop %pI4, cur count %d",
+						   dest, path_buf,
+						   &new_mpath->attr->nexthop,
+						   mpath_count);
 				}
 			}
 			mp_node = mp_next_node;
@@ -728,8 +730,12 @@ void bgp_path_info_mpath_update(struct bgp *bgp, struct bgp_dest *dest,
 	if (new_best) {
 		bgp_path_info_mpath_count_set(new_best, mpath_count - 1);
 		if (mpath_count <= 1 ||
-		    !ecommunity_linkbw_present(
-			    bgp_attr_get_ecommunity(new_best->attr), &bwval))
+		    (!ecommunity_linkbw_present(bgp_attr_get_ecommunity(
+							new_best->attr),
+						&bwval) &&
+		     !ecommunity_linkbw_present(bgp_attr_get_ipv6_ecommunity(
+							new_best->attr),
+						&bwval)))
 			all_paths_lb = false;
 		else
 			cum_bw += bwval;
@@ -737,11 +743,10 @@ void bgp_path_info_mpath_update(struct bgp *bgp, struct bgp_dest *dest,
 					      all_paths_lb, cum_bw);
 
 		if (debug)
-			zlog_debug(
-				"%pRN(%s): New mpath count (incl newbest) %d mpath-change %s all_paths_lb %d cum_bw %" PRIu64,
-				bgp_dest_to_rnode(dest), bgp->name_pretty,
-				mpath_count, mpath_changed ? "YES" : "NO",
-				all_paths_lb, cum_bw);
+			zlog_debug("%pBD(%s): New mpath count (incl newbest) %d mpath-change %s all_paths_lb %d cum_bw %" PRIu64,
+				   dest, bgp->name_pretty, mpath_count,
+				   mpath_changed ? "YES" : "NO", all_paths_lb,
+				   cum_bw);
 
 		if (mpath_changed
 		    || (bgp_path_info_mpath_count(new_best) != old_mpath_count))
