@@ -26,6 +26,8 @@ PREDECL_RBTREE_UNIQ(bgp_nexthop_cache);
 
 /* BGP nexthop cache value structure. */
 struct bgp_nexthop_cache {
+	afi_t afi;
+
 	/* The ifindex of the outgoing interface *if* it's a v6 LL */
 	ifindex_t ifindex_ipv6_ll;
 
@@ -37,6 +39,18 @@ struct bgp_nexthop_cache {
 
 	/* Nexthop number and nexthop linked list.*/
 	uint8_t nexthop_num;
+
+	/* This flag is set to TRUE for a bnc that is gateway IP overlay index
+	 * nexthop.
+	 */
+	bool is_evpn_gwip_nexthop;
+
+	uint16_t change_flags;
+#define BGP_NEXTHOP_CHANGED	      (1 << 0)
+#define BGP_NEXTHOP_METRIC_CHANGED    (1 << 1)
+#define BGP_NEXTHOP_CONNECTED_CHANGED (1 << 2)
+#define BGP_NEXTHOP_MACIP_CHANGED     (1 << 3)
+
 	struct nexthop *nexthop;
 	time_t last_update;
 	uint16_t flags;
@@ -70,27 +84,17 @@ struct bgp_nexthop_cache {
  */
 #define BGP_NEXTHOP_EVPN_INCOMPLETE (1 << 7)
 
-	uint16_t change_flags;
-
-#define BGP_NEXTHOP_CHANGED           (1 << 0)
-#define BGP_NEXTHOP_METRIC_CHANGED    (1 << 1)
-#define BGP_NEXTHOP_CONNECTED_CHANGED (1 << 2)
-#define BGP_NEXTHOP_MACIP_CHANGED (1 << 3)
+	uint32_t srte_color;
 
 	/* Back pointer to the cache tree this entry belongs to. */
 	struct bgp_nexthop_cache_head *tree;
 
-	uint32_t srte_color;
 	struct prefix prefix;
+	struct prefix resolved_prefix;
 	void *nht_info; /* In BGP, peer session */
 	LIST_HEAD(path_list, bgp_path_info) paths;
 	unsigned int path_count;
 	struct bgp *bgp;
-
-	/* This flag is set to TRUE for a bnc that is gateway IP overlay index
-	 * nexthop.
-	 */
-	bool is_evpn_gwip_nexthop;
 };
 
 extern int bgp_nexthop_cache_compare(const struct bgp_nexthop_cache *a,
