@@ -1,21 +1,8 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  *
  * Copyright 2009-2016, LabN Consulting, L.L.C.
  *
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 /*
@@ -134,10 +121,11 @@ static void encap_attr_export_ce(struct attr *new, struct attr *orig,
 static int getce(struct bgp *bgp, struct attr *attr, struct prefix *pfx_ce)
 {
 	uint8_t *ecp;
-	int i;
+	uint32_t i;
 	uint16_t localadmin = bgp->rfapi_cfg->resolve_nve_roo_local_admin;
+	struct ecommunity *ecomm = bgp_attr_get_ecommunity(attr);
 
-	for (ecp = attr->ecommunity->val, i = 0; i < attr->ecommunity->size;
+	for (ecp = ecomm->val, i = 0; i < ecomm->size;
 	     ++i, ecp += ECOMMUNITY_SIZE) {
 
 		if (VNC_DEBUG(EXPORT_BGP_GETCE)) {
@@ -164,7 +152,7 @@ static int getce(struct bgp *bgp, struct attr *attr, struct prefix *pfx_ce)
 		memset((uint8_t *)pfx_ce, 0, sizeof(*pfx_ce));
 		memcpy(&pfx_ce->u.prefix4, ecp + 2, 4);
 		pfx_ce->family = AF_INET;
-		pfx_ce->prefixlen = 32;
+		pfx_ce->prefixlen = IPV4_MAX_BITLEN;
 
 		return 0;
 	}
@@ -179,7 +167,7 @@ void vnc_direct_bgp_add_route_ce(struct bgp *bgp, struct agg_node *rn,
 	struct peer *peer = bpi->peer;
 	const struct prefix *prefix = agg_node_get_prefix(rn);
 	afi_t afi = family2afi(prefix->family);
-	struct bgp_node *urn;
+	struct bgp_dest *udest;
 	struct bgp_path_info *ubpi;
 	struct attr hattr;
 	struct attr *iattr;
@@ -254,9 +242,10 @@ void vnc_direct_bgp_add_route_ce(struct bgp *bgp, struct agg_node *rn,
 	 * Is this route already represented in the unicast RIB?
 	 * (look up prefix; compare route type, sub_type, peer, nexthop)
 	 */
-	urn = bgp_afi_node_get(bgp->rib[afi][SAFI_UNICAST], afi, SAFI_UNICAST,
-			       prefix, NULL);
-	for (ubpi = bgp_node_get_bgp_path_info(urn); ubpi; ubpi = ubpi->next) {
+	udest = bgp_afi_node_get(bgp->rib[afi][SAFI_UNICAST], afi, SAFI_UNICAST,
+				 prefix, NULL);
+	for (ubpi = bgp_dest_get_bgp_path_info(udest); ubpi;
+	     ubpi = ubpi->next) {
 		struct prefix unicast_nexthop;
 
 		if (CHECK_FLAG(ubpi->flags, BGP_PATH_REMOVED))
@@ -289,7 +278,7 @@ void vnc_direct_bgp_add_route_ce(struct bgp *bgp, struct agg_node *rn,
 		info.peer = peer;
 		info.attr = &hattr;
 		ret = route_map_apply(bgp->rfapi_cfg->routemap_export_bgp,
-				      prefix, RMAP_BGP, &info);
+				      prefix, &info);
 		if (ret == RMAP_DENYMATCH) {
 			bgp_attr_flush(&hattr);
 			return;
@@ -397,7 +386,6 @@ void vnc_direct_bgp_del_route_ce(struct bgp *bgp, struct agg_node *rn,
 	 * withdraw the route
 	 */
 	bgp_withdraw(bpi->peer, p, 0, /* addpath_id */
-		     NULL,	      /* attr, ignored */
 		     afi, SAFI_UNICAST, ZEBRA_ROUTE_VNC_DIRECT,
 		     BGP_ROUTE_REDISTRIBUTE, NULL, /* RD not used for unicast */
 		     NULL, 0, NULL); /* tag not used for unicast */
@@ -456,7 +444,7 @@ static void vnc_direct_bgp_vpn_enable_ce(struct bgp *bgp, afi_t afi)
 
 static void vnc_direct_bgp_vpn_disable_ce(struct bgp *bgp, afi_t afi)
 {
-	struct bgp_node *rn;
+	struct bgp_dest *dest;
 
 	vnc_zlog_debug_verbose("%s: entry, afi=%d", __func__, afi);
 
@@ -472,14 +460,14 @@ static void vnc_direct_bgp_vpn_disable_ce(struct bgp *bgp, afi_t afi)
 	 * Go through the entire BGP unicast table and remove routes that
 	 * originated from us
 	 */
-	for (rn = bgp_table_top(bgp->rib[afi][SAFI_UNICAST]); rn;
-	     rn = bgp_route_next(rn)) {
+	for (dest = bgp_table_top(bgp->rib[afi][SAFI_UNICAST]); dest;
+	     dest = bgp_route_next(dest)) {
 
 		struct bgp_path_info *ri;
 		struct bgp_path_info *next;
 
-		for (ri = bgp_node_get_bgp_path_info(rn), next = NULL;
-		     ri; ri = next) {
+		for (ri = bgp_dest_get_bgp_path_info(dest), next = NULL; ri;
+		     ri = next) {
 
 			next = ri->next;
 
@@ -487,9 +475,8 @@ static void vnc_direct_bgp_vpn_disable_ce(struct bgp *bgp, afi_t afi)
 			    && ri->sub_type == BGP_ROUTE_REDISTRIBUTE) {
 
 				bgp_withdraw(
-					ri->peer, bgp_node_get_prefix(rn),
+					ri->peer, bgp_dest_get_prefix(dest),
 					0,    /* addpath_id */
-					NULL, /* ignored */
 					AFI_IP, SAFI_UNICAST,
 					ZEBRA_ROUTE_VNC_DIRECT,
 					BGP_ROUTE_REDISTRIBUTE,
@@ -562,7 +549,6 @@ static struct ecommunity *vnc_route_origin_ecom_single(struct in_addr *origin)
 	roec.val[7] = 0;
 
 	new = ecommunity_new();
-	assert(new);
 	ecommunity_add_val(new, &roec, false, false);
 
 	if (!new->size) {
@@ -596,10 +582,10 @@ encap_attr_export(struct attr *new, struct attr *orig,
 		orig_nexthop.family =
 			BGP_MP_NEXTHOP_FAMILY(orig->mp_nexthop_len);
 		if (orig_nexthop.family == AF_INET) {
-			orig_nexthop.prefixlen = 32;
+			orig_nexthop.prefixlen = IPV4_MAX_BITLEN;
 			orig_nexthop.u.prefix4 = orig->mp_nexthop_global_in;
 		} else if (orig_nexthop.family == AF_INET6) {
-			orig_nexthop.prefixlen = 128;
+			orig_nexthop.prefixlen = IPV6_MAX_BITLEN;
 			orig_nexthop.u.prefix6 = orig->mp_nexthop_global;
 		} else {
 			return -1; /* FAIL - can't compute nexthop */
@@ -639,15 +625,14 @@ encap_attr_export(struct attr *new, struct attr *orig,
 		/* TBD  use lcom for IPv6 */
 		ecom_ro = vnc_route_origin_ecom_single(&use_nexthop->u.prefix4);
 	}
-	if (new->ecommunity) {
+	if (bgp_attr_get_ecommunity(new)) {
 		if (ecom_ro)
-			new->ecommunity =
-				ecommunity_merge(ecom_ro, new->ecommunity);
+			bgp_attr_set_ecommunity(
+				new,
+				ecommunity_merge(ecom_ro,
+						 bgp_attr_get_ecommunity(new)));
 	} else {
-		new->ecommunity = ecom_ro;
-	}
-	if (ecom_ro) {
-		new->flag |= ATTR_FLAG_BIT(BGP_ATTR_EXT_COMMUNITIES);
+		bgp_attr_set_ecommunity(new, ecom_ro);
 	}
 
 	/*
@@ -730,7 +715,7 @@ void vnc_direct_bgp_add_prefix(struct bgp *bgp,
 		return;
 	}
 
-	bgp_attr_default_set(&attr, BGP_ORIGIN_INCOMPLETE);
+	bgp_attr_default_set(&attr, bgp, BGP_ORIGIN_INCOMPLETE);
 	/* TBD set some configured med, see add_vnc_route() */
 
 	vnc_zlog_debug_verbose(
@@ -876,7 +861,6 @@ void vnc_direct_bgp_del_prefix(struct bgp *bgp,
 
 			bgp_withdraw(irfd->peer, p, /* prefix */
 				     0,		    /* addpath_id */
-				     NULL,	    /* attr, ignored */
 				     afi, SAFI_UNICAST, ZEBRA_ROUTE_VNC_DIRECT,
 				     BGP_ROUTE_REDISTRIBUTE,
 				     NULL, /* RD not used for unicast */
@@ -906,7 +890,6 @@ void vnc_direct_bgp_del_prefix(struct bgp *bgp,
 
 			bgp_withdraw(irfd->peer, p, /* prefix */
 				     0,		    /* addpath_id */
-				     NULL,	    /* attr, ignored */
 				     afi, SAFI_UNICAST, ZEBRA_ROUTE_VNC_DIRECT,
 				     BGP_ROUTE_REDISTRIBUTE,
 				     NULL, /* RD not used for unicast */
@@ -971,9 +954,6 @@ void vnc_direct_bgp_add_nve(struct bgp *bgp, struct rfapi_descriptor *rfd)
 
 			import_table = rfg->rfapi_import_table;
 
-			bgp_attr_default_set(&attr, BGP_ORIGIN_INCOMPLETE);
-			/* TBD set some configured med, see add_vnc_route() */
-
 			if (afi == AFI_IP || afi == AFI_IP6) {
 				rt = import_table->imported_vpn[afi];
 			} else {
@@ -982,6 +962,9 @@ void vnc_direct_bgp_add_nve(struct bgp *bgp, struct rfapi_descriptor *rfd)
 				return;
 			}
 
+			bgp_attr_default_set(&attr, bgp, BGP_ORIGIN_INCOMPLETE);
+			/* TBD set some configured med, see add_vnc_route() */
+
 			/*
 			 * Walk the NVE-Group's VNC Import table
 			 */
@@ -1032,7 +1015,7 @@ void vnc_direct_bgp_add_nve(struct bgp *bgp, struct rfapi_descriptor *rfd)
 						ret = route_map_apply(
 							rfgn->rfg
 								->routemap_export_bgp,
-							p, RMAP_BGP, &info);
+							p, &info);
 						if (ret == RMAP_DENYMATCH) {
 							bgp_attr_flush(&hattr);
 							continue;
@@ -1143,7 +1126,6 @@ void vnc_direct_bgp_del_nve(struct bgp *bgp, struct rfapi_descriptor *rfd)
 
 					bgp_withdraw(irfd->peer, p, /* prefix */
 						     0,	   /* addpath_id */
-						     NULL, /* attr, ignored */
 						     afi, SAFI_UNICAST,
 						     ZEBRA_ROUTE_VNC_DIRECT,
 						     BGP_ROUTE_REDISTRIBUTE,
@@ -1176,19 +1158,19 @@ static void vnc_direct_add_rn_group_rd(struct bgp *bgp,
 
 		if (!rfg->rt_export_list || !rfg->rfapi_import_table) {
 			vnc_zlog_debug_verbose(
-				"%s: VRF \"%s\" is missing RT import/export configuration.\n",
+				"%s: VRF \"%s\" is missing RT import/export configuration.",
 				__func__, rfg->name);
 			return;
 		}
 		if (!rfg->rd.prefixlen) {
 			vnc_zlog_debug_verbose(
-				"%s: VRF \"%s\" is missing RD configuration.\n",
+				"%s: VRF \"%s\" is missing RD configuration.",
 				__func__, rfg->name);
 			return;
 		}
 		if (rfg->label > MPLS_LABEL_MAX) {
 			vnc_zlog_debug_verbose(
-				"%s: VRF \"%s\" is missing default label configuration.\n",
+				"%s: VRF \"%s\" is missing default label configuration.",
 				__func__, rfg->name);
 			return;
 		}
@@ -1241,8 +1223,7 @@ static void vnc_direct_add_rn_group_rd(struct bgp *bgp,
 
 		info.peer = irfd->peer;
 		info.attr = &hattr;
-		ret = route_map_apply(rfg->routemap_export_bgp, p, RMAP_BGP,
-				      &info);
+		ret = route_map_apply(rfg->routemap_export_bgp, p, &info);
 		if (ret == RMAP_DENYMATCH) {
 			bgp_attr_flush(&hattr);
 			vnc_zlog_debug_verbose(
@@ -1307,7 +1288,7 @@ static void vnc_direct_bgp_add_group_afi(struct bgp *bgp,
 		return;
 	}
 
-	bgp_attr_default_set(&attr, BGP_ORIGIN_INCOMPLETE);
+	bgp_attr_default_set(&attr, bgp, BGP_ORIGIN_INCOMPLETE);
 	/* TBD set some configured med, see add_vnc_route() */
 
 	/*
@@ -1376,7 +1357,6 @@ static void vnc_direct_del_rn_group_rd(struct bgp *bgp,
 
 	bgp_withdraw(irfd->peer, agg_node_get_prefix(rn), /* prefix */
 		     0,					  /* addpath_id */
-		     NULL,				  /* attr, ignored */
 		     afi, SAFI_UNICAST, ZEBRA_ROUTE_VNC_DIRECT,
 		     BGP_ROUTE_REDISTRIBUTE, NULL, /* RD not used for unicast */
 		     NULL, 0, NULL); /* tag not used for unicast */
@@ -1495,7 +1475,6 @@ static void vnc_direct_bgp_unexport_table(afi_t afi, struct agg_table *rt,
 					bgp_withdraw(irfd->peer,
 						     agg_node_get_prefix(rn),
 						     0,	   /* addpath_id */
-						     NULL, /* attr, ignored */
 						     afi, SAFI_UNICAST,
 						     ZEBRA_ROUTE_VNC_DIRECT,
 						     BGP_ROUTE_REDISTRIBUTE,
@@ -1690,8 +1669,7 @@ void vnc_direct_bgp_rh_add_route(struct bgp *bgp, afi_t afi,
 		memset(&info, 0, sizeof(info));
 		info.peer = peer;
 		info.attr = &hattr;
-		ret = route_map_apply(hc->routemap_export_bgp, prefix, RMAP_BGP,
-				      &info);
+		ret = route_map_apply(hc->routemap_export_bgp, prefix, &info);
 		if (ret == RMAP_DENYMATCH) {
 			bgp_attr_flush(&hattr);
 			return;
@@ -1710,14 +1688,11 @@ void vnc_direct_bgp_rh_add_route(struct bgp *bgp, afi_t afi,
 	rfapiGetVncLifetime(attr, &eti->lifetime);
 	eti->lifetime = rfapiGetHolddownFromLifetime(eti->lifetime);
 
-	if (eti->timer) {
-		/*
-		 * export expiration timer is already running on
-		 * this route: cancel it
-		 */
-		thread_cancel(eti->timer);
-		eti->timer = NULL;
-	}
+	/*
+	 * export expiration timer is already running on
+	 * this route: cancel it
+	 */
+	EVENT_OFF(eti->timer);
 
 	bgp_update(peer, prefix, /* prefix */
 		   0,		 /* addpath_id */
@@ -1729,16 +1704,15 @@ void vnc_direct_bgp_rh_add_route(struct bgp *bgp, afi_t afi,
 	bgp_attr_unintern(&iattr);
 }
 
-static int vncExportWithdrawTimer(struct thread *t)
+static void vncExportWithdrawTimer(struct event *t)
 {
-	struct vnc_export_info *eti = t->arg;
+	struct vnc_export_info *eti = EVENT_ARG(t);
 	const struct prefix *p = agg_node_get_prefix(eti->node);
 
 	/*
 	 * withdraw the route
 	 */
 	bgp_withdraw(eti->peer, p, 0, /* addpath_id */
-		     NULL,	      /* attr, ignored */
 		     family2afi(p->family), SAFI_UNICAST, eti->type,
 		     eti->subtype, NULL, /* RD not used for unicast */
 		     NULL, 0,
@@ -1748,8 +1722,6 @@ static int vncExportWithdrawTimer(struct thread *t)
 	 * Free the eti
 	 */
 	vnc_eti_delete(eti);
-
-	return 0;
 }
 
 /*
@@ -1793,8 +1765,8 @@ void vnc_direct_bgp_rh_del_route(struct bgp *bgp, afi_t afi,
 
 	if (!eti->timer && eti->lifetime <= INT32_MAX) {
 		eti->timer = NULL;
-		thread_add_timer(bm->master, vncExportWithdrawTimer, eti,
-				 eti->lifetime, &eti->timer);
+		event_add_timer(bm->master, vncExportWithdrawTimer, eti,
+				eti->lifetime, &eti->timer);
 		vnc_zlog_debug_verbose(
 			"%s: set expiration timer for %u seconds", __func__,
 			eti->lifetime);
@@ -1805,7 +1777,7 @@ void vnc_direct_bgp_rh_del_route(struct bgp *bgp, afi_t afi,
 void vnc_direct_bgp_rh_vpn_enable(struct bgp *bgp, afi_t afi)
 {
 	struct prefix_rd prd;
-	struct bgp_node *prn;
+	struct bgp_dest *pdest;
 	struct rfapi_cfg *hc;
 
 	vnc_zlog_debug_verbose("%s: entry, afi=%d", __func__, afi);
@@ -1835,45 +1807,46 @@ void vnc_direct_bgp_rh_vpn_enable(struct bgp *bgp, afi_t afi)
 	vnc_zlog_debug_verbose("%s: starting RD loop", __func__);
 
 	/* Loop over all the RDs */
-	for (prn = bgp_table_top(bgp->rib[afi][SAFI_MPLS_VPN]); prn;
-	     prn = bgp_route_next(prn)) {
+	for (pdest = bgp_table_top(bgp->rib[afi][SAFI_MPLS_VPN]); pdest;
+	     pdest = bgp_route_next(pdest)) {
 
 		struct bgp_table *table;
-		struct bgp_node *rn;
+		struct bgp_dest *dest;
 		struct bgp_path_info *ri;
-		const struct prefix *prn_p = bgp_node_get_prefix(prn);
+		const struct prefix *pdest_p = bgp_dest_get_prefix(pdest);
 
 		memset(&prd, 0, sizeof(prd));
 		prd.family = AF_UNSPEC;
 		prd.prefixlen = 64;
-		memcpy(prd.val, prn_p->u.val, 8);
+		memcpy(prd.val, pdest_p->u.val, 8);
 
 		/* This is the per-RD table of prefixes */
-		table = bgp_node_get_bgp_table_info(prn);
+		table = bgp_dest_get_bgp_table_info(pdest);
 
 		if (!table)
 			continue;
 
-		for (rn = bgp_table_top(table); rn; rn = bgp_route_next(rn)) {
-			const struct prefix *rn_p;
+		for (dest = bgp_table_top(table); dest;
+		     dest = bgp_route_next(dest)) {
+			const struct prefix *dest_p;
 
 			/*
 			 * skip prefix list check if no routes here
 			 */
-			if (!bgp_node_has_bgp_path_info_data(rn))
+			if (!bgp_dest_has_bgp_path_info_data(dest))
 				continue;
 
-			vnc_zlog_debug_verbose("%s: checking prefix %pRN",
-					       __func__, rn);
+			vnc_zlog_debug_verbose("%s: checking prefix %pBD",
+					       __func__, dest);
 
-			rn_p = bgp_node_get_prefix(rn);
+			dest_p = bgp_dest_get_prefix(dest);
 
 			/*
 			 * prefix list check
 			 */
 			if (hc->plist_export_bgp[afi]) {
 				if (prefix_list_apply(hc->plist_export_bgp[afi],
-						      rn_p)
+						      dest_p)
 				    == PREFIX_DENY) {
 
 					vnc_zlog_debug_verbose(
@@ -1883,8 +1856,8 @@ void vnc_direct_bgp_rh_vpn_enable(struct bgp *bgp, afi_t afi)
 				}
 			}
 
-			for (ri = bgp_node_get_bgp_path_info(rn);
-			     ri; ri = ri->next) {
+			for (ri = bgp_dest_get_bgp_path_info(dest); ri;
+			     ri = ri->next) {
 
 				vnc_zlog_debug_verbose("%s: ri->sub_type: %d",
 						       __func__, ri->sub_type);
@@ -1918,7 +1891,7 @@ void vnc_direct_bgp_rh_vpn_enable(struct bgp *bgp, afi_t afi)
 						info.attr = &hattr;
 						ret = route_map_apply(
 							hc->routemap_export_bgp,
-							rn_p, RMAP_BGP, &info);
+							dest_p, &info);
 						if (ret == RMAP_DENYMATCH) {
 							bgp_attr_flush(&hattr);
 							vnc_zlog_debug_verbose(
@@ -1937,40 +1910,37 @@ void vnc_direct_bgp_rh_vpn_enable(struct bgp *bgp, afi_t afi)
 					 * this route
 					 */
 					eti = vnc_eti_get(
-						bgp, EXPORT_TYPE_BGP, rn_p,
+						bgp, EXPORT_TYPE_BGP, dest_p,
 						ri->peer,
 						ZEBRA_ROUTE_VNC_DIRECT_RH,
 						BGP_ROUTE_REDISTRIBUTE);
 					rfapiGetVncLifetime(ri->attr,
 							    &eti->lifetime);
 
-					if (eti->timer) {
-						/*
-						 * export expiration timer is
-						 * already running on
-						 * this route: cancel it
-						 */
-						thread_cancel(eti->timer);
-						eti->timer = NULL;
-					}
+					/*
+					 * export expiration timer is
+					 * already running on
+					 * this route: cancel it
+					 */
+					EVENT_OFF(eti->timer);
 
 					vnc_zlog_debug_verbose(
 						"%s: calling bgp_update",
 						__func__);
 
-					bgp_update(ri->peer, rn_p, /* prefix */
-						   0,	  /* addpath_id */
-						   iattr, /* bgp_update copies
-							     it */
-						   AFI_IP, SAFI_UNICAST,
-						   ZEBRA_ROUTE_VNC_DIRECT_RH,
-						   BGP_ROUTE_REDISTRIBUTE, NULL,
-						   /* RD not used for unicast */
-						   NULL,
-						   /* tag not used for unicast,
-						      or EVPN */
-						   0, 0,
-						   NULL); /* EVPN not used */
+					bgp_update(
+						ri->peer, dest_p, /* prefix */
+						0,     /* addpath_id */
+						iattr, /* bgp_update copies
+							  it */
+						AFI_IP, SAFI_UNICAST,
+						ZEBRA_ROUTE_VNC_DIRECT_RH,
+						BGP_ROUTE_REDISTRIBUTE, NULL,
+						/* RD not used for unicast */
+						NULL,
+						/* tag not used for unicast,
+						   or EVPN */
+						0, 0, NULL); /* EVPN not used */
 
 					bgp_attr_unintern(&iattr);
 				}
@@ -1981,7 +1951,7 @@ void vnc_direct_bgp_rh_vpn_enable(struct bgp *bgp, afi_t afi)
 
 void vnc_direct_bgp_rh_vpn_disable(struct bgp *bgp, afi_t afi)
 {
-	struct bgp_node *rn;
+	struct bgp_dest *dest;
 
 	vnc_zlog_debug_verbose("%s: entry, afi=%d", __func__, afi);
 
@@ -1997,13 +1967,14 @@ void vnc_direct_bgp_rh_vpn_disable(struct bgp *bgp, afi_t afi)
 	 * Go through the entire BGP unicast table and remove routes that
 	 * originated from us
 	 */
-	for (rn = bgp_table_top(bgp->rib[afi][SAFI_UNICAST]); rn;
-	     rn = bgp_route_next(rn)) {
-		const struct prefix *rn_p = bgp_node_get_prefix(rn);
+	for (dest = bgp_table_top(bgp->rib[afi][SAFI_UNICAST]); dest;
+	     dest = bgp_route_next(dest)) {
+		const struct prefix *dest_p = bgp_dest_get_prefix(dest);
 		struct bgp_path_info *ri;
 		struct bgp_path_info *next;
 
-		for (ri = bgp_node_get_bgp_path_info(rn), next = NULL; ri; ri = next) {
+		for (ri = bgp_dest_get_bgp_path_info(dest), next = NULL; ri;
+		     ri = next) {
 
 			next = ri->next;
 
@@ -2016,18 +1987,16 @@ void vnc_direct_bgp_rh_vpn_disable(struct bgp *bgp, afi_t afi)
 				 * Delete routes immediately (no timer)
 				 */
 				eti = vnc_eti_checktimer(
-					bgp, EXPORT_TYPE_BGP, rn_p, ri->peer,
+					bgp, EXPORT_TYPE_BGP, dest_p, ri->peer,
 					ZEBRA_ROUTE_VNC_DIRECT_RH,
 					BGP_ROUTE_REDISTRIBUTE);
 				if (eti) {
-					if (eti->timer)
-						thread_cancel(eti->timer);
+					EVENT_OFF(eti->timer);
 					vnc_eti_delete(eti);
 				}
 
-				bgp_withdraw(ri->peer, rn_p, /* prefix */
-					     0,		     /* addpath_id */
-					     NULL,	     /* ignored */
+				bgp_withdraw(ri->peer, dest_p, /* prefix */
+					     0,		       /* addpath_id */
 					     AFI_IP, SAFI_UNICAST,
 					     ZEBRA_ROUTE_VNC_DIRECT_RH,
 					     BGP_ROUTE_REDISTRIBUTE,
