@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * IS-IS Rout(e)ing protocol - isis_adjacency.c
  *                             handling of IS-IS adjacencies
@@ -5,20 +6,6 @@
  * Copyright (C) 2001,2002   Sampo Saaristo
  *                           Tampere University of Technology
  *                           Institute of Communications Engineering
- *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public Licenseas published by the Free
- * Software Foundation; either version 2 of the License, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful,but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -28,7 +15,7 @@
 #include "hash.h"
 #include "vty.h"
 #include "linklist.h"
-#include "thread.h"
+#include "frrevent.h"
 #include "if.h"
 #include "stream.h"
 #include "bfd.h"
@@ -103,6 +90,7 @@ struct isis_adjacency *isis_new_adj(const uint8_t *id, const uint8_t *snpa,
 		}
 	}
 	adj->adj_sids = list_new();
+	adj->srv6_endx_sids = list_new();
 	listnode_add(circuit->area->adjacency_list, adj);
 
 	return adj;
@@ -161,7 +149,7 @@ void isis_delete_adj(void *arg)
 	/* Remove self from snmp list without walking the list*/
 	list_delete_node(adj->circuit->snmp_adj_list, adj->snmp_list_node);
 
-	THREAD_OFF(adj->t_expire);
+	EVENT_OFF(adj->t_expire);
 	if (adj->adj_state != ISIS_ADJ_DOWN)
 		adj->adj_state = ISIS_ADJ_DOWN;
 
@@ -173,6 +161,7 @@ void isis_delete_adj(void *arg)
 	XFREE(MTYPE_ISIS_ADJACENCY_INFO, adj->global_ipv6_addrs);
 	adj_mt_finish(adj);
 	list_delete(&adj->adj_sids);
+	list_delete(&adj->srv6_endx_sids);
 
 	listnode_delete(adj->circuit->area->adjacency_list, adj);
 	XFREE(MTYPE_ISIS_ADJACENCY, adj);
@@ -242,11 +231,12 @@ static void isis_adj_route_switchover(struct isis_adjacency *adj)
 	}
 }
 
-void isis_adj_process_threeway(struct isis_adjacency *adj,
+void isis_adj_process_threeway(struct isis_adjacency **padj,
 			       struct isis_threeway_adj *tw_adj,
 			       enum isis_adj_usage adj_usage)
 {
 	enum isis_threeway_state next_tw_state = ISIS_THREEWAY_DOWN;
+	struct isis_adjacency *adj = *padj;
 
 	if (tw_adj && !adj->circuit->disable_threeway_adj) {
 		if (tw_adj->state == ISIS_THREEWAY_DOWN) {
@@ -276,14 +266,13 @@ void isis_adj_process_threeway(struct isis_adjacency *adj,
 		fabricd_initial_sync_hello(adj->circuit);
 
 	if (next_tw_state == ISIS_THREEWAY_DOWN) {
-		isis_adj_state_change(&adj, ISIS_ADJ_DOWN,
-				      "Neighbor restarted");
+		isis_adj_state_change(padj, ISIS_ADJ_DOWN, "Neighbor restarted");
 		return;
 	}
 
 	if (next_tw_state == ISIS_THREEWAY_UP) {
 		if (adj->adj_state != ISIS_ADJ_UP) {
-			isis_adj_state_change(&adj, ISIS_ADJ_UP, NULL);
+			isis_adj_state_change(padj, ISIS_ADJ_UP, NULL);
 			adj->adj_usage = adj_usage;
 		}
 	}
@@ -296,6 +285,8 @@ void isis_adj_process_threeway(struct isis_adjacency *adj,
 }
 const char *isis_adj_name(const struct isis_adjacency *adj)
 {
+	static char buf[ISO_SYSID_STRLEN];
+
 	if (!adj)
 		return "NONE";
 
@@ -304,8 +295,9 @@ const char *isis_adj_name(const struct isis_adjacency *adj)
 	dyn = dynhn_find_by_id(adj->circuit->isis, adj->sysid);
 	if (dyn)
 		return dyn->hostname;
-	else
-		return sysid_print(adj->sysid);
+
+	snprintfrr(buf, sizeof(buf), "%pSY", adj->sysid);
+	return buf;
 }
 void isis_log_adj_change(struct isis_adjacency *adj,
 			 enum isis_adj_state old_state,
@@ -366,12 +358,15 @@ void isis_adj_state_change(struct isis_adjacency **padj,
 				 * purposes */
 				adj->last_flap = time(NULL);
 				adj->flaps++;
-			} else if (old_state == ISIS_ADJ_UP) {
-				circuit->adj_state_changes++;
+			} else {
+				if (old_state == ISIS_ADJ_UP) {
+					circuit->adj_state_changes++;
 
-				circuit->upadjcount[level - 1]--;
-				if (circuit->upadjcount[level - 1] == 0)
-					isis_tx_queue_clean(circuit->tx_queue);
+					circuit->upadjcount[level - 1]--;
+					if (circuit->upadjcount[level - 1] == 0)
+						isis_tx_queue_clean(
+							circuit->tx_queue);
+				}
 
 				if (new_state == ISIS_ADJ_DOWN) {
 					listnode_delete(
@@ -409,18 +404,21 @@ void isis_adj_state_change(struct isis_adjacency **padj,
 				adj->flaps++;
 
 				if (level == IS_LEVEL_1) {
-					thread_add_timer(master, send_l1_csnp,
-							 circuit, 0,
-							 &circuit->t_send_csnp[0]);
+					event_add_timer(
+						master, send_l1_csnp, circuit,
+						0, &circuit->t_send_csnp[0]);
 				} else {
-					thread_add_timer(master, send_l2_csnp,
-							 circuit, 0,
-							 &circuit->t_send_csnp[1]);
+					event_add_timer(
+						master, send_l2_csnp, circuit,
+						0, &circuit->t_send_csnp[1]);
+				}
+			} else {
+				if (old_state == ISIS_ADJ_UP) {
+					circuit->upadjcount[level - 1]--;
+					if (circuit->upadjcount[level - 1] == 0)
+						isis_tx_queue_clean(
+							circuit->tx_queue);
 				}
-			} else if (old_state == ISIS_ADJ_UP) {
-				circuit->upadjcount[level - 1]--;
-				if (circuit->upadjcount[level - 1] == 0)
-					isis_tx_queue_clean(circuit->tx_queue);
 
 				if (new_state == ISIS_ADJ_DOWN) {
 					if (adj->circuit->u.p2p.neighbor == adj)
@@ -452,9 +450,8 @@ void isis_adj_print(struct isis_adjacency *adj)
 	if (dyn)
 		zlog_debug("%s", dyn->hostname);
 
-	zlog_debug("SystemId %20s SNPA %s, level %d; Holding Time %d",
-		   sysid_print(adj->sysid), snpa_print(adj->snpa), adj->level,
-		   adj->hold_time);
+	zlog_debug("SystemId %20pSY SNPA %pSY, level %d; Holding Time %d",
+		   adj->sysid, adj->snpa, adj->level, adj->hold_time);
 	if (adj->ipv4_address_count) {
 		zlog_debug("IPv4 Address(es):");
 		for (unsigned int i = 0; i < adj->ipv4_address_count; i++)
@@ -484,19 +481,21 @@ const char *isis_adj_yang_state(enum isis_adj_state state)
 		return "up";
 	case ISIS_ADJ_INITIALIZING:
 		return "init";
-	default:
+	case ISIS_ADJ_UNKNOWN:
 		return "failed";
 	}
+
+	assert(!"Reached end of function where we are not expecting to");
 }
 
-void isis_adj_expire(struct thread *thread)
+void isis_adj_expire(struct event *thread)
 {
 	struct isis_adjacency *adj;
 
 	/*
 	 * Get the adjacency
 	 */
-	adj = THREAD_ARG(thread);
+	adj = EVENT_ARG(thread);
 	assert(adj);
 	adj->t_expire = NULL;
 
@@ -541,7 +540,7 @@ void isis_adj_print_json(struct isis_adjacency *adj, struct json_object *json,
 					time2string(adj->last_upd +
 						    adj->hold_time - now));
 		}
-		json_object_string_add(json, "snpa", snpa_print(adj->snpa));
+		json_object_string_addf(json, "snpa", "%pSY", adj->snpa);
 	}
 
 	if (detail == ISIS_UI_LEVEL_DETAIL) {
@@ -592,8 +591,7 @@ void isis_adj_print_json(struct isis_adjacency *adj, struct json_object *json,
 					isis_mtid2str(adj->mt_set[i]));
 			}
 		}
-		json_object_string_add(iface_json, "snpa",
-				       snpa_print(adj->snpa));
+		json_object_string_addf(iface_json, "snpa", "%pSY", adj->snpa);
 		if (adj->circuit &&
 		    (adj->circuit->circ_type == CIRCUIT_T_BROADCAST)) {
 			dyn = dynhn_find_by_id(adj->circuit->isis, adj->lanid);
@@ -604,11 +602,8 @@ void isis_adj_print_json(struct isis_adjacency *adj, struct json_object *json,
 				json_object_string_add(iface_json, "lan-id",
 						       buf);
 			} else {
-				snprintfrr(buf, sizeof(buf), "%s-%02x",
-					   sysid_print(adj->lanid),
-					   adj->lanid[ISIS_SYS_ID_LEN]);
-				json_object_string_add(iface_json, "lan-id",
-						       buf);
+				json_object_string_addf(iface_json, "lan-id",
+							"%pSY", adj->lanid);
 			}
 
 			json_object_int_add(iface_json, "lan-prio",
@@ -637,12 +632,9 @@ void isis_adj_print_json(struct isis_adjacency *adj, struct json_object *json,
 					       area_addr_json);
 			for (unsigned int i = 0; i < adj->area_address_count;
 			     i++) {
-				json_object_string_add(
-					area_addr_json, "isonet",
-					isonet_print(adj->area_addresses[i]
-							     .area_addr,
-						     adj->area_addresses[i]
-							     .addr_len));
+				json_object_string_addf(
+					area_addr_json, "isonet", "%pIS",
+					&adj->area_addresses[i]);
 			}
 		}
 		if (adj->ipv4_address_count) {
@@ -701,7 +693,7 @@ void isis_adj_print_json(struct isis_adjacency *adj, struct json_object *json,
 			default:
 				continue;
 			}
-			backup = (sra->type == ISIS_SR_LAN_BACKUP) ? " (backup)"
+			backup = (sra->type == ISIS_SR_ADJ_BACKUP) ? " (backup)"
 								   : "";
 
 			json_object_string_add(adj_sid_json, "nexthop",
@@ -740,14 +732,14 @@ void isis_adj_print_vty(struct isis_adjacency *adj, struct vty *vty,
 		now = time(NULL);
 		if (adj->last_upd) {
 			if (adj->last_upd + adj->hold_time < now)
-				vty_out(vty, " Expiring");
+				vty_out(vty, " Expiring ");
 			else
 				vty_out(vty, " %-9llu",
 					(unsigned long long)adj->last_upd
 						+ adj->hold_time - now);
 		} else
-			vty_out(vty, "-        ");
-		vty_out(vty, "%-10s", snpa_print(adj->snpa));
+			vty_out(vty, " -        ");
+		vty_out(vty, "%-10pSY", adj->snpa);
 		vty_out(vty, "\n");
 	}
 
@@ -791,7 +783,7 @@ void isis_adj_print_vty(struct isis_adjacency *adj, struct vty *vty,
 				vty_out(vty, "      %s\n",
 					isis_mtid2str(adj->mt_set[i]));
 		}
-		vty_out(vty, "    SNPA: %s", snpa_print(adj->snpa));
+		vty_out(vty, "    SNPA: %pSY", adj->snpa);
 		if (adj->circuit
 		    && (adj->circuit->circ_type == CIRCUIT_T_BROADCAST)) {
 			dyn = dynhn_find_by_id(adj->circuit->isis, adj->lanid);
@@ -799,9 +791,7 @@ void isis_adj_print_vty(struct isis_adjacency *adj, struct vty *vty,
 				vty_out(vty, ", LAN id: %s.%02x", dyn->hostname,
 					adj->lanid[ISIS_SYS_ID_LEN]);
 			else
-				vty_out(vty, ", LAN id: %s.%02x",
-					sysid_print(adj->lanid),
-					adj->lanid[ISIS_SYS_ID_LEN]);
+				vty_out(vty, ", LAN id: %pPN", adj->lanid);
 
 			vty_out(vty, "\n");
 			vty_out(vty, "    LAN Priority: %u",
@@ -822,11 +812,8 @@ void isis_adj_print_vty(struct isis_adjacency *adj, struct vty *vty,
 			vty_out(vty, "    Area Address(es):\n");
 			for (unsigned int i = 0; i < adj->area_address_count;
 			     i++) {
-				vty_out(vty, "      %s\n",
-					isonet_print(adj->area_addresses[i]
-							     .area_addr,
-						     adj->area_addresses[i]
-							     .addr_len));
+				vty_out(vty, "      %pIS\n",
+					&adj->area_addresses[i]);
 			}
 		}
 		if (adj->ipv4_address_count) {
@@ -881,7 +868,7 @@ void isis_adj_print_vty(struct isis_adjacency *adj, struct vty *vty,
 			default:
 				continue;
 			}
-			backup = (sra->type == ISIS_SR_LAN_BACKUP) ? " (backup)"
+			backup = (sra->type == ISIS_SR_ADJ_BACKUP) ? " (backup)"
 								   : "";
 
 			vty_out(vty, "    %s %s%s: %u\n",
@@ -954,8 +941,9 @@ int isis_adj_usage2levels(enum isis_adj_usage usage)
 		return IS_LEVEL_2;
 	case ISIS_ADJ_LEVEL1AND2:
 		return IS_LEVEL_1 | IS_LEVEL_2;
-	default:
-		break;
+	case ISIS_ADJ_NONE:
+		return 0;
 	}
-	return 0;
+
+	assert(!"Reached end of function where we are not expecting to");
 }
