@@ -489,7 +489,6 @@ static uint8_t isis_circuit_id_gen(struct isis *isis, struct interface *ifp)
 
 void isis_circuit_if_add(struct isis_circuit *circuit, struct interface *ifp)
 {
-	struct listnode *node, *nnode;
 	struct connected *conn;
 
 	if (if_is_broadcast(ifp)) {
@@ -509,20 +508,18 @@ void isis_circuit_if_add(struct isis_circuit *circuit, struct interface *ifp)
 		circuit->circ_type = CIRCUIT_T_UNKNOWN;
 	}
 
-	for (ALL_LIST_ELEMENTS(ifp->connected, node, nnode, conn))
+	frr_each (if_connected, ifp->connected, conn)
 		isis_circuit_add_addr(circuit, conn);
-
 }
 
 void isis_circuit_if_del(struct isis_circuit *circuit, struct interface *ifp)
 {
-	struct listnode *node, *nnode;
 	struct connected *conn;
 
 	assert(circuit->interface == ifp);
 
 	/* destroy addresses */
-	for (ALL_LIST_ELEMENTS(ifp->connected, node, nnode, conn))
+	frr_each_safe (if_connected, ifp->connected, conn)
 		isis_circuit_del_addr(circuit, conn);
 
 	circuit->circ_type = CIRCUIT_T_UNKNOWN;
@@ -1009,45 +1006,40 @@ void isis_circuit_print_json(struct isis_circuit *circuit,
 					       circuit_t2string(level));
 			if (circuit->area->newmetric)
 				json_object_int_add(level_json, "metric",
-						    circuit->te_metric[0]);
+						    circuit->te_metric[level - 1]);
 			else
 				json_object_int_add(level_json, "metric",
-						    circuit->metric[0]);
+						    circuit->metric[level - 1]);
 			if (!circuit->is_passive) {
-				json_object_int_add(level_json,
-						    "active-neighbors",
-						    circuit->upadjcount[0]);
-				json_object_int_add(level_json,
-						    "hello-interval",
-						    circuit->hello_interval[0]);
+				json_object_int_add(level_json, "active-neighbors",
+						    circuit->upadjcount[level - 1]);
+				json_object_int_add(level_json, "hello-interval",
+						    circuit->hello_interval[level - 1]);
 				hold_json = json_object_new_object();
 				json_object_object_add(level_json, "holddown",
 						       hold_json);
-				json_object_int_add(
-					hold_json, "count",
-					circuit->hello_multiplier[0]);
+				json_object_int_add(hold_json, "count",
+						    circuit->hello_multiplier[level - 1]);
 				json_object_string_add(
 					hold_json, "pad",
 					isis_hello_padding2string(
 						circuit->pad_hellos));
 				json_object_int_add(level_json, "cnsp-interval",
-						    circuit->csnp_interval[0]);
+						    circuit->csnp_interval[level - 1]);
 				json_object_int_add(level_json, "psnp-interval",
-						    circuit->psnp_interval[0]);
+						    circuit->psnp_interval[level - 1]);
 				if (circuit->circ_type == CIRCUIT_T_BROADCAST) {
 					lan_prio_json =
 						json_object_new_object();
 					json_object_object_add(level_json,
 							       "lan",
 							       lan_prio_json);
-					json_object_int_add(
-						lan_prio_json, "priority",
-						circuit->priority[0]);
-					json_object_string_add(
-						lan_prio_json, "is-dis",
-						(circuit->u.bc.is_dr[0]
-							 ? "yes"
-							 : "no"));
+					json_object_int_add(lan_prio_json, "priority",
+							    circuit->priority[level - 1]);
+					json_object_string_add(lan_prio_json, "is-dis",
+							       (circuit->u.bc.is_dr[level - 1]
+									? "yes"
+									: "no"));
 				}
 			}
 			json_object_array_add(levels_json, level_json);
@@ -1680,6 +1672,8 @@ void isis_circuit_init(void)
 #else
 	if_cmd_init_default();
 #endif
-	if_zapi_callbacks(isis_ifp_create, isis_ifp_up,
-			  isis_ifp_down, isis_ifp_destroy);
+	hook_register_prio(if_real, 0, isis_ifp_create);
+	hook_register_prio(if_up, 0, isis_ifp_up);
+	hook_register_prio(if_down, 0, isis_ifp_down);
+	hook_register_prio(if_unreal, 0, isis_ifp_destroy);
 }
