@@ -1,20 +1,7 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * Copyright (C) 2018        Volta Networks
  *                           Emanuele Di Pascale
- *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the Free
- * Software Foundation; either version 2 of the License, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -145,8 +132,11 @@ lib_interface_state_isis_adjacencies_adjacency_neighbor_sysid_get_elem(
 	struct nb_cb_get_elem_args *args)
 {
 	const struct isis_adjacency *adj = args->list_entry;
+	char xpath_value[ISO_SYSID_STRLEN];
 
-	return yang_data_new_string(args->xpath, sysid_print(adj->sysid));
+	snprintfrr(xpath_value, ISO_SYSID_STRLEN, "%pSY", adj->sysid);
+
+	return yang_data_new_string(args->xpath, xpath_value);
 }
 
 /*
@@ -171,8 +161,11 @@ lib_interface_state_isis_adjacencies_adjacency_neighbor_snpa_get_elem(
 	struct nb_cb_get_elem_args *args)
 {
 	const struct isis_adjacency *adj = args->list_entry;
+	char xpath_value[ISO_SYSID_STRLEN];
 
-	return yang_data_new_string(args->xpath, snpa_print(adj->snpa));
+	snprintfrr(xpath_value, ISO_SYSID_STRLEN, "%pSY", adj->snpa);
+
+	return yang_data_new_string(args->xpath, xpath_value);
 }
 
 /*
