@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * IS-IS Rout(e)ing protocol - isis_spf.h
  *                             IS-IS Shortest Path First algorithm
@@ -5,26 +6,13 @@
  * Copyright (C) 2001,2002   Sampo Saaristo
  *                           Tampere University of Technology
  *                           Institute of Communications Engineering
- *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public Licenseas published by the Free
- * Software Foundation; either version 2 of the License, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful,but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #ifndef _ZEBRA_ISIS_SPF_H
 #define _ZEBRA_ISIS_SPF_H
 
 #include "isisd/isis_lfa.h"
+#include "lib/json.h"
 
 struct isis_spftree;
 
@@ -50,16 +38,15 @@ struct isis_spf_adj {
 #define F_ISIS_SPF_ADJ_METRIC_INFINITY 0x04
 };
 
-struct isis_spftree *isis_spftree_new(struct isis_area *area,
-				      struct lspdb_head *lspdb,
-				      const uint8_t *sysid, int level,
-				      enum spf_tree_id tree_id,
-				      enum spf_type type, uint8_t flags);
+struct isis_spftree *
+isis_spftree_new(struct isis_area *area, struct lspdb_head *lspdb,
+		 const uint8_t *sysid, int level, enum spf_tree_id tree_id,
+		 enum spf_type type, uint8_t flags, uint8_t algorithm);
 struct isis_vertex *isis_spf_prefix_sid_lookup(struct isis_spftree *spftree,
 					       struct isis_prefix_sid *psid);
 void isis_spf_invalidate_routes(struct isis_spftree *tree);
-void isis_spf_verify_routes(struct isis_area *area,
-			    struct isis_spftree **trees);
+void isis_spf_verify_routes(struct isis_area *area, struct isis_spftree **trees,
+			    int tree);
 void isis_spf_switchover_routes(struct isis_area *area,
 				struct isis_spftree **trees, int family,
 				union g_addr *nexthop_ip, ifindex_t ifindex,
@@ -76,7 +63,7 @@ int _isis_spf_schedule(struct isis_area *area, int level,
 		       const char *func, const char *file, int line);
 void isis_print_spftree(struct vty *vty, struct isis_spftree *spftree);
 void isis_print_routes(struct vty *vty, struct isis_spftree *spftree,
-		       bool prefix_sid, bool backup);
+		       json_object **json, bool prefix_sid, bool backup);
 void isis_spf_init(void);
 void isis_spf_print(struct isis_spftree *spftree, struct vty *vty);
 void isis_spf_print_json(struct isis_spftree *spftree,
