@@ -454,7 +454,7 @@ void isis_zebra_send_adjacency_sid(int cmd, const struct sr_adjacency *sra)
 	znh->labels[0] = MPLS_LABEL_IMPLICIT_NULL;
 
 	/* Set backup nexthops. */
-	if (sra->type == ISIS_SR_LAN_BACKUP) {
+	if (sra->type == ISIS_SR_ADJ_BACKUP) {
 		int count;
 
 		count = isis_zebra_add_nexthops(isis, sra->backup_nexthops,
@@ -1395,9 +1395,7 @@ void isis_zebra_init(struct event_loop *master, int instance)
 	zclient->zebra_connected = isis_zebra_connected;
 
 	/* Initialize special zclient for synchronous message exchanges. */
-	struct zclient_options options = zclient_options_default;
-	options.synchronous = true;
-	zclient_sync = zclient_new(master, &options, NULL, 0);
+	zclient_sync = zclient_new(master, &zclient_options_sync, NULL, 0);
 	zclient_sync->sock = -1;
 	zclient_sync->redist_default = ZEBRA_ROUTE_ISIS;
 	zclient_sync->instance = instance;
