@@ -4,12 +4,14 @@
  */
 
 #include <zebra.h>
+#include <fcntl.h>
 
 #ifdef SNMP_AGENTX
 #include <net-snmp/net-snmp-config.h>
 #include <net-snmp/net-snmp-includes.h>
 #include <net-snmp/agent/net-snmp-agent-includes.h>
 #include <net-snmp/agent/snmp_vars.h>
+#include <net-snmp/library/large_fd_set.h>
 
 #include "command.h"
 #include "smux.h"
@@ -20,12 +22,13 @@
 #include "hook.h"
 #include "libfrr.h"
 #include "xref.h"
+#include "lib/libagentx.h"
 
 XREF_SETUP();
 
 DEFINE_HOOK(agentx_enabled, (), ());
 
-static bool agentx_enabled = false;
+//bool agentx_enabled = false;
 
 static struct event_loop *agentx_tm;
 static struct event *timeout_thr = NULL;
@@ -43,7 +46,7 @@ static void agentx_timeout(struct event *t)
 
 static void agentx_read(struct event *t)
 {
-	fd_set fds;
+	netsnmp_large_fd_set lfds;
 	int flags, new_flags = 0;
 	int nonblock = false;
 	struct listnode *ln = EVENT_ARG(t);
@@ -68,9 +71,9 @@ static void agentx_read(struct event *t)
 		flog_err(EC_LIB_SYSTEM_CALL, "Failed to set snmp fd non blocking: %s(%d)",
 			 strerror(errno), errno);
 
-	FD_ZERO(&fds);
-	FD_SET(EVENT_FD(t), &fds);
-	snmp_read(&fds);
+	netsnmp_large_fd_set_init(&lfds, FD_SETSIZE);
+	netsnmp_large_fd_setfd(t->u.fd, &lfds);
+	snmp_read2(&lfds);
 
 	/* Reset the flag */
 	if (!nonblock) {
@@ -85,6 +88,7 @@ static void agentx_read(struct event *t)
 
 	netsnmp_check_outstanding_agent_requests();
 	agentx_events_update();
+	netsnmp_large_fd_set_cleanup(&lfds);
 }
 
 static void agentx_events_update(void)
@@ -92,15 +96,15 @@ static void agentx_events_update(void)
 	int maxfd = 0;
 	int block = 1;
 	struct timeval timeout = {.tv_sec = 0, .tv_usec = 0};
-	fd_set fds;
+	netsnmp_large_fd_set lfds;
 	struct listnode *ln;
 	struct event **thr;
 	int fd, thr_fd;
 
 	event_cancel(&timeout_thr);
 
-	FD_ZERO(&fds);
-	snmp_select_info(&maxfd, &fds, &timeout, &block);
+	netsnmp_large_fd_set_init(&lfds, FD_SETSIZE);
+	snmp_select_info2(&maxfd, &lfds, &timeout, &block);
 
 	if (!block) {
 		event_add_timer_tv(agentx_tm, agentx_timeout, NULL, &timeout,
@@ -118,7 +122,7 @@ static void agentx_events_update(void)
 		/* caught up */
 		if (thr_fd == fd) {
 			struct listnode *nextln = listnextnode(ln);
-			if (!FD_ISSET(fd, &fds)) {
+			if (!netsnmp_large_fd_is_set(fd, &lfds)) {
 				event_cancel(thr);
 				XFREE(MTYPE_TMP, thr);
 				list_delete_node(events, ln);
@@ -128,7 +132,7 @@ static void agentx_events_update(void)
 			thr_fd = thr ? EVENT_FD(*thr) : -1;
 		}
 		/* need listener, but haven't hit one where it would be */
-		else if (FD_ISSET(fd, &fds)) {
+		else if (netsnmp_large_fd_is_set(fd, &lfds)) {
 			struct listnode *newln;
 
 			thr = XCALLOC(MTYPE_TMP, sizeof(struct event *));
@@ -147,17 +151,9 @@ static void agentx_events_update(void)
 		list_delete_node(events, ln);
 		ln = nextln;
 	}
+	netsnmp_large_fd_set_cleanup(&lfds);
 }
 
-/* AgentX node. */
-static int config_write_agentx(struct vty *vty);
-static struct cmd_node agentx_node = {
-	.name = "smux",
-	.node = SMUX_NODE,
-	.prompt = "",
-	.config_write = config_write_agentx,
-};
-
 /* Logging NetSNMP messages */
 static int agentx_log_callback(int major, int minor, void *serverarg,
 			       void *clientarg)
@@ -197,17 +193,7 @@ static int agentx_log_callback(int major, int minor, void *serverarg,
 	return SNMP_ERR_NOERROR;
 }
 
-static int config_write_agentx(struct vty *vty)
-{
-	if (agentx_enabled)
-		vty_out(vty, "agentx\n");
-	return 1;
-}
-
-DEFUN (agentx_enable,
-       agentx_enable_cmd,
-       "agentx",
-       "SNMP AgentX protocol settings\n")
+static int agentx_cli_on(void)
 {
 	if (!agentx_enabled) {
 		init_snmp(FRR_SMUX_NAME);
@@ -217,19 +203,14 @@ DEFUN (agentx_enable,
 		hook_call(agentx_enabled);
 	}
 
-	return CMD_SUCCESS;
+	return 1;
 }
 
-DEFUN (no_agentx,
-       no_agentx_cmd,
-       "no agentx",
-       NO_STR
-       "SNMP AgentX protocol settings\n")
+static int agentx_cli_off(void)
 {
 	if (!agentx_enabled)
-		return CMD_SUCCESS;
-	vty_out(vty, "SNMP AgentX support cannot be disabled once enabled\n");
-	return CMD_WARNING_CONFIG_FAILED;
+		return 1;
+	return 0;
 }
 
 static int smux_disable(void)
@@ -248,6 +229,9 @@ void smux_init(struct event_loop *tm)
 {
 	agentx_tm = tm;
 
+	hook_register(agentx_cli_enabled, agentx_cli_on);
+	hook_register(agentx_cli_disabled, agentx_cli_off);
+
 	netsnmp_enable_subagent();
 	snmp_disable_log();
 	snmp_enable_calllog();
@@ -255,10 +239,6 @@ void smux_init(struct event_loop *tm)
 			       agentx_log_callback, NULL);
 	init_agent(FRR_SMUX_NAME);
 
-	install_node(&agentx_node);
-	install_element(CONFIG_NODE, &agentx_enable_cmd);
-	install_element(CONFIG_NODE, &no_agentx_cmd);
-
 	hook_register(frr_early_fini, smux_disable);
 }
 
