@@ -10,6 +10,10 @@
  */
 
 #include <zebra.h>
+#include <sys/utsname.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+
 #include <lib/version.h>
 
 #include "command.h"
@@ -39,6 +43,8 @@
 
 #include "frrscript.h"
 
+#include "lib/config_paths.h"
+
 DEFINE_MTYPE_STATIC(LIB, HOST, "Host config");
 DEFINE_MTYPE(LIB, COMPLETION, "Completion item");
 
@@ -672,6 +678,21 @@ vector cmd_describe_command(vector vline, struct vty *vty, int *status)
 
 static struct list *varhandlers = NULL;
 
+static int __add_key_comp(const struct lyd_node *dnode, void *arg)
+{
+	const char *key_value = yang_dnode_get_string(dnode, NULL);
+
+	vector_set((vector)arg, XSTRDUP(MTYPE_COMPLETION, key_value));
+
+	return YANG_ITER_CONTINUE;
+}
+
+static void __get_list_keys(vector comps, const char *xpath)
+{
+	yang_dnode_iterate(__add_key_comp, comps,
+			   vty_shared_candidate_config->dnode, "%s", xpath);
+}
+
 void cmd_variable_complete(struct cmd_token *token, const char *arg,
 			   vector comps)
 {
@@ -688,7 +709,10 @@ void cmd_variable_complete(struct cmd_token *token, const char *arg,
 		if (cvh->varname && (!token->varname
 				     || strcmp(cvh->varname, token->varname)))
 			continue;
-		cvh->completions(tmpcomps, token);
+		if (cvh->xpath)
+			__get_list_keys(tmpcomps, cvh->xpath);
+		if (cvh->completions)
+			cvh->completions(tmpcomps, token);
 		break;
 	}
 
@@ -747,7 +771,7 @@ void cmd_variable_handler_register(const struct cmd_variable_handler *cvh)
 	if (!varhandlers)
 		return;
 
-	for (; cvh->completions; cvh++)
+	for (; cvh->completions || cvh->xpath; cvh++)
 		listnode_add(varhandlers, (void *)cvh);
 }
 
@@ -1350,7 +1374,7 @@ DEFUN (disable,
 }
 
 /* Down vty node level. */
-DEFUN (config_exit,
+DEFUN_YANG (config_exit,
        config_exit_cmd,
        "exit",
        "Exit current mode and down to previous mode\n")
@@ -1629,6 +1653,10 @@ static int vty_write_config(struct vty *vty)
 	return CMD_SUCCESS;
 }
 
+/* cross-reference frr_daemon_state_save in libfrr.c
+ * the code there is similar but not identical (state files always use the same
+ * name for the new write, and don't keep a backup of previous state.)
+ */
 static int file_write_config(struct vty *vty)
 {
 	int fd, dirfd;
