@@ -8,6 +8,10 @@
 
 #include <zebra.h>
 
+#ifdef HAVE_GLIBC_BACKTRACE
+#include <execinfo.h>
+#endif /* HAVE_GLIBC_BACKTRACE */
+
 #include "zclient.h"
 #include "log.h"
 #include "memory.h"
@@ -440,11 +444,11 @@ static const struct zebra_desc_table command_types[] = {
 	DESC_ENTRY(ZEBRA_NEIGH_DISCOVER),
 	DESC_ENTRY(ZEBRA_ROUTE_NOTIFY_REQUEST),
 	DESC_ENTRY(ZEBRA_CLIENT_CLOSE_NOTIFY),
-	DESC_ENTRY(ZEBRA_NHRP_NEIGH_ADDED),
-	DESC_ENTRY(ZEBRA_NHRP_NEIGH_REMOVED),
-	DESC_ENTRY(ZEBRA_NHRP_NEIGH_GET),
-	DESC_ENTRY(ZEBRA_NHRP_NEIGH_REGISTER),
-	DESC_ENTRY(ZEBRA_NHRP_NEIGH_UNREGISTER),
+	DESC_ENTRY(ZEBRA_NEIGH_ADDED),
+	DESC_ENTRY(ZEBRA_NEIGH_REMOVED),
+	DESC_ENTRY(ZEBRA_NEIGH_GET),
+	DESC_ENTRY(ZEBRA_NEIGH_REGISTER),
+	DESC_ENTRY(ZEBRA_NEIGH_UNREGISTER),
 	DESC_ENTRY(ZEBRA_NEIGH_IP_ADD),
 	DESC_ENTRY(ZEBRA_NEIGH_IP_DEL),
 	DESC_ENTRY(ZEBRA_CONFIGURE_ARP),
@@ -547,6 +551,8 @@ int proto_redistnum(int afi, const char *s)
 			return ZEBRA_ROUTE_KERNEL;
 		else if (strmatch(s, "connected"))
 			return ZEBRA_ROUTE_CONNECT;
+		else if (strmatch(s, "local"))
+			return ZEBRA_ROUTE_LOCAL;
 		else if (strmatch(s, "static"))
 			return ZEBRA_ROUTE_STATIC;
 		else if (strmatch(s, "rip"))
@@ -573,12 +579,16 @@ int proto_redistnum(int afi, const char *s)
 			return ZEBRA_ROUTE_SHARP;
 		else if (strmatch(s, "openfabric"))
 			return ZEBRA_ROUTE_OPENFABRIC;
+		else if (strmatch(s, "table-direct"))
+			return ZEBRA_ROUTE_TABLE_DIRECT;
 	}
 	if (afi == AFI_IP6) {
 		if (strmatch(s, "kernel"))
 			return ZEBRA_ROUTE_KERNEL;
 		else if (strmatch(s, "connected"))
 			return ZEBRA_ROUTE_CONNECT;
+		else if (strmatch(s, "local"))
+			return ZEBRA_ROUTE_LOCAL;
 		else if (strmatch(s, "static"))
 			return ZEBRA_ROUTE_STATIC;
 		else if (strmatch(s, "ripng"))
@@ -603,6 +613,8 @@ int proto_redistnum(int afi, const char *s)
 			return ZEBRA_ROUTE_SHARP;
 		else if (strmatch(s, "openfabric"))
 			return ZEBRA_ROUTE_OPENFABRIC;
+		else if (strmatch(s, "table-direct"))
+			return ZEBRA_ROUTE_TABLE_DIRECT;
 	}
 	return -1;
 }
