@@ -19,8 +19,6 @@
 #include <sysrepo/values.h>
 #include <sysrepo/xpath.h>
 
-DEFINE_MTYPE_STATIC(LIB, SYSREPO, "Sysrepo module");
-
 static struct debug nb_dbg_client_sysrepo = {0, "Northbound client: Sysrepo"};
 
 static struct event_loop *master;
@@ -118,6 +116,9 @@ static int yang_data_frr2sr(struct yang_data *frr_data, sr_val_t *sr_data)
 		sr_data->type = SR_INT64_T;
 		sr_data->data.int64_val = yang_str2int64(frr_data->value);
 		break;
+	case LY_TYPE_LEAFREF:
+		sr_val_set_str_data(sr_data, SR_STRING_T, frr_data->value);
+		break;
 	case LY_TYPE_STRING:
 		sr_val_set_str_data(sr_data, SR_STRING_T, frr_data->value);
 		break;
@@ -137,6 +138,11 @@ static int yang_data_frr2sr(struct yang_data *frr_data, sr_val_t *sr_data)
 		sr_data->type = SR_UINT64_T;
 		sr_data->data.uint64_val = yang_str2uint64(frr_data->value);
 		break;
+	case LY_TYPE_UNION:
+		/* No way to deal with this using un-typed yang_data object */
+		sr_val_set_str_data(sr_data, SR_STRING_T, frr_data->value);
+		break;
+	case LY_TYPE_UNKNOWN:
 	default:
 		return -1;
 	}
@@ -178,12 +184,12 @@ static int frr_sr_process_change(struct nb_config *candidate,
 	/* Map operation values. */
 	switch (sr_op) {
 	case SR_OP_CREATED:
+		nb_op = NB_OP_CREATE;
+		break;
 	case SR_OP_MODIFIED:
-		if (nb_operation_is_valid(NB_OP_CREATE, nb_node->snode))
-			nb_op = NB_OP_CREATE;
-		else if (nb_operation_is_valid(NB_OP_MODIFY, nb_node->snode)) {
+		if (nb_is_operation_allowed(nb_node, NB_OP_MODIFY))
 			nb_op = NB_OP_MODIFY;
-		} else
+		else
 			/* Ignore list keys modifications. */
 			return NB_OK;
 		break;
@@ -193,7 +199,7 @@ static int frr_sr_process_change(struct nb_config *candidate,
 		 * notified about the removal of all of its leafs, even the ones
 		 * that are non-optional. We need to ignore these notifications.
 		 */
-		if (!nb_operation_is_valid(NB_OP_DESTROY, nb_node->snode))
+		if (!nb_is_operation_allowed(nb_node, NB_OP_DESTROY))
 			return NB_OK;
 
 		nb_op = NB_OP_DESTROY;
@@ -213,7 +219,7 @@ static int frr_sr_process_change(struct nb_config *candidate,
 
 	ret = nb_candidate_edit(candidate, nb_node, nb_op, xpath, NULL, data);
 	yang_data_free(data);
-	if (ret != NB_OK && ret != NB_ERR_NOT_FOUND) {
+	if (ret != NB_OK) {
 		flog_warn(
 			EC_LIB_NB_CANDIDATE_EDIT_ERROR,
 			"%s: failed to edit candidate configuration: operation [%s] xpath [%s]",
@@ -271,11 +277,12 @@ static int frr_sr_config_change_cb_prepare(sr_session_ctx_t *session,
 	ret = nb_candidate_commit_prepare(context, candidate, NULL,
 					  &transaction, false, false, errmsg,
 					  sizeof(errmsg));
-	if (ret != NB_OK && ret != NB_ERR_NO_CHANGES)
-		flog_warn(
-			EC_LIB_LIBSYSREPO,
-			"%s: failed to prepare configuration transaction: %s (%s)",
-			__func__, nb_err_name(ret), errmsg);
+	if (ret != NB_OK && ret != NB_ERR_NO_CHANGES) {
+		flog_warn(EC_LIB_LIBSYSREPO,
+			  "%s: failed to prepare configuration transaction: %s (%s)",
+			  __func__, nb_err_name(ret), errmsg);
+		sr_session_set_error_message(session, errmsg);
+	}
 
 	if (!transaction)
 		nb_config_free(candidate);
@@ -340,6 +347,8 @@ static int frr_sr_config_change_cb(sr_session_ctx_t *session, uint32_t sub_id,
 		return frr_sr_config_change_cb_apply(session, module_name);
 	case SR_EV_ABORT:
 		return frr_sr_config_change_cb_abort(session, module_name);
+	case SR_EV_RPC:
+	case SR_EV_UPDATE:
 	default:
 		flog_err(EC_LIB_LIBSYSREPO, "%s: unexpected sysrepo event: %u",
 			 __func__, sr_ev);
@@ -347,39 +356,16 @@ static int frr_sr_config_change_cb(sr_session_ctx_t *session, uint32_t sub_id,
 	}
 }
 
-static int frr_sr_state_data_iter_cb(const struct lysc_node *snode,
-				     struct yang_translator *translator,
-				     struct yang_data *data, void *arg)
-{
-	struct lyd_node *dnode = arg;
-	LY_ERR ly_errno;
-
-	ly_errno = 0;
-	ly_errno = lyd_new_path(NULL, ly_native_ctx, data->xpath, data->value,
-				0, &dnode);
-	if (ly_errno) {
-		flog_warn(EC_LIB_LIBYANG, "%s: lyd_new_path() failed",
-			  __func__);
-		yang_data_free(data);
-		return NB_ERR;
-	}
-
-	yang_data_free(data);
-	return NB_OK;
-}
-
 /* Callback for state retrieval. */
 static int frr_sr_state_cb(sr_session_ctx_t *session, uint32_t sub_id,
 			   const char *module_name, const char *xpath,
 			   const char *request_xpath, uint32_t request_id,
 			   struct lyd_node **parent, void *private_ctx)
 {
-	struct lyd_node *dnode;
+	struct lyd_node *dnode = NULL;
 
 	dnode = *parent;
-	if (nb_oper_data_iterate(request_xpath, NULL, 0,
-				 frr_sr_state_data_iter_cb, dnode)
-	    != NB_OK) {
+	if (nb_oper_iterate_legacy(request_xpath, NULL, 0, NULL, NULL, &dnode)) {
 		flog_warn(EC_LIB_NB_OPERATIONAL_DATA,
 			  "%s: failed to obtain operational data [xpath %s]",
 			  __func__, xpath);
@@ -391,16 +377,11 @@ static int frr_sr_state_cb(sr_session_ctx_t *session, uint32_t sub_id,
 	return SR_ERR_OK;
 }
 static int frr_sr_config_rpc_cb(sr_session_ctx_t *session, uint32_t sub_id,
-				const char *xpath, const sr_val_t *sr_input,
-				const size_t input_cnt, sr_event_t sr_ev,
-				uint32_t request_id, sr_val_t **sr_output,
-				size_t *sr_output_cnt, void *private_ctx)
+				const char *xpath, const struct lyd_node *input,
+				sr_event_t sr_ev, uint32_t request_id,
+				struct lyd_node *output, void *private_ctx)
 {
 	struct nb_node *nb_node;
-	struct list *input;
-	struct list *output;
-	struct yang_data *data;
-	size_t cb_output_cnt;
 	int ret = SR_ERR_OK;
 	char errmsg[BUFSIZ] = {0};
 
@@ -411,19 +392,6 @@ static int frr_sr_config_rpc_cb(sr_session_ctx_t *session, uint32_t sub_id,
 		return SR_ERR_INTERNAL;
 	}
 
-	input = yang_data_list_new();
-	output = yang_data_list_new();
-
-	/* Process input. */
-	for (size_t i = 0; i < input_cnt; i++) {
-		char value_str[YANG_VALUE_MAXLEN];
-
-		sr_val_to_buff(&sr_input[i], value_str, sizeof(value_str));
-
-		data = yang_data_new(xpath, value_str);
-		listnode_add(input, data);
-	}
-
 	/* Execute callback registered for this XPath. */
 	if (nb_callback_rpc(nb_node, xpath, input, output, errmsg,
 			    sizeof(errmsg))
@@ -431,44 +399,8 @@ static int frr_sr_config_rpc_cb(sr_session_ctx_t *session, uint32_t sub_id,
 		flog_warn(EC_LIB_NB_CB_RPC, "%s: rpc callback failed: %s",
 			  __func__, xpath);
 		ret = SR_ERR_OPERATION_FAILED;
-		goto exit;
 	}
 
-	/* Process output. */
-	if (listcount(output) > 0) {
-		sr_val_t *values = NULL;
-		struct listnode *node;
-		int i = 0;
-
-		cb_output_cnt = listcount(output);
-		ret = sr_new_values(cb_output_cnt, &values);
-		if (ret != SR_ERR_OK) {
-			flog_err(EC_LIB_LIBSYSREPO, "%s: sr_new_values(): %s",
-				 __func__, sr_strerror(ret));
-			goto exit;
-		}
-
-		for (ALL_LIST_ELEMENTS_RO(output, node, data)) {
-			if (yang_data_frr2sr(data, &values[i++]) != 0) {
-				flog_err(
-					EC_LIB_SYSREPO_DATA_CONVERT,
-					"%s: failed to convert data to Sysrepo format",
-					__func__);
-				ret = SR_ERR_INTERNAL;
-				sr_free_values(values, cb_output_cnt);
-				goto exit;
-			}
-		}
-
-		*sr_output = values;
-		*sr_output_cnt = cb_output_cnt;
-	}
-
-exit:
-	/* Release memory. */
-	list_delete(&input);
-	list_delete(&output);
-
 	return ret;
 }
 
@@ -593,8 +525,9 @@ static int frr_sr_subscribe_rpc(const struct lysc_node *snode, void *arg)
 	DEBUGD(&nb_dbg_client_sysrepo, "sysrepo: providing RPC to '%s'",
 	       nb_node->xpath);
 
-	ret = sr_rpc_subscribe(session, nb_node->xpath, frr_sr_config_rpc_cb,
-			       NULL, 0, 0, &module->sr_subscription);
+	ret = sr_rpc_subscribe_tree(session, nb_node->xpath,
+				    frr_sr_config_rpc_cb, NULL, 0, 0,
+				    &module->sr_subscription);
 	if (ret != SR_ERR_OK)
 		flog_err(EC_LIB_LIBSYSREPO, "sr_rpc_subscribe(): %s",
 			 sr_strerror(ret));
