@@ -1,10 +1,6 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /* NHRP vty handling
  * Copyright (c) 2014-2015 Timo Teräs
- *
- * This file is free software: you may copy, redistribute and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 2 of the License, or
- * (at your option) any later version.
  */
 
 #include "zebra.h"
@@ -830,7 +826,7 @@ static void show_ip_nhrp_shortcut(struct nhrp_shortcut *s, void *pctx)
 	char buf1[PREFIX_STRLEN], buf2[SU_ADDRSTRLEN];
 	struct json_object *json = NULL;
 
-	if (!ctx->count) {
+	if (!ctx->count && !ctx->json) {
 		vty_out(vty, "%-8s %-24s %-24s %s\n", "Type", "Prefix", "Via",
 			"Identity");
 	}
