@@ -1,27 +1,12 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * Copyright (C) 2003 Yasuhiro Ohara
- *
- * This file is part of GNU Zebra.
- *
- * GNU Zebra is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * GNU Zebra is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
 
 #include "log.h"
-#include "thread.h"
+#include "frrevent.h"
 #include "linklist.h"
 #include "vty.h"
 #include "command.h"
@@ -40,6 +25,8 @@
 #include "ospf6_neighbor.h"
 
 #include "ospf6_flood.h"
+#include "ospf6_nssa.h"
+#include "ospf6_gr.h"
 
 unsigned char conf_debug_ospf6_flooding;
 
@@ -83,18 +70,29 @@ struct ospf6_lsdb *ospf6_get_scoped_lsdb_self(struct ospf6_lsa *lsa)
 	return lsdb_self;
 }
 
-void ospf6_lsa_originate(struct ospf6_lsa *lsa)
+void ospf6_lsa_originate(struct ospf6 *ospf6, struct ospf6_lsa *lsa)
 {
 	struct ospf6_lsa *old;
 	struct ospf6_lsdb *lsdb_self;
 
+	if (lsa->header->adv_router == INADDR_ANY) {
+		if (IS_OSPF6_DEBUG_ORIGINATE_TYPE(lsa->header->type))
+			zlog_debug(
+				"Refusing to originate LSA (zero router ID): %s",
+				lsa->name);
+
+		ospf6_lsa_delete(lsa);
+		return;
+	}
+
 	/* find previous LSA */
 	old = ospf6_lsdb_lookup(lsa->header->type, lsa->header->id,
 				lsa->header->adv_router, lsa->lsdb);
 
 	/* if the new LSA does not differ from previous,
 	   suppress this update of the LSA */
-	if (old && !OSPF6_LSA_IS_DIFFER(lsa, old)) {
+	if (old && !OSPF6_LSA_IS_DIFFER(lsa, old)
+	    && !ospf6->gr_info.finishing_restart) {
 		if (IS_OSPF6_DEBUG_ORIGINATE_TYPE(lsa->header->type))
 			zlog_debug("Suppress updating LSA: %s", lsa->name);
 		ospf6_lsa_delete(lsa);
@@ -105,9 +103,9 @@ void ospf6_lsa_originate(struct ospf6_lsa *lsa)
 	lsdb_self = ospf6_get_scoped_lsdb_self(lsa);
 	ospf6_lsdb_add(ospf6_lsa_copy(lsa), lsdb_self);
 
-	lsa->refresh = NULL;
-	thread_add_timer(master, ospf6_lsa_refresh, lsa, OSPF_LS_REFRESH_TIME,
-			 &lsa->refresh);
+	EVENT_OFF(lsa->refresh);
+	event_add_timer(master, ospf6_lsa_refresh, lsa, OSPF_LS_REFRESH_TIME,
+			&lsa->refresh);
 
 	if (IS_OSPF6_DEBUG_LSA_TYPE(lsa->header->type)
 	    || IS_OSPF6_DEBUG_ORIGINATE_TYPE(lsa->header->type)) {
@@ -122,20 +120,42 @@ void ospf6_lsa_originate(struct ospf6_lsa *lsa)
 void ospf6_lsa_originate_process(struct ospf6_lsa *lsa, struct ospf6 *process)
 {
 	lsa->lsdb = process->lsdb;
-	ospf6_lsa_originate(lsa);
+	ospf6_lsa_originate(process, lsa);
 }
 
 void ospf6_lsa_originate_area(struct ospf6_lsa *lsa, struct ospf6_area *oa)
 {
 	lsa->lsdb = oa->lsdb;
-	ospf6_lsa_originate(lsa);
+	ospf6_lsa_originate(oa->ospf6, lsa);
 }
 
 void ospf6_lsa_originate_interface(struct ospf6_lsa *lsa,
 				   struct ospf6_interface *oi)
 {
 	lsa->lsdb = oi->lsdb;
-	ospf6_lsa_originate(lsa);
+	ospf6_lsa_originate(oi->area->ospf6, lsa);
+}
+
+void ospf6_external_lsa_purge(struct ospf6 *ospf6, struct ospf6_lsa *lsa)
+{
+	uint32_t id = lsa->header->id;
+	struct ospf6_area *oa;
+	struct listnode *lnode;
+
+	ospf6_lsa_purge(lsa);
+
+	/* Delete the corresponding NSSA LSA */
+	for (ALL_LIST_ELEMENTS_RO(ospf6->area_list, lnode, oa)) {
+		lsa = ospf6_lsdb_lookup(htons(OSPF6_LSTYPE_TYPE_7), id,
+					ospf6->router_id, oa->lsdb);
+		if (lsa) {
+			if (IS_OSPF6_DEBUG_NSSA)
+				zlog_debug("withdraw type 7 lsa, LS ID: %u",
+					   htonl(id));
+
+			ospf6_lsa_purge(lsa);
+		}
+	}
 }
 
 void ospf6_lsa_purge(struct ospf6_lsa *lsa)
@@ -148,8 +168,8 @@ void ospf6_lsa_purge(struct ospf6_lsa *lsa)
 	self = ospf6_lsdb_lookup(lsa->header->type, lsa->header->id,
 				 lsa->header->adv_router, lsdb_self);
 	if (self) {
-		THREAD_OFF(self->expire);
-		THREAD_OFF(self->refresh);
+		EVENT_OFF(self->expire);
+		EVENT_OFF(self->refresh);
 		ospf6_lsdb_remove(self, lsdb_self);
 	}
 
@@ -211,26 +231,36 @@ void ospf6_decrement_retrans_count(struct ospf6_lsa *lsa)
 /* RFC2328 section 13.2 Installing LSAs in the database */
 void ospf6_install_lsa(struct ospf6_lsa *lsa)
 {
+	struct ospf6 *ospf6;
 	struct timeval now;
 	struct ospf6_lsa *old;
+	struct ospf6_area *area = NULL;
+
+	ospf6 = ospf6_get_by_lsdb(lsa);
+	assert(ospf6);
 
 	/* Remove the old instance from all neighbors' Link state
 	   retransmission list (RFC2328 13.2 last paragraph) */
 	old = ospf6_lsdb_lookup(lsa->header->type, lsa->header->id,
 				lsa->header->adv_router, lsa->lsdb);
 	if (old) {
-		THREAD_OFF(old->expire);
-		THREAD_OFF(old->refresh);
+		if (ntohs(lsa->header->type) == OSPF6_LSTYPE_TYPE_7) {
+			if (IS_OSPF6_DEBUG_NSSA)
+				zlog_debug("%s : old LSA %s", __func__,
+					   lsa->name);
+			lsa->external_lsa_id = old->external_lsa_id;
+		}
+		EVENT_OFF(old->expire);
+		EVENT_OFF(old->refresh);
 		ospf6_flood_clear(old);
 	}
 
 	monotime(&now);
 	if (!OSPF6_LSA_IS_MAXAGE(lsa)) {
-		lsa->expire = NULL;
-		thread_add_timer(master, ospf6_lsa_expire, lsa,
-				 OSPF_LSA_MAXAGE + lsa->birth.tv_sec
-					 - now.tv_sec,
-				 &lsa->expire);
+		event_add_timer(master, ospf6_lsa_expire, lsa,
+				OSPF_LSA_MAXAGE + lsa->birth.tv_sec -
+					now.tv_sec,
+				&lsa->expire);
 	} else
 		lsa->expire = NULL;
 
@@ -263,8 +293,33 @@ void ospf6_install_lsa(struct ospf6_lsa *lsa)
 
 	/* actually install */
 	lsa->installed = now;
+
+	/* Topo change handling */
+	if (CHECK_LSA_TOPO_CHG_ELIGIBLE(ntohs(lsa->header->type))) {
+		/* check if it is new lsa ? or existing lsa got modified ?*/
+		if (!old || OSPF6_LSA_IS_CHANGED(old, lsa))
+			ospf6_helper_handle_topo_chg(ospf6, lsa);
+	}
+
 	ospf6_lsdb_add(lsa, lsa->lsdb);
 
+	if (ntohs(lsa->header->type) == OSPF6_LSTYPE_TYPE_7
+	    && lsa->header->adv_router != ospf6->router_id) {
+		area = OSPF6_AREA(lsa->lsdb->data);
+		ospf6_translated_nssa_refresh(area, lsa, NULL);
+		ospf6_schedule_abr_task(area->ospf6);
+	}
+
+	if (ntohs(lsa->header->type) == OSPF6_LSTYPE_ROUTER) {
+		area = OSPF6_AREA(lsa->lsdb->data);
+		if (old == NULL) {
+			if (IS_OSPF6_DEBUG_LSA_TYPE(lsa->header->type)
+			    || IS_OSPF6_DEBUG_EXAMIN_TYPE(lsa->header->type))
+				zlog_debug("%s: New router LSA %s", __func__,
+					   lsa->name);
+			ospf6_abr_nssa_check_status(area->ospf6);
+		}
+	}
 	return;
 }
 
@@ -275,7 +330,7 @@ void ospf6_flood_interface(struct ospf6_neighbor *from, struct ospf6_lsa *lsa,
 {
 	struct listnode *node, *nnode;
 	struct ospf6_neighbor *on;
-	struct ospf6_lsa *req;
+	struct ospf6_lsa *req, *old;
 	int retrans_added = 0;
 	int is_debug = 0;
 
@@ -332,7 +387,7 @@ void ospf6_flood_interface(struct ospf6_neighbor *from, struct ospf6_lsa *lsa,
 					if (req == on->last_ls_req) {
 						/* sanity check refcount */
 						assert(req->lock >= 2);
-						req = ospf6_lsa_unlock(req);
+						ospf6_lsa_unlock(&req);
 						on->last_ls_req = NULL;
 					}
 					if (req)
@@ -349,7 +404,7 @@ void ospf6_flood_interface(struct ospf6_neighbor *from, struct ospf6_lsa *lsa,
 						zlog_debug(
 							"Received is newer, remove requesting");
 					if (req == on->last_ls_req) {
-						req = ospf6_lsa_unlock(req);
+						ospf6_lsa_unlock(&req);
 						on->last_ls_req = NULL;
 					}
 					if (req)
@@ -370,7 +425,8 @@ void ospf6_flood_interface(struct ospf6_neighbor *from, struct ospf6_lsa *lsa,
 			continue;
 		}
 
-		if (ospf6->inst_shutdown) {
+		if ((oi->area->ospf6->inst_shutdown)
+		    || CHECK_FLAG(lsa->flag, OSPF6_LSA_FLUSH)) {
 			if (is_debug)
 				zlog_debug(
 					"%s: Send LSA %s (age %d) update now",
@@ -381,13 +437,46 @@ void ospf6_flood_interface(struct ospf6_neighbor *from, struct ospf6_lsa *lsa,
 		} else {
 			/* (d) add retrans-list, schedule retransmission */
 			if (is_debug)
-				zlog_debug("Add retrans-list of this neighbor");
-			ospf6_increment_retrans_count(lsa);
-			ospf6_lsdb_add(ospf6_lsa_copy(lsa), on->retrans_list);
-			thread_add_timer(master, ospf6_lsupdate_send_neighbor,
-					 on, on->ospf6_if->rxmt_interval,
-					 &on->thread_send_lsupdate);
-			retrans_added++;
+				zlog_debug("Add retrans-list of neighbor %s ",
+					   on->name);
+
+			/* Do not increment the retrans count if the lsa is
+			 * already present in the retrans list.
+			 */
+			old = ospf6_lsdb_lookup(
+				lsa->header->type, lsa->header->id,
+				lsa->header->adv_router, on->retrans_list);
+			if (!old) {
+				struct ospf6_lsa *orig;
+				struct ospf6_lsdb *lsdb;
+
+				if (is_debug)
+					zlog_debug(
+						"Increment %s from retrans_list of %s",
+						lsa->name, on->name);
+
+				/* Increment the retrans count on the original
+				 * copy of LSA if present, to maintain the
+				 * counter consistency.
+				 */
+
+				lsdb = ospf6_get_scoped_lsdb(lsa);
+				orig = ospf6_lsdb_lookup(
+					lsa->header->type, lsa->header->id,
+					lsa->header->adv_router, lsdb);
+				if (orig)
+					ospf6_increment_retrans_count(orig);
+				else
+					ospf6_increment_retrans_count(lsa);
+
+				ospf6_lsdb_add(ospf6_lsa_copy(lsa),
+					       on->retrans_list);
+				event_add_timer(master,
+						ospf6_lsupdate_send_neighbor,
+						on, on->ospf6_if->rxmt_interval,
+						&on->thread_send_lsupdate);
+				retrans_added++;
+			}
 		}
 	}
 
@@ -395,7 +484,8 @@ void ospf6_flood_interface(struct ospf6_neighbor *from, struct ospf6_lsa *lsa,
 	if (retrans_added == 0) {
 		if (is_debug)
 			zlog_debug(
-				"No retransmission scheduled, next interface");
+				"No retransmission scheduled, next interface %s",
+				oi->interface->name);
 		return;
 	}
 
@@ -428,15 +518,14 @@ void ospf6_flood_interface(struct ospf6_neighbor *from, struct ospf6_lsa *lsa,
 	if ((oi->type == OSPF_IFTYPE_BROADCAST)
 	    || (oi->type == OSPF_IFTYPE_POINTOPOINT)) {
 		ospf6_lsdb_add(ospf6_lsa_copy(lsa), oi->lsupdate_list);
-		thread_add_event(master, ospf6_lsupdate_send_interface, oi, 0,
-				 &oi->thread_send_lsupdate);
+		event_add_event(master, ospf6_lsupdate_send_interface, oi, 0,
+				&oi->thread_send_lsupdate);
 	} else {
 		/* reschedule retransmissions to all neighbors */
 		for (ALL_LIST_ELEMENTS(oi->neighbor_list, node, nnode, on)) {
-			THREAD_OFF(on->thread_send_lsupdate);
-			on->thread_send_lsupdate = NULL;
-			thread_add_event(master, ospf6_lsupdate_send_neighbor,
-					 on, 0, &on->thread_send_lsupdate);
+			EVENT_OFF(on->thread_send_lsupdate);
+			event_add_event(master, ospf6_lsupdate_send_neighbor,
+					on, 0, &on->thread_send_lsupdate);
 		}
 	}
 }
@@ -452,12 +541,6 @@ void ospf6_flood_area(struct ospf6_neighbor *from, struct ospf6_lsa *lsa,
 		    && oi != OSPF6_INTERFACE(lsa->lsdb->data))
 			continue;
 
-#if 0
-      if (OSPF6_LSA_SCOPE (lsa->header->type) == OSPF6_SCOPE_AS &&
-          ospf6_is_interface_virtual_link (oi))
-        continue;
-#endif /*0*/
-
 		ospf6_flood_interface(from, lsa, oi);
 	}
 }
@@ -469,6 +552,19 @@ static void ospf6_flood_process(struct ospf6_neighbor *from,
 	struct ospf6_area *oa;
 
 	for (ALL_LIST_ELEMENTS(process->area_list, node, nnode, oa)) {
+
+		/* If unknown LSA and U-bit clear, treat as link local
+		 * flooding scope
+		 */
+		if (!OSPF6_LSA_IS_KNOWN(lsa->header->type)
+		    && !(ntohs(lsa->header->type) & OSPF6_LSTYPE_UBIT_MASK)
+		    && (oa != OSPF6_INTERFACE(lsa->lsdb->data)->area)) {
+
+			if (IS_OSPF6_DEBUG_FLOODING)
+				zlog_debug("Unknown LSA, do not flood");
+			continue;
+		}
+
 		if (OSPF6_LSA_SCOPE(lsa->header->type) == OSPF6_SCOPE_AREA
 		    && oa != OSPF6_AREA(lsa->lsdb->data))
 			continue;
@@ -477,7 +573,12 @@ static void ospf6_flood_process(struct ospf6_neighbor *from,
 			continue;
 
 		if (ntohs(lsa->header->type) == OSPF6_LSTYPE_AS_EXTERNAL
-		    && IS_AREA_STUB(oa))
+		    && (IS_AREA_STUB(oa) || IS_AREA_NSSA(oa)))
+			continue;
+
+		/* Check for NSSA LSA */
+		if (ntohs(lsa->header->type) == OSPF6_LSTYPE_TYPE_7
+		    && !IS_AREA_NSSA(oa) && !OSPF6_LSA_IS_MAXAGE(lsa))
 			continue;
 
 		ospf6_flood_area(from, lsa, oa);
@@ -486,6 +587,12 @@ static void ospf6_flood_process(struct ospf6_neighbor *from,
 
 void ospf6_flood(struct ospf6_neighbor *from, struct ospf6_lsa *lsa)
 {
+	struct ospf6 *ospf6;
+
+	ospf6 = ospf6_get_by_lsdb(lsa);
+	if (ospf6 == NULL)
+		return;
+
 	ospf6_flood_process(from, lsa, ospf6);
 }
 
@@ -511,7 +618,7 @@ static void ospf6_flood_clear_interface(struct ospf6_lsa *lsa,
 	}
 }
 
-static void ospf6_flood_clear_area(struct ospf6_lsa *lsa, struct ospf6_area *oa)
+void ospf6_flood_clear_area(struct ospf6_lsa *lsa, struct ospf6_area *oa)
 {
 	struct listnode *node, *nnode;
 	struct ospf6_interface *oi;
@@ -521,12 +628,6 @@ static void ospf6_flood_clear_area(struct ospf6_lsa *lsa, struct ospf6_area *oa)
 		    && oi != OSPF6_INTERFACE(lsa->lsdb->data))
 			continue;
 
-#if 0
-      if (OSPF6_LSA_SCOPE (lsa->header->type) == OSPF6_SCOPE_AS &&
-          ospf6_is_interface_virtual_link (oi))
-        continue;
-#endif /*0*/
-
 		ospf6_flood_clear_interface(lsa, oi);
 	}
 }
@@ -546,7 +647,11 @@ static void ospf6_flood_clear_process(struct ospf6_lsa *lsa,
 			continue;
 
 		if (ntohs(lsa->header->type) == OSPF6_LSTYPE_AS_EXTERNAL
-		    && IS_AREA_STUB(oa))
+		    && (IS_AREA_STUB(oa) || (IS_AREA_NSSA(oa))))
+			continue;
+		/* Check for NSSA LSA */
+		if (ntohs(lsa->header->type) == OSPF6_LSTYPE_TYPE_7
+		    && !IS_AREA_NSSA(oa))
 			continue;
 
 		ospf6_flood_clear_area(lsa, oa);
@@ -555,6 +660,11 @@ static void ospf6_flood_clear_process(struct ospf6_lsa *lsa,
 
 void ospf6_flood_clear(struct ospf6_lsa *lsa)
 {
+	struct ospf6 *ospf6;
+
+	ospf6 = ospf6_get_by_lsdb(lsa);
+	if (ospf6 == NULL)
+		return;
 	ospf6_flood_clear_process(lsa, ospf6);
 }
 
@@ -585,8 +695,8 @@ static void ospf6_acknowledge_lsa_bdrouter(struct ospf6_lsa *lsa,
 					"Delayed acknowledgement (BDR & MoreRecent & from DR)");
 			/* Delayed acknowledgement */
 			ospf6_lsdb_add(ospf6_lsa_copy(lsa), oi->lsack_list);
-			thread_add_timer(master, ospf6_lsack_send_interface, oi,
-					 3, &oi->thread_send_lsack);
+			event_add_timer(master, ospf6_lsack_send_interface, oi,
+					3, &oi->thread_send_lsack);
 		} else {
 			if (is_debug)
 				zlog_debug(
@@ -606,8 +716,8 @@ static void ospf6_acknowledge_lsa_bdrouter(struct ospf6_lsa *lsa,
 					"Delayed acknowledgement (BDR & Duplicate & ImpliedAck & from DR)");
 			/* Delayed acknowledgement */
 			ospf6_lsdb_add(ospf6_lsa_copy(lsa), oi->lsack_list);
-			thread_add_timer(master, ospf6_lsack_send_interface, oi,
-					 3, &oi->thread_send_lsack);
+			event_add_timer(master, ospf6_lsack_send_interface, oi,
+					3, &oi->thread_send_lsack);
 		} else {
 			if (is_debug)
 				zlog_debug(
@@ -624,8 +734,8 @@ static void ospf6_acknowledge_lsa_bdrouter(struct ospf6_lsa *lsa,
 		if (is_debug)
 			zlog_debug("Direct acknowledgement (BDR & Duplicate)");
 		ospf6_lsdb_add(ospf6_lsa_copy(lsa), from->lsack_list);
-		thread_add_event(master, ospf6_lsack_send_neighbor, from, 0,
-				 &from->thread_send_lsack);
+		event_add_event(master, ospf6_lsack_send_neighbor, from, 0,
+				&from->thread_send_lsack);
 		return;
 	}
 
@@ -666,8 +776,8 @@ static void ospf6_acknowledge_lsa_allother(struct ospf6_lsa *lsa,
 				"Delayed acknowledgement (AllOther & MoreRecent)");
 		/* Delayed acknowledgement */
 		ospf6_lsdb_add(ospf6_lsa_copy(lsa), oi->lsack_list);
-		thread_add_timer(master, ospf6_lsack_send_interface, oi, 3,
-				 &oi->thread_send_lsack);
+		event_add_timer(master, ospf6_lsack_send_interface, oi, 3,
+				&oi->thread_send_lsack);
 		return;
 	}
 
@@ -690,8 +800,8 @@ static void ospf6_acknowledge_lsa_allother(struct ospf6_lsa *lsa,
 			zlog_debug(
 				"Direct acknowledgement (AllOther & Duplicate)");
 		ospf6_lsdb_add(ospf6_lsa_copy(lsa), from->lsack_list);
-		thread_add_event(master, ospf6_lsack_send_neighbor, from, 0,
-				 &from->thread_send_lsack);
+		event_add_event(master, ospf6_lsack_send_neighbor, from, 0,
+				&from->thread_send_lsack);
 		return;
 	}
 
@@ -751,6 +861,28 @@ static int ospf6_is_maxage_lsa_drop(struct ospf6_lsa *lsa,
 	return 0;
 }
 
+static bool ospf6_lsa_check_min_arrival(struct ospf6_lsa *lsa,
+					struct ospf6_neighbor *from)
+{
+	struct timeval now, res;
+	unsigned int time_delta_ms;
+
+	monotime(&now);
+	timersub(&now, &lsa->installed, &res);
+	time_delta_ms = (res.tv_sec * 1000) + (int)(res.tv_usec / 1000);
+
+	if (time_delta_ms < from->ospf6_if->area->ospf6->lsa_minarrival) {
+		if (IS_OSPF6_DEBUG_FLOODING ||
+		    IS_OSPF6_DEBUG_FLOOD_TYPE(lsa->header->type))
+			zlog_debug(
+				"LSA can't be updated within MinLSArrival, %dms < %dms, discard",
+				time_delta_ms,
+				from->ospf6_if->area->ospf6->lsa_minarrival);
+		return true;
+	}
+	return false;
+}
+
 /* RFC2328 section 13 The Flooding Procedure */
 void ospf6_receive_lsa(struct ospf6_neighbor *from,
 		       struct ospf6_lsa_header *lsa_header)
@@ -758,11 +890,22 @@ void ospf6_receive_lsa(struct ospf6_neighbor *from,
 	struct ospf6_lsa *new = NULL, *old = NULL, *rem = NULL;
 	int ismore_recent;
 	int is_debug = 0;
-	unsigned int time_delta_ms;
 
 	ismore_recent = 1;
 	assert(from);
 
+	/* if we receive a LSA with invalid seqnum drop it */
+	if (ntohl(lsa_header->seqnum) - 1 == OSPF_MAX_SEQUENCE_NUMBER) {
+		if (IS_OSPF6_DEBUG_EXAMIN_TYPE(lsa_header->type)) {
+			zlog_debug(
+				"received lsa [%s Id:%pI4 Adv:%pI4] with invalid seqnum 0x%x, ignore",
+				ospf6_lstype_name(lsa_header->type),
+				&lsa_header->id, &lsa_header->adv_router,
+				ntohl(lsa_header->seqnum));
+		}
+		return;
+	}
+
 	/* make lsa structure for received lsa */
 	new = ospf6_lsa_create(lsa_header);
 
@@ -776,7 +919,11 @@ void ospf6_receive_lsa(struct ospf6_neighbor *from,
 	/* (1) LSA Checksum */
 	if (!ospf6_lsa_checksum_valid(new->header)) {
 		if (is_debug)
-			zlog_debug("Wrong LSA Checksum, discard");
+			zlog_debug(
+				"Wrong LSA Checksum %s (Router-ID: %pI4) [Type:%s Checksum:%#06hx), discard",
+				from->name, &from->router_id,
+				ospf6_lstype_name(new->header->type),
+				ntohs(new->header->checksum));
 		ospf6_lsa_delete(new);
 		return;
 	}
@@ -824,8 +971,8 @@ void ospf6_receive_lsa(struct ospf6_neighbor *from,
 		/* a) Acknowledge back to neighbor (Direct acknowledgement,
 		 * 13.5) */
 		ospf6_lsdb_add(ospf6_lsa_copy(new), from->lsack_list);
-		thread_add_event(master, ospf6_lsack_send_neighbor, from, 0,
-				 &from->thread_send_lsack);
+		event_add_event(master, ospf6_lsack_send_neighbor, from, 0,
+				&from->thread_send_lsack);
 
 		/* b) Discard */
 		ospf6_lsa_delete(new);
@@ -847,24 +994,14 @@ void ospf6_receive_lsa(struct ospf6_neighbor *from,
 
 	/* if no database copy or received is more recent */
 	if (old == NULL || ismore_recent < 0) {
+		bool self_originated;
+
 		/* in case we have no database copy */
 		ismore_recent = -1;
 
 		/* (a) MinLSArrival check */
 		if (old) {
-			struct timeval now, res;
-			monotime(&now);
-			timersub(&now, &old->installed, &res);
-			time_delta_ms =
-				(res.tv_sec * 1000) + (int)(res.tv_usec / 1000);
-			if (time_delta_ms
-			    < from->ospf6_if->area->ospf6->lsa_minarrival) {
-				if (is_debug)
-					zlog_debug(
-						"LSA can't be updated within MinLSArrival, %dms < %dms, discard",
-						time_delta_ms,
-						from->ospf6_if->area->ospf6
-							->lsa_minarrival);
+			if (ospf6_lsa_check_min_arrival(old, from)) {
 				ospf6_lsa_delete(new);
 				return; /* examin next lsa */
 			}
@@ -880,13 +1017,60 @@ void ospf6_receive_lsa(struct ospf6_neighbor *from,
 		if (old)
 			ospf6_flood_clear(old);
 
+		self_originated = (new->header->adv_router
+				   == from->ospf6_if->area->ospf6->router_id);
+
+		/* Received non-self-originated Grace LSA. */
+		if (IS_GRACE_LSA(new) && !self_originated) {
+			struct ospf6 *ospf6;
+
+			ospf6 = ospf6_get_by_lsdb(new);
+
+			assert(ospf6);
+
+			if (OSPF6_LSA_IS_MAXAGE(new)) {
+
+				if (IS_DEBUG_OSPF6_GR)
+					zlog_debug(
+						"%s, Received a maxage GraceLSA from router %pI4",
+						__func__,
+						&new->header->adv_router);
+				if (old) {
+					ospf6_process_maxage_grace_lsa(
+						ospf6, new, from);
+				} else {
+					if (IS_DEBUG_OSPF6_GR)
+						zlog_debug(
+							"%s, GraceLSA doesn't exist in lsdb, so discarding GraceLSA",
+							__func__);
+					ospf6_lsa_delete(new);
+					return;
+				}
+			} else {
+
+				if (IS_DEBUG_OSPF6_GR)
+					zlog_debug(
+						"%s, Received a GraceLSA from router %pI4",
+						__func__,
+						&new->header->adv_router);
+
+				if (ospf6_process_grace_lsa(ospf6, new, from)
+				    == OSPF6_GR_NOT_HELPER) {
+					if (IS_DEBUG_OSPF6_GR)
+						zlog_debug(
+							"%s, Not moving to HELPER role, So dicarding GraceLSA",
+							__func__);
+					return;
+				}
+			}
+		}
+
 		/* (b) immediately flood and (c) remove from all retrans-list */
 		/* Prevent self-originated LSA to be flooded. this is to make
-		reoriginated instance of the LSA not to be rejected by other
-		routers
-		due to MinLSArrival. */
-		if (new->header->adv_router
-		    != from->ospf6_if->area->ospf6->router_id)
+		 * reoriginated instance of the LSA not to be rejected by other
+		 * routers due to MinLSArrival.
+		 */
+		if (!self_originated)
 			ospf6_flood(from, new);
 
 		/* (d), installing lsdb, which may cause routing
@@ -900,8 +1084,16 @@ void ospf6_receive_lsa(struct ospf6_neighbor *from,
 		ospf6_acknowledge_lsa(new, ismore_recent, from);
 
 		/* (f) Self Originated LSA, section 13.4 */
-		if (new->header->adv_router
-		    == from->ospf6_if->area->ospf6->router_id) {
+		if (self_originated) {
+			if (from->ospf6_if->area->ospf6->gr_info
+				    .restart_in_progress) {
+				if (IS_DEBUG_OSPF6_GR)
+					zlog_debug(
+						"Graceful Restart in progress -- not flushing self-originated LSA: %s",
+						new->name);
+				return;
+			}
+
 			/* Self-originated LSA (newer than ours) is received
 			   from
 			   another router. We have to make a new instance of the
@@ -912,11 +1104,18 @@ void ospf6_receive_lsa(struct ospf6_neighbor *from,
 					"Newer instance of the self-originated LSA");
 				zlog_debug("Schedule reorigination");
 			}
-			new->refresh = NULL;
-			thread_add_event(master, ospf6_lsa_refresh, new, 0,
-					 &new->refresh);
+			event_add_event(master, ospf6_lsa_refresh, new, 0,
+					&new->refresh);
 		}
 
+		/* GR: check for network topology change. */
+		struct ospf6 *ospf6 = from->ospf6_if->area->ospf6;
+		struct ospf6_area *area = from->ospf6_if->area;
+		if (ospf6->gr_info.restart_in_progress &&
+		    (new->header->type == ntohs(OSPF6_LSTYPE_ROUTER) ||
+		     new->header->type == ntohs(OSPF6_LSTYPE_NETWORK)))
+			ospf6_gr_check_lsdb_consistency(ospf6, area);
+
 		return;
 	}
 
@@ -926,14 +1125,15 @@ void ospf6_receive_lsa(struct ospf6_neighbor *from,
 		/* if no database copy, should go above state (5) */
 		assert(old);
 
-		if (is_debug) {
-			zlog_debug(
-				"Received is not newer, on the neighbor's request-list");
-			zlog_debug("BadLSReq, discard the received LSA");
-		}
+		zlog_warn(
+			"Received is not newer, on the neighbor %s request-list",
+			from->name);
+		zlog_warn(
+			"BadLSReq, discard the received LSA lsa %s send badLSReq",
+			new->name);
 
 		/* BadLSReq */
-		thread_add_event(master, bad_lsreq, from, 0, NULL);
+		event_add_event(master, bad_lsreq, from, 0, NULL);
 
 		ospf6_lsa_delete(new);
 		return;
@@ -1001,32 +1201,41 @@ void ospf6_receive_lsa(struct ospf6_neighbor *from,
 			 * MAXAGEd and not removed.*/
 			if (OSPF6_LSA_IS_MAXAGE(old)
 			    && !OSPF6_LSA_IS_MAXAGE(new)) {
-
+				if (new->header->adv_router
+				    != from->ospf6_if->area->ospf6->router_id) {
+					if (is_debug)
+						zlog_debug(
+							"%s: Current copy of LSA %s is MAXAGE, but new has recent age, flooding/installing.",
+							__PRETTY_FUNCTION__, old->name);
+					ospf6_lsa_purge(old);
+					ospf6_flood(from, new);
+					ospf6_install_lsa(new);
+					return;
+				}
+				/* For self-originated LSA, only trust
+				 * ourselves. Fall through and send
+				 * LS Update with our current copy.
+				 */
 				if (is_debug)
 					zlog_debug(
-						"%s: Current copy of LSA %s is MAXAGE, but new has recent Age.",
-						old->name, __func__);
-
-				ospf6_lsa_purge(old);
-				if (new->header->adv_router
-				    != from->ospf6_if->area->ospf6->router_id)
-					ospf6_flood(from, new);
-
-				ospf6_install_lsa(new);
-				return;
+						"%s: Current copy of self-originated LSA %s is MAXAGE, but new has recent age, re-sending current one.",
+						__PRETTY_FUNCTION__, old->name);
 			}
 
-			/* XXX, MinLSArrival check !? RFC 2328 13 (8) */
+			/* MinLSArrival check as per RFC 2328 13 (8) */
+			if (ospf6_lsa_check_min_arrival(old, from)) {
+				ospf6_lsa_delete(new);
+				return; /* examin next lsa */
+			}
 
 			ospf6_lsdb_add(ospf6_lsa_copy(old),
 				       from->lsupdate_list);
-			thread_add_event(master, ospf6_lsupdate_send_neighbor,
-					 from, 0, &from->thread_send_lsupdate);
+			event_add_event(master, ospf6_lsupdate_send_neighbor,
+					from, 0, &from->thread_send_lsupdate);
 
 			ospf6_lsa_delete(new);
 			return;
 		}
-		return;
 	}
 }
 
