@@ -1,21 +1,6 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * Copyright (C) 2003 Yasuhiro Ohara
- *
- * This file is part of GNU Zebra.
- *
- * GNU Zebra is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * GNU Zebra is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -28,12 +13,17 @@
 #include "vty.h"
 
 #include "ospf6_proto.h"
+#include "ospf6_area.h"
 #include "ospf6_lsa.h"
 #include "ospf6_lsdb.h"
+#include "ospf6_abr.h"
+#include "ospf6_asbr.h"
 #include "ospf6_route.h"
 #include "ospf6d.h"
 #include "bitfield.h"
 
+DEFINE_MTYPE_STATIC(OSPF6D, OSPF6_LSDB, "OSPF6 LSA database");
+
 struct ospf6_lsdb *ospf6_lsdb_create(void *data)
 {
 	struct ospf6_lsdb *lsdb;
@@ -68,9 +58,9 @@ static void ospf6_lsdb_set_key(struct prefix_ipv6 *key, const void *value,
 #ifdef DEBUG
 static void _lsdb_count_assert(struct ospf6_lsdb *lsdb)
 {
-	struct ospf6_lsa *debug;
+	struct ospf6_lsa *debug, *debugnext;
 	unsigned int num = 0;
-	for (ALL_LSDB(lsdb, debug))
+	for (ALL_LSDB(lsdb, debug, debugnext))
 		num++;
 
 	if (num == lsdb->count)
@@ -78,7 +68,7 @@ static void _lsdb_count_assert(struct ospf6_lsdb *lsdb)
 
 	zlog_debug("PANIC !! lsdb[%p]->count = %d, real = %d", lsdb,
 		   lsdb->count, num);
-	for (ALL_LSDB(lsdb, debug))
+	for (ALL_LSDB(lsdb, debug, debugnext))
 		zlog_debug("%s lsdb[%p]", debug->name, debug->lsdb);
 	zlog_debug("DUMP END");
 
@@ -89,6 +79,16 @@ static void _lsdb_count_assert(struct ospf6_lsdb *lsdb)
 #define ospf6_lsdb_count_assert(t) ((void) 0)
 #endif /*DEBUG*/
 
+static inline void ospf6_lsdb_stats_update(struct ospf6_lsa *lsa,
+					   struct ospf6_lsdb *lsdb, int count)
+{
+	uint16_t stat = ntohs(lsa->header->type) & OSPF6_LSTYPE_FCODE_MASK;
+
+	if (stat >= OSPF6_LSTYPE_SIZE)
+		stat = OSPF6_LSTYPE_UNKNOWN;
+	lsdb->stats[stat] += count;
+}
+
 void ospf6_lsdb_add(struct ospf6_lsa *lsa, struct ospf6_lsdb *lsdb)
 {
 	struct prefix_ipv6 key;
@@ -109,6 +109,7 @@ void ospf6_lsdb_add(struct ospf6_lsa *lsa, struct ospf6_lsdb *lsdb)
 
 	if (!old) {
 		lsdb->count++;
+		ospf6_lsdb_stats_update(lsa, lsdb, 1);
 
 		if (OSPF6_LSA_IS_MAXAGE(lsa)) {
 			if (lsdb->hook_remove)
@@ -118,6 +119,8 @@ void ospf6_lsdb_add(struct ospf6_lsa *lsa, struct ospf6_lsdb *lsdb)
 				(*lsdb->hook_add)(lsa);
 		}
 	} else {
+		lsa->retrans_count = old->retrans_count;
+
 		if (OSPF6_LSA_IS_CHANGED(old, lsa)) {
 			if (OSPF6_LSA_IS_MAXAGE(lsa)) {
 				if (lsdb->hook_remove) {
@@ -136,7 +139,7 @@ void ospf6_lsdb_add(struct ospf6_lsa *lsa, struct ospf6_lsdb *lsdb)
 		}
 		/* to free the lookup lock in node get*/
 		route_unlock_node(current);
-		ospf6_lsa_unlock(old);
+		ospf6_lsa_unlock(&old);
 	}
 
 	ospf6_lsdb_count_assert(lsdb);
@@ -158,13 +161,14 @@ void ospf6_lsdb_remove(struct ospf6_lsa *lsa, struct ospf6_lsdb *lsdb)
 
 	node->info = NULL;
 	lsdb->count--;
+	ospf6_lsdb_stats_update(lsa, lsdb, -1);
 
 	if (lsdb->hook_remove)
 		(*lsdb->hook_remove)(lsa);
 
 	route_unlock_node(node); /* to free the lookup lock */
 	route_unlock_node(node); /* to free the original lock */
-	ospf6_lsa_unlock(lsa);
+	ospf6_lsa_unlock(&lsa);
 
 	ospf6_lsdb_count_assert(lsdb);
 }
@@ -192,6 +196,54 @@ struct ospf6_lsa *ospf6_lsdb_lookup(uint16_t type, uint32_t id,
 	return (struct ospf6_lsa *)node->info;
 }
 
+struct ospf6_lsa *ospf6_find_external_lsa(struct ospf6 *ospf6, struct prefix *p)
+{
+	struct ospf6_route *match;
+	struct ospf6_lsa *lsa;
+	struct ospf6_external_info *info;
+
+	match = ospf6_route_lookup(p, ospf6->external_table);
+	if (match == NULL) {
+		if (IS_OSPF6_DEBUG_ASBR)
+			zlog_debug("No such route %pFX to withdraw", p);
+
+		return NULL;
+	}
+
+	info = match->route_option;
+	assert(info);
+
+	lsa = ospf6_lsdb_lookup(htons(OSPF6_LSTYPE_AS_EXTERNAL),
+				htonl(info->id), ospf6->router_id, ospf6->lsdb);
+	return lsa;
+}
+
+struct ospf6_lsa *ospf6_find_inter_prefix_lsa(struct ospf6 *ospf6,
+					      struct ospf6_area *area,
+					      struct prefix *p)
+{
+	struct ospf6_lsa *lsa;
+	uint16_t type = htons(OSPF6_LSTYPE_INTER_PREFIX);
+
+	for (ALL_LSDB_TYPED_ADVRTR(area->lsdb, type, ospf6->router_id, lsa)) {
+		struct ospf6_inter_prefix_lsa *prefix_lsa;
+		struct prefix prefix;
+
+		prefix_lsa = (struct ospf6_inter_prefix_lsa *)
+			ospf6_lsa_header_end(lsa->header);
+		prefix.family = AF_INET6;
+		prefix.prefixlen = prefix_lsa->prefix.prefix_length;
+		ospf6_prefix_in6_addr(&prefix.u.prefix6, prefix_lsa,
+				      &prefix_lsa->prefix);
+		if (prefix_same(p, &prefix)) {
+			ospf6_lsa_unlock(&lsa);
+			return lsa;
+		}
+	}
+
+	return NULL;
+}
+
 struct ospf6_lsa *ospf6_lsdb_lookup_next(uint16_t type, uint32_t id,
 					 uint32_t adv_router,
 					 struct ospf6_lsdb *lsdb)
@@ -207,11 +259,7 @@ struct ospf6_lsa *ospf6_lsdb_lookup_next(uint16_t type, uint32_t id,
 	ospf6_lsdb_set_key(&key, &adv_router, sizeof(adv_router));
 	ospf6_lsdb_set_key(&key, &id, sizeof(id));
 
-	{
-		char buf[PREFIX2STR_BUFFER];
-		prefix2str(&key, buf, sizeof(buf));
-		zlog_debug("lsdb_lookup_next: key: %s", buf);
-	}
+	zlog_debug("lsdb_lookup_next: key: %pFX", &key);
 
 	node = route_table_get_next(lsdb->table, &key);
 
@@ -279,7 +327,7 @@ struct ospf6_lsa *ospf6_lsdb_next(const struct route_node *iterend,
 {
 	struct route_node *node = lsa->rn;
 
-	ospf6_lsa_unlock(lsa);
+	ospf6_lsa_unlock(&lsa);
 
 	do
 		node = route_next_until(node, iterend);
@@ -298,17 +346,13 @@ struct ospf6_lsa *ospf6_lsdb_next(const struct route_node *iterend,
 
 void ospf6_lsdb_remove_all(struct ospf6_lsdb *lsdb)
 {
-	struct ospf6_lsa *lsa, *lsa_next;
-	const struct route_node *iterend;
+	struct ospf6_lsa *lsa, *lsanext;
 
 	if (lsdb == NULL)
 		return;
 
-	for (iterend = ospf6_lsdb_head(lsdb, 0, 0, 0, &lsa); lsa;
-	     lsa = lsa_next) {
-		lsa_next = ospf6_lsdb_next(iterend, lsa);
+	for (ALL_LSDB(lsdb, lsa, lsanext))
 		ospf6_lsdb_remove(lsa, lsdb);
-	}
 }
 
 void ospf6_lsdb_lsa_unlock(struct ospf6_lsa *lsa)
@@ -316,22 +360,27 @@ void ospf6_lsdb_lsa_unlock(struct ospf6_lsa *lsa)
 	if (lsa != NULL) {
 		if (lsa->rn != NULL)
 			route_unlock_node(lsa->rn);
-		ospf6_lsa_unlock(lsa);
+		ospf6_lsa_unlock(&lsa);
 	}
 }
 
 int ospf6_lsdb_maxage_remover(struct ospf6_lsdb *lsdb)
 {
 	int reschedule = 0;
-	struct ospf6_lsa *lsa, *lsa_next;
-	const struct route_node *iterend;
+	struct ospf6_lsa *lsa, *lsanext;
 
-	for (iterend = ospf6_lsdb_head(lsdb, 0, 0, 0, &lsa); lsa;
-	     lsa = lsa_next) {
-		lsa_next = ospf6_lsdb_next(iterend, lsa);
-		if (!OSPF6_LSA_IS_MAXAGE(lsa))
+	for (ALL_LSDB(lsdb, lsa, lsanext)) {
+		if (!OSPF6_LSA_IS_MAXAGE(lsa)) {
+			if (IS_OSPF6_DEBUG_LSA_TYPE(lsa->header->type))
+				zlog_debug("Not MaxAge %s", lsa->name);
 			continue;
+		}
+
 		if (lsa->retrans_count != 0) {
+			if (IS_OSPF6_DEBUG_LSA_TYPE(lsa->header->type))
+				zlog_debug("Remove MaxAge %s retrans_count %d",
+					   lsa->name, lsa->retrans_count);
+
 			reschedule = 1;
 			continue;
 		}
@@ -347,9 +396,10 @@ int ospf6_lsdb_maxage_remover(struct ospf6_lsdb *lsdb)
 				htonl(OSPF_MAX_SEQUENCE_NUMBER + 1);
 			ospf6_lsa_checksum(lsa->header);
 
-			THREAD_OFF(lsa->refresh);
-			thread_execute(master, ospf6_lsa_refresh, lsa, 0);
+			EVENT_OFF(lsa->refresh);
+			event_execute(master, ospf6_lsa_refresh, lsa, 0, NULL);
 		} else {
+			zlog_debug("calling ospf6_lsdb_remove %s", lsa->name);
 			ospf6_lsdb_remove(lsa, lsdb);
 		}
 	}
@@ -357,55 +407,6 @@ int ospf6_lsdb_maxage_remover(struct ospf6_lsdb *lsdb)
 	return (reschedule);
 }
 
-void ospf6_lsdb_show(struct vty *vty, enum ospf_lsdb_show_level level,
-		     uint16_t *type, uint32_t *id, uint32_t *adv_router,
-		     struct ospf6_lsdb *lsdb)
-{
-	struct ospf6_lsa *lsa;
-	const struct route_node *end = NULL;
-	void (*showfunc)(struct vty *, struct ospf6_lsa *) = NULL;
-
-	switch (level) {
-	case OSPF6_LSDB_SHOW_LEVEL_DETAIL:
-		showfunc = ospf6_lsa_show;
-		break;
-	case OSPF6_LSDB_SHOW_LEVEL_INTERNAL:
-		showfunc = ospf6_lsa_show_internal;
-		break;
-	case OSPF6_LSDB_SHOW_LEVEL_DUMP:
-		showfunc = ospf6_lsa_show_dump;
-		break;
-	case OSPF6_LSDB_SHOW_LEVEL_NORMAL:
-	default:
-		showfunc = ospf6_lsa_show_summary;
-	}
-
-	if (type && id && adv_router) {
-		lsa = ospf6_lsdb_lookup(*type, *id, *adv_router, lsdb);
-		if (lsa) {
-			if (level == OSPF6_LSDB_SHOW_LEVEL_NORMAL)
-				ospf6_lsa_show(vty, lsa);
-			else
-				(*showfunc)(vty, lsa);
-		}
-		return;
-	}
-
-	if (level == OSPF6_LSDB_SHOW_LEVEL_NORMAL)
-		ospf6_lsa_show_summary_header(vty);
-
-	end = ospf6_lsdb_head(lsdb, !!type + !!(type && adv_router),
-			      type ? *type : 0, adv_router ? *adv_router : 0,
-			      &lsa);
-	while (lsa) {
-		if ((!adv_router || lsa->header->adv_router == *adv_router)
-		    && (!id || lsa->header->id == *id))
-			(*showfunc)(vty, lsa);
-
-		lsa = ospf6_lsdb_next(end, lsa);
-	}
-}
-
 uint32_t ospf6_new_ls_id(uint16_t type, uint32_t adv_router,
 			 struct ospf6_lsdb *lsdb)
 {
