@@ -221,12 +221,12 @@ void ospf6_lsupdate_print(struct ospf6_header *oh, int action)
 	     && action == OSPF6_ACTION_RECV)
 	    || (IS_OSPF6_DEBUG_MESSAGE(oh->type, SEND)
 		&& action == OSPF6_ACTION_SEND)) {
-
-		for (p = (char *)((caddr_t)lsupdate
-				  + sizeof(struct ospf6_lsupdate));
-		     p < OSPF6_MESSAGE_END(oh)
-		     && p + OSPF6_LSA_SIZE(p) <= OSPF6_MESSAGE_END(oh);
-		     p += OSPF6_LSA_SIZE(p)) {
+		for (p = (char *)((caddr_t)lsupdate +
+				  sizeof(struct ospf6_lsupdate));
+		     p < OSPF6_MESSAGE_END(oh) &&
+		     p + ospf6_lsa_size((struct ospf6_lsa_header *)p) <=
+			     OSPF6_MESSAGE_END(oh);
+		     p += ospf6_lsa_size((struct ospf6_lsa_header *)p)) {
 			ospf6_lsa_header_print_raw(
 				(struct ospf6_lsa_header *)p);
 		}
@@ -268,6 +268,18 @@ static struct ospf6_packet *ospf6_packet_new(size_t size)
 	return new;
 }
 
+static struct ospf6_packet *ospf6_packet_dup(struct ospf6_packet *old)
+{
+	struct ospf6_packet *new;
+
+	new = XCALLOC(MTYPE_OSPF6_PACKET, sizeof(struct ospf6_packet));
+	new->s = stream_dup(old->s);
+	new->dst = old->dst;
+	new->length = old->length;
+
+	return new;
+}
+
 static void ospf6_packet_free(struct ospf6_packet *op)
 {
 	if (op->s)
@@ -407,6 +419,25 @@ static void ospf6_hello_recv(struct in6_addr *src, struct in6_addr *dst,
 	hello = (struct ospf6_hello *)((caddr_t)oh
 				       + sizeof(struct ospf6_header));
 
+	if ((oi->state == OSPF6_INTERFACE_POINTTOPOINT
+	     || oi->state == OSPF6_INTERFACE_POINTTOMULTIPOINT)
+	    && oi->p2xp_only_cfg_neigh) {
+		/* NEVER, never, ever, do this on broadcast (or NBMA)!
+		 * DR/BDR election requires everyone to talk to everyone else
+		 * only for PtP/PtMP we can be selective in adjacencies!
+		 */
+		struct ospf6_if_p2xp_neighcfg *p2xp_cfg;
+
+		p2xp_cfg = ospf6_if_p2xp_find(oi, src);
+		if (!p2xp_cfg) {
+			if (IS_OSPF6_DEBUG_MESSAGE(oh->type, RECV_HDR))
+				zlog_debug(
+					"ignoring PtP/PtMP hello from %pI6, neighbor not configured",
+					src);
+			return;
+		}
+	}
+
 	/* HelloInterval check */
 	if (ntohs(hello->hello_interval) != oi->hello_interval) {
 		zlog_warn(
@@ -479,7 +510,7 @@ static void ospf6_hello_recv(struct in6_addr *src, struct in6_addr *dst,
 	on->hello_in++;
 
 	/* Always override neighbor's source address */
-	memcpy(&on->linklocal_addr, src, sizeof(struct in6_addr));
+	ospf6_neighbor_lladdr_set(on, src);
 
 	/* Neighbor ifindex check */
 	if (on->ifindex != (ifindex_t)ntohl(hello->interface_id)) {
@@ -631,8 +662,8 @@ static void ospf6_dbdesc_recv_master(struct ospf6_header *oh,
 					"Neighbor state is not ExStart, ignore");
 			return;
 		}
-	/* else fall through to ExStart */
-	/* fallthru */
+		/* else fall through to ExStart */
+		fallthrough;
 	case OSPF6_NEIGHBOR_EXSTART:
 		/* if neighbor obeys us as our slave, schedule negotiation_done
 		   and process LSA Headers. Otherwise, ignore this message */
@@ -650,8 +681,8 @@ static void ospf6_dbdesc_recv_master(struct ospf6_header *oh,
 				  on->ospf6_if->interface->vrf->name, on->name);
 			return;
 		}
-	/* fall through to exchange */
-
+		/* fall through to exchange */
+		fallthrough;
 	case OSPF6_NEIGHBOR_EXCHANGE:
 		if (!memcmp(dbdesc, &on->dbdesc_last,
 			    sizeof(struct ospf6_dbdesc))) {
@@ -835,8 +866,8 @@ static void ospf6_dbdesc_recv_slave(struct ospf6_header *oh,
 					"Neighbor state is not ExStart, ignore");
 			return;
 		}
-	/* else fall through to ExStart */
-	/* fallthru */
+		/* else fall through to ExStart */
+		fallthrough;
 	case OSPF6_NEIGHBOR_EXSTART:
 		/* If the neighbor is Master, act as Slave. Schedule
 		   negotiation_done
@@ -1383,7 +1414,7 @@ ospf6_lsaseq_examin(struct ospf6_lsa_header *lsah, /* start of buffered data */
 			return MSG_NG;
 		}
 		/* save on ntohs() calls here and in the LSA validator */
-		lsalen = OSPF6_LSA_SIZE(lsah);
+		lsalen = ospf6_lsa_size(lsah);
 		if (lsalen < OSPF6_LSA_HEADER_SIZE) {
 			zlog_warn(
 				"%s: malformed LSA header #%u, declared length is %u B",
@@ -1615,9 +1646,10 @@ static void ospf6_lsupdate_recv(struct in6_addr *src, struct in6_addr *dst,
 
 	/* Process LSAs */
 	for (p = (char *)((caddr_t)lsupdate + sizeof(struct ospf6_lsupdate));
-	     p < OSPF6_MESSAGE_END(oh)
-	     && p + OSPF6_LSA_SIZE(p) <= OSPF6_MESSAGE_END(oh);
-	     p += OSPF6_LSA_SIZE(p)) {
+	     p < OSPF6_MESSAGE_END(oh) &&
+	     p + ospf6_lsa_size((struct ospf6_lsa_header *)p) <=
+		     OSPF6_MESSAGE_END(oh);
+	     p += ospf6_lsa_size((struct ospf6_lsa_header *)p)) {
 		ospf6_receive_lsa(on, (struct ospf6_lsa_header *)p);
 	}
 
@@ -2239,8 +2271,6 @@ static void ospf6_write(struct event *thread)
 void ospf6_hello_send(struct event *thread)
 {
 	struct ospf6_interface *oi;
-	struct ospf6_packet *op;
-	uint16_t length = OSPF6_HEADER_SIZE;
 
 	oi = (struct ospf6_interface *)EVENT_ARG(thread);
 
@@ -2266,6 +2296,20 @@ void ospf6_hello_send(struct event *thread)
 		return;
 	}
 
+	event_add_timer(master, ospf6_hello_send, oi, oi->hello_interval,
+			 &oi->thread_send_hello);
+
+	ospf6_hello_send_addr(oi, NULL);
+}
+
+/* used to send polls for PtP/PtMP too */
+void ospf6_hello_send_addr(struct ospf6_interface *oi,
+			   const struct in6_addr *addr)
+{
+	struct ospf6_packet *op;
+	uint16_t length = OSPF6_HEADER_SIZE;
+	bool anything = false;
+
 	op = ospf6_packet_new(oi->ifmtu);
 
 	ospf6_make_header(OSPF6_MESSAGE_TYPE_HELLO, oi, op->s);
@@ -2284,20 +2328,40 @@ void ospf6_hello_send(struct event *thread)
 	/* Set packet length. */
 	op->length = length;
 
-	op->dst = allspfrouters6;
+	if ((oi->state == OSPF6_INTERFACE_POINTTOPOINT
+	     || oi->state == OSPF6_INTERFACE_POINTTOMULTIPOINT)
+	    && !addr && oi->p2xp_no_multicast_hello) {
+		struct listnode *node;
+		struct ospf6_neighbor *on;
+		struct ospf6_packet *opdup;
 
-	ospf6_fill_hdr_checksum(oi, op);
+		for (ALL_LIST_ELEMENTS_RO(oi->neighbor_list, node, on)) {
+			if (on->state < OSPF6_NEIGHBOR_INIT)
+				/* poll-interval for these */
+				continue;
 
-	/* Add packet to the top of the interface output queue, so that they
-	 * can't get delayed by things like long queues of LS Update packets
-	 */
-	ospf6_packet_add_top(oi, op);
+			opdup = ospf6_packet_dup(op);
+			opdup->dst = on->linklocal_addr;
+			ospf6_fill_hdr_checksum(oi, opdup);
+			ospf6_packet_add_top(oi, opdup);
+			anything = true;
+		}
 
-	/* set next thread */
-	event_add_timer(master, ospf6_hello_send, oi, oi->hello_interval,
-			&oi->thread_send_hello);
+		ospf6_packet_free(op);
+	} else {
+		op->dst = addr ? *addr : allspfrouters6;
 
-	OSPF6_MESSAGE_WRITE_ON(oi);
+		/* Add packet to the top of the interface output queue, so that
+		 * they can't get delayed by things like long queues of LS
+		 * Update packets
+		 */
+		ospf6_fill_hdr_checksum(oi, op);
+		ospf6_packet_add_top(oi, op);
+		anything = true;
+	}
+
+	if (anything)
+		OSPF6_MESSAGE_WRITE_ON(oi);
 }
 
 static uint16_t ospf6_make_dbdesc(struct ospf6_neighbor *on, struct stream *s)
@@ -2639,7 +2703,7 @@ static uint16_t ospf6_make_lsupdate_list(struct ospf6_neighbor *on,
 	stream_forward_endp((*op)->s, OSPF6_LS_UPD_MIN_SIZE);
 
 	for (ALL_LSDB(on->lsupdate_list, lsa, lsanext)) {
-		if ((length + OSPF6_LSA_SIZE(lsa->header) + OSPF6_HEADER_SIZE) >
+		if ((length + ospf6_lsa_size(lsa->header) + OSPF6_HEADER_SIZE) >
 		    ospf6_packet_max(on->ospf6_if)) {
 			ospf6_fill_header(on->ospf6_if, (*op)->s,
 					  length + OSPF6_HEADER_SIZE);
@@ -2656,9 +2720,9 @@ static uint16_t ospf6_make_lsupdate_list(struct ospf6_neighbor *on,
 			stream_forward_endp((*op)->s, OSPF6_LS_UPD_MIN_SIZE);
 		}
 		ospf6_lsa_age_update_to_send(lsa, on->ospf6_if->transdelay);
-		stream_put((*op)->s, lsa->header, OSPF6_LSA_SIZE(lsa->header));
+		stream_put((*op)->s, lsa->header, ospf6_lsa_size(lsa->header));
 		(*lsa_cnt)++;
-		length += OSPF6_LSA_SIZE(lsa->header);
+		length += ospf6_lsa_size(lsa->header);
 		assert(lsa->lock == 2);
 		ospf6_lsdb_remove(lsa, on->lsupdate_list);
 	}
@@ -2676,7 +2740,7 @@ static uint16_t ospf6_make_ls_retrans_list(struct ospf6_neighbor *on,
 	stream_forward_endp((*op)->s, OSPF6_LS_UPD_MIN_SIZE);
 
 	for (ALL_LSDB(on->retrans_list, lsa, lsanext)) {
-		if ((length + OSPF6_LSA_SIZE(lsa->header) + OSPF6_HEADER_SIZE) >
+		if ((length + ospf6_lsa_size(lsa->header) + OSPF6_HEADER_SIZE) >
 		    ospf6_packet_max(on->ospf6_if)) {
 			ospf6_fill_header(on->ospf6_if, (*op)->s,
 					  length + OSPF6_HEADER_SIZE);
@@ -2700,9 +2764,9 @@ static uint16_t ospf6_make_ls_retrans_list(struct ospf6_neighbor *on,
 			stream_forward_endp((*op)->s, OSPF6_LS_UPD_MIN_SIZE);
 		}
 		ospf6_lsa_age_update_to_send(lsa, on->ospf6_if->transdelay);
-		stream_put((*op)->s, lsa->header, OSPF6_LSA_SIZE(lsa->header));
+		stream_put((*op)->s, lsa->header, ospf6_lsa_size(lsa->header));
 		(*lsa_cnt)++;
-		length += OSPF6_LSA_SIZE(lsa->header);
+		length += ospf6_lsa_size(lsa->header);
 	}
 	return length;
 }
@@ -2786,9 +2850,9 @@ int ospf6_lsupdate_send_neighbor_now(struct ospf6_neighbor *on,
 	/* skip over fixed header */
 	stream_forward_endp(op->s, OSPF6_LS_UPD_MIN_SIZE);
 	ospf6_lsa_age_update_to_send(lsa, on->ospf6_if->transdelay);
-	stream_put(op->s, lsa->header, OSPF6_LSA_SIZE(lsa->header));
-	length = OSPF6_HEADER_SIZE + OSPF6_LS_UPD_MIN_SIZE
-		 + OSPF6_LSA_SIZE(lsa->header);
+	stream_put(op->s, lsa->header, ospf6_lsa_size(lsa->header));
+	length = OSPF6_HEADER_SIZE + OSPF6_LS_UPD_MIN_SIZE +
+		 ospf6_lsa_size(lsa->header);
 	ospf6_fill_header(on->ospf6_if, op->s, length);
 	ospf6_fill_lsupdate_header(op->s, 1);
 	op->length = length;
@@ -2814,7 +2878,7 @@ static uint16_t ospf6_make_lsupdate_interface(struct ospf6_interface *oi,
 	stream_forward_endp((*op)->s, OSPF6_LS_UPD_MIN_SIZE);
 
 	for (ALL_LSDB(oi->lsupdate_list, lsa, lsanext)) {
-		if (length + OSPF6_LSA_SIZE(lsa->header) + OSPF6_HEADER_SIZE >
+		if (length + ospf6_lsa_size(lsa->header) + OSPF6_HEADER_SIZE >
 		    ospf6_packet_max(oi)) {
 			ospf6_fill_header(oi, (*op)->s,
 					  length + OSPF6_HEADER_SIZE);
@@ -2832,9 +2896,9 @@ static uint16_t ospf6_make_lsupdate_interface(struct ospf6_interface *oi,
 		}
 
 		ospf6_lsa_age_update_to_send(lsa, oi->transdelay);
-		stream_put((*op)->s, lsa->header, OSPF6_LSA_SIZE(lsa->header));
+		stream_put((*op)->s, lsa->header, ospf6_lsa_size(lsa->header));
 		(*lsa_cnt)++;
-		length += OSPF6_LSA_SIZE(lsa->header);
+		length += ospf6_lsa_size(lsa->header);
 
 		assert(lsa->lock == 2);
 		ospf6_lsdb_remove(lsa, oi->lsupdate_list);
