@@ -1,22 +1,9 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * OSPFv3 Not So Stubby Area implementation.
  *
  * Copyright (C) 2021 Kaushik Nath
  * Copyright (C) 2021 Soman K.S
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include <zebra.h>
@@ -26,7 +13,7 @@
 #include "vty.h"
 #include "linklist.h"
 #include "command.h"
-#include "thread.h"
+#include "frrevent.h"
 #include "plist.h"
 #include "filter.h"
 
@@ -65,10 +52,8 @@ static int ospf6_abr_nssa_am_elected(struct ospf6_area *oa)
 
 	/* Verify all the router LSA to compare the router ID */
 	for (ALL_LSDB_TYPED(oa->lsdb, type, lsa)) {
-
-		router_lsa = (struct ospf6_router_lsa
-				      *)((caddr_t)lsa->header
-					 + sizeof(struct ospf6_lsa_header));
+		router_lsa = (struct ospf6_router_lsa *)ospf6_lsa_header_end(
+			lsa->header);
 
 		/* ignore non-ABR routers */
 		if (!CHECK_FLAG(router_lsa->bits, OSPF6_ROUTER_BIT_B))
@@ -429,7 +414,7 @@ static struct ospf6_lsa *ospf6_lsa_translated_nssa_new(struct ospf6_area *area,
 	}
 
 	/* find the translated Type-5 for this Type-7 */
-	nssa = (struct ospf6_as_external_lsa *)OSPF6_LSA_HEADER_END(
+	nssa = (struct ospf6_as_external_lsa *)ospf6_lsa_header_end(
 		type7->header);
 	prefix.family = AF_INET6;
 	prefix.prefixlen = nssa->prefix.prefix_length;
@@ -450,12 +435,10 @@ static struct ospf6_lsa *ospf6_lsa_translated_nssa_new(struct ospf6_area *area,
 	/* prepare buffer */
 	memset(buffer, 0, sizeof(buffer));
 	lsa_header = (struct ospf6_lsa_header *)buffer;
-	extnew = (struct ospf6_as_external_lsa
-			  *)((caddr_t)lsa_header
-			     + sizeof(struct ospf6_lsa_header));
-	ext = (struct ospf6_as_external_lsa
-		       *)((caddr_t)(type7->header)
-			  + sizeof(struct ospf6_lsa_header));
+	extnew = (struct ospf6_as_external_lsa *)ospf6_lsa_header_end(
+		lsa_header);
+	ext = (struct ospf6_as_external_lsa *)ospf6_lsa_header_end(
+		type7->header);
 	old_ptr =
 		(caddr_t)((caddr_t)ext + sizeof(struct ospf6_as_external_lsa));
 	new_ptr = (caddr_t)((caddr_t)extnew
@@ -563,7 +546,7 @@ struct ospf6_lsa *ospf6_translated_nssa_refresh(struct ospf6_area *area,
 				"%s: try to find translated Type-5 LSA for %s",
 				__func__, type7->name);
 
-		ext_lsa = (struct ospf6_as_external_lsa *)OSPF6_LSA_HEADER_END(
+		ext_lsa = (struct ospf6_as_external_lsa *)ospf6_lsa_header_end(
 			type7->header);
 		prefix.family = AF_INET6;
 		prefix.prefixlen = ext_lsa->prefix.prefix_length;
@@ -631,7 +614,7 @@ static void ospf6_abr_translate_nssa(struct ospf6_area *area,
 	struct ospf6 *ospf6;
 
 	ospf6 = area->ospf6;
-	nssa_lsa = (struct ospf6_as_external_lsa *)OSPF6_LSA_HEADER_END(
+	nssa_lsa = (struct ospf6_as_external_lsa *)ospf6_lsa_header_end(
 		lsa->header);
 
 	if (!CHECK_FLAG(nssa_lsa->prefix.prefix_options,
@@ -989,9 +972,9 @@ int ospf6_redistribute_check(struct ospf6 *ospf6, struct ospf6_route *route,
 }
 
 /* This function performs ABR related processing */
-static void ospf6_abr_task_timer(struct thread *thread)
+static void ospf6_abr_task_timer(struct event *thread)
 {
-	struct ospf6 *ospf6 = THREAD_ARG(thread);
+	struct ospf6 *ospf6 = EVENT_ARG(thread);
 
 	if (IS_OSPF6_DEBUG_ABR)
 		zlog_debug("Running ABR task on timer");
@@ -1005,7 +988,7 @@ static void ospf6_abr_task_timer(struct thread *thread)
 
 void ospf6_schedule_abr_task(struct ospf6 *ospf6)
 {
-	if (thread_is_scheduled(ospf6->t_abr_task)) {
+	if (event_is_scheduled(ospf6->t_abr_task)) {
 		if (IS_OSPF6_DEBUG_ABR)
 			zlog_debug("ABR task already scheduled");
 		return;
@@ -1014,8 +997,8 @@ void ospf6_schedule_abr_task(struct ospf6 *ospf6)
 	if (IS_OSPF6_DEBUG_ABR)
 		zlog_debug("Scheduling ABR task");
 
-	thread_add_timer(master, ospf6_abr_task_timer, ospf6,
-			 OSPF6_ABR_TASK_DELAY, &ospf6->t_abr_task);
+	event_add_timer(master, ospf6_abr_task_timer, ospf6,
+			OSPF6_ABR_TASK_DELAY, &ospf6->t_abr_task);
 }
 
 /* Flush the NSSA LSAs from the area */
@@ -1102,9 +1085,9 @@ static void ospf6_ase_lsa_refresh(struct ospf6 *o)
 					route->path.origin.id, o->router_id,
 					o->lsdb);
 		if (old) {
-			THREAD_OFF(old->refresh);
-			thread_add_event(master, ospf6_lsa_refresh, old, 0,
-					 &old->refresh);
+			EVENT_OFF(old->refresh);
+			event_add_event(master, ospf6_lsa_refresh, old, 0,
+					&old->refresh);
 		} else {
 			ospf6_as_external_lsa_originate(route, o);
 		}
@@ -1177,9 +1160,9 @@ void ospf6_area_nssa_update(struct ospf6_area *area)
 						   lsa)) {
 				if (IS_OSPF6_DEBUG_NSSA)
 					ospf6_lsa_header_print(lsa);
-				THREAD_OFF(lsa->refresh);
-				thread_add_event(master, ospf6_lsa_refresh, lsa,
-						 0, &lsa->refresh);
+				EVENT_OFF(lsa->refresh);
+				event_add_event(master, ospf6_lsa_refresh, lsa,
+						0, &lsa->refresh);
 			}
 		}
 	}
@@ -1257,9 +1240,8 @@ void ospf6_nssa_lsa_originate(struct ospf6_route *route,
 	/* prepare buffer */
 	memset(buffer, 0, sizeof(buffer));
 	lsa_header = (struct ospf6_lsa_header *)buffer;
-	as_external_lsa = (struct ospf6_as_external_lsa
-				   *)((caddr_t)lsa_header
-				      + sizeof(struct ospf6_lsa_header));
+	as_external_lsa = (struct ospf6_as_external_lsa *)ospf6_lsa_header_end(
+		lsa_header);
 	p = (caddr_t)((caddr_t)as_external_lsa
 		      + sizeof(struct ospf6_as_external_lsa));
 
@@ -1449,7 +1431,7 @@ DEFPY (no_area_nssa_range,
 		SET_FLAG(range->flag, OSPF6_ROUTE_REMOVE);
 
 		/* Redo summaries if required */
-		thread_execute(master, ospf6_abr_task_timer, ospf6, 0);
+		event_execute(master, ospf6_abr_task_timer, ospf6, 0, NULL);
 	}
 
 	ospf6_route_remove(range, oa->nssa_range_table);
