@@ -1,21 +1,6 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * Copyright (C) 2003 Yasuhiro Ohara
- *
- * This file is part of GNU Zebra.
- *
- * GNU Zebra is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * GNU Zebra is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 /* Shortest Path First calculation for OSPFv3 */
@@ -28,7 +13,7 @@
 #include "vty.h"
 #include "prefix.h"
 #include "linklist.h"
-#include "thread.h"
+#include "frrevent.h"
 #include "lib_errors.h"
 
 #include "ospf6_lsa.h"
@@ -145,10 +130,10 @@ static struct ospf6_vertex *ospf6_vertex_create(struct ospf6_lsa *lsa)
 	v->lsa = lsa;
 
 	/* capability bits + options */
-	v->capability = *(uint8_t *)(OSPF6_LSA_HEADER_END(lsa->header));
-	v->options[0] = *(uint8_t *)(OSPF6_LSA_HEADER_END(lsa->header) + 1);
-	v->options[1] = *(uint8_t *)(OSPF6_LSA_HEADER_END(lsa->header) + 2);
-	v->options[2] = *(uint8_t *)(OSPF6_LSA_HEADER_END(lsa->header) + 3);
+	v->capability = *(uint8_t *)(ospf6_lsa_header_end(lsa->header));
+	v->options[0] = *(uint8_t *)(ospf6_lsa_header_end(lsa->header) + 1);
+	v->options[1] = *(uint8_t *)(ospf6_lsa_header_end(lsa->header) + 2);
+	v->options[2] = *(uint8_t *)(ospf6_lsa_header_end(lsa->header) + 3);
 
 	v->nh_list = list_new();
 	v->nh_list->cmp = (int (*)(void *, void *))ospf6_nexthop_cmp;
@@ -202,7 +187,7 @@ static struct ospf6_lsa *ospf6_lsdesc_lsa(caddr_t lsdesc,
 		inet_ntop(AF_INET, &adv_router, abuf, sizeof(abuf));
 		if (lsa)
 			zlog_debug("  Link to: %s len %u, V %s", lsa->name,
-				   ntohs(lsa->header->length), v->name);
+				   ospf6_lsa_size(lsa->header), v->name);
 		else
 			zlog_debug("  Link to: [%s Id:%s Adv:%s] No LSA , V %s",
 				   ospf6_lstype_name(type), ibuf, abuf,
@@ -221,8 +206,8 @@ static char *ospf6_lsdesc_backlink(struct ospf6_lsa *lsa, caddr_t lsdesc,
 	size = (OSPF6_LSA_IS_TYPE(ROUTER, lsa)
 			? sizeof(struct ospf6_router_lsdesc)
 			: sizeof(struct ospf6_network_lsdesc));
-	for (backlink = OSPF6_LSA_HEADER_END(lsa->header) + 4;
-	     backlink + size <= OSPF6_LSA_END(lsa->header); backlink += size) {
+	for (backlink = ospf6_lsa_header_end(lsa->header) + 4;
+	     backlink + size <= ospf6_lsa_end(lsa->header); backlink += size) {
 		assert(!(OSPF6_LSA_IS_TYPE(NETWORK, lsa)
 			 && VERTEX_IS_TYPE(NETWORK, v)));
 
@@ -305,7 +290,7 @@ static void ospf6_nexthop_calc(struct ospf6_vertex *w, struct ospf6_vertex *v,
 			       != lsa->header->id)
 			continue;
 
-		link_lsa = (struct ospf6_link_lsa *)OSPF6_LSA_HEADER_END(
+		link_lsa = (struct ospf6_link_lsa *)ospf6_lsa_header_end(
 			lsa->header);
 		if (IS_OSPF6_DEBUG_SPF(PROCESS)) {
 			inet_ntop(AF_INET6, &link_lsa->linklocal_addr, buf,
@@ -525,8 +510,8 @@ void ospf6_spf_calculation(uint32_t router_id,
 		size = (VERTEX_IS_TYPE(ROUTER, v)
 				? sizeof(struct ospf6_router_lsdesc)
 				: sizeof(struct ospf6_network_lsdesc));
-		for (lsdesc = OSPF6_LSA_HEADER_END(v->lsa->header) + 4;
-		     lsdesc + size <= OSPF6_LSA_END(v->lsa->header);
+		for (lsdesc = ospf6_lsa_header_end(v->lsa->header) + 4;
+		     lsdesc + size <= ospf6_lsa_end(v->lsa->header);
 		     lsdesc += size) {
 			lsa = ospf6_lsdesc_lsa(lsdesc, v);
 			if (lsa == NULL)
@@ -604,7 +589,7 @@ static void ospf6_spf_log_database(struct ospf6_area *oa)
 	zlog_debug("%s", buffer);
 }
 
-static void ospf6_spf_calculation_thread(struct thread *t)
+static void ospf6_spf_calculation_thread(struct event *t)
 {
 	struct ospf6_area *oa;
 	struct ospf6 *ospf6;
@@ -613,7 +598,7 @@ static void ospf6_spf_calculation_thread(struct thread *t)
 	int areas_processed = 0;
 	char rbuf[32];
 
-	ospf6 = (struct ospf6 *)THREAD_ARG(t);
+	ospf6 = (struct ospf6 *)EVENT_ARG(t);
 
 	/* execute SPF calculation */
 	monotime(&start);
@@ -701,7 +686,7 @@ void ospf6_spf_schedule(struct ospf6 *ospf6, unsigned int reason)
 	}
 
 	/* SPF calculation timer is already scheduled. */
-	if (thread_is_scheduled(ospf6->t_spf_calc)) {
+	if (event_is_scheduled(ospf6->t_spf_calc)) {
 		if (IS_OSPF6_DEBUG_SPF(PROCESS) || IS_OSPF6_DEBUG_SPF(TIME))
 			zlog_debug(
 				"SPF: calculation timer is already scheduled: %p",
@@ -738,9 +723,9 @@ void ospf6_spf_schedule(struct ospf6 *ospf6, unsigned int reason)
 	if (IS_OSPF6_DEBUG_SPF(PROCESS) || IS_OSPF6_DEBUG_SPF(TIME))
 		zlog_debug("SPF: Rescheduling in %ld msec", delay);
 
-	THREAD_OFF(ospf6->t_spf_calc);
-	thread_add_timer_msec(master, ospf6_spf_calculation_thread, ospf6,
-			      delay, &ospf6->t_spf_calc);
+	EVENT_OFF(ospf6->t_spf_calc);
+	event_add_timer_msec(master, ospf6_spf_calculation_thread, ospf6, delay,
+			     &ospf6->t_spf_calc);
 }
 
 void ospf6_spf_display_subtree(struct vty *vty, const char *prefix, int rest,
@@ -1026,7 +1011,7 @@ struct ospf6_lsa *ospf6_create_single_router_lsa(struct ospf6_area *area,
 			continue;
 		}
 		lsa_header = rtr_lsa->header;
-		total_lsa_length += (ntohs(lsa_header->length) - lsa_length);
+		total_lsa_length += (ospf6_lsa_size(lsa_header) - lsa_length);
 		num_lsa++;
 		rtr_lsa = ospf6_lsdb_next(end, rtr_lsa);
 	}
@@ -1059,11 +1044,11 @@ struct ospf6_lsa *ospf6_create_single_router_lsa(struct ospf6_area *area,
 	if (!OSPF6_LSA_IS_MAXAGE(rtr_lsa)) {
 		/* Append first Link State ID LSA */
 		lsa_header = rtr_lsa->header;
-		memcpy(new_header, lsa_header, ntohs(lsa_header->length));
+		memcpy(new_header, lsa_header, ospf6_lsa_size(lsa_header));
 		/* Assign new lsa length as aggregated length. */
 		((struct ospf6_lsa_header *)new_header)->length =
 			htons(total_lsa_length);
-		new_header += ntohs(lsa_header->length);
+		new_header += ospf6_lsa_size(lsa_header);
 		num_lsa--;
 	}
 
@@ -1078,20 +1063,19 @@ struct ospf6_lsa *ospf6_create_single_router_lsa(struct ospf6_area *area,
 		}
 
 		if (IS_OSPF6_DEBUG_SPF(PROCESS)) {
-			lsd = OSPF6_LSA_HEADER_END(rtr_lsa->header) + 4;
+			lsd = ospf6_lsa_header_end(rtr_lsa->header) + 4;
 			interface_id = ROUTER_LSDESC_GET_IFID(lsd);
 			inet_ntop(AF_INET, &interface_id, ifbuf, sizeof(ifbuf));
-			zlog_debug(
-				"%s: Next Router LSA %s to aggreat with len %u interface_id %s",
-				__func__, rtr_lsa->name,
-				ntohs(lsa_header->length), ifbuf);
+			zlog_debug("%s: Next Router LSA %s to aggreat with len %u interface_id %s",
+				   __func__, rtr_lsa->name,
+				   ospf6_lsa_size(lsa_header), ifbuf);
 		}
 
 		/* Append Next Link State ID LSA */
 		lsa_header = rtr_lsa->header;
-		memcpy(new_header, (OSPF6_LSA_HEADER_END(rtr_lsa->header) + 4),
-		       (ntohs(lsa_header->length) - lsa_length));
-		new_header += (ntohs(lsa_header->length) - lsa_length);
+		memcpy(new_header, (ospf6_lsa_header_end(rtr_lsa->header) + 4),
+		       (ospf6_lsa_size(lsa_header) - lsa_length));
+		new_header += (ospf6_lsa_size(lsa_header) - lsa_length);
 		num_lsa--;
 
 		rtr_lsa = ospf6_lsdb_next(end, rtr_lsa);
@@ -1106,8 +1090,8 @@ struct ospf6_lsa *ospf6_create_single_router_lsa(struct ospf6_area *area,
 	if (IS_OSPF6_DEBUG_SPF(PROCESS))
 		zlog_debug("%s: LSA %s id %u type 0%x len %u num_lsa %u",
 			   __func__, lsa->name, ntohl(lsa->header->id),
-			   ntohs(lsa->header->type), ntohs(lsa->header->length),
-			   num_lsa);
+			   ntohs(lsa->header->type),
+			   ospf6_lsa_size(lsa->header), num_lsa);
 
 	return lsa;
 }
@@ -1152,7 +1136,7 @@ int ospf6_ase_calculate_route(struct ospf6 *ospf6, struct ospf6_lsa *lsa,
 		return 0;
 	}
 
-	external = (struct ospf6_as_external_lsa *)OSPF6_LSA_HEADER_END(
+	external = (struct ospf6_as_external_lsa *)ospf6_lsa_header_end(
 		lsa->header);
 	prefix.family = AF_INET6;
 	prefix.prefixlen = external->prefix.prefix_length;
@@ -1242,7 +1226,7 @@ int ospf6_ase_calculate_route(struct ospf6 *ospf6, struct ospf6_lsa *lsa,
 	return 0;
 }
 
-static void ospf6_ase_calculate_timer(struct thread *t)
+static void ospf6_ase_calculate_timer(struct event *t)
 {
 	struct ospf6 *ospf6;
 	struct ospf6_lsa *lsa;
@@ -1250,7 +1234,7 @@ static void ospf6_ase_calculate_timer(struct thread *t)
 	struct ospf6_area *area;
 	uint16_t type;
 
-	ospf6 = THREAD_ARG(t);
+	ospf6 = EVENT_ARG(t);
 
 	/* Calculate external route for each AS-external-LSA */
 	type = htons(OSPF6_LSTYPE_AS_EXTERNAL);
@@ -1277,6 +1261,7 @@ static void ospf6_ase_calculate_timer(struct thread *t)
 		 * no longer valid.
 		 */
 		ospf6_zebra_gr_disable(ospf6);
+		ospf6_zebra_gr_enable(ospf6, ospf6->gr_info.grace_period);
 		ospf6->gr_info.finishing_restart = false;
 	}
 }
@@ -1286,8 +1271,8 @@ void ospf6_ase_calculate_timer_add(struct ospf6 *ospf6)
 	if (ospf6 == NULL)
 		return;
 
-	thread_add_timer(master, ospf6_ase_calculate_timer, ospf6,
-			 OSPF6_ASE_CALC_INTERVAL, &ospf6->t_ase_calc);
+	event_add_timer(master, ospf6_ase_calculate_timer, ospf6,
+			OSPF6_ASE_CALC_INTERVAL, &ospf6->t_ase_calc);
 }
 
 bool ospf6_merge_parents_nh_to_child(struct ospf6_vertex *v,
