@@ -765,8 +765,9 @@ int ospf_flood_through_interface(struct ospf_interface *oi,
 	    packets must be sent, as unicasts, to each adjacent	neighbor
 	    (i.e., those in state Exchange or greater).	 The destination
 	    IP addresses for these packets are the neighbors' IP
-	    addresses.   */
-	if (oi->type == OSPF_IFTYPE_NBMA) {
+	    addresses. This behavior is extended to P2MP networks which
+	    don't support broadcast. */
+	if (OSPF_IF_NON_BROADCAST(oi)) {
 		struct ospf_neighbor *nbr;
 
 		for (rn = route_top(oi->nbrs); rn; rn = route_next(rn)) {
@@ -954,7 +955,7 @@ int ospf_flood_through(struct ospf *ospf, struct ospf_neighbor *inbr,
 
 		if (IS_DEBUG_OSPF_NSSA)
 			zlog_debug("%s: LOCAL NSSA FLOOD of Type-7.", __func__);
-	/* Fallthrough */
+		fallthrough;
 	default:
 		lsa_ack_flag = ospf_flood_through_area(lsa->area, inbr, lsa);
 		break;
