@@ -28,6 +28,7 @@
 #include "libfrr.h"
 #include "routemap.h"
 #include "keychain.h"
+#include "libagentx.h"
 
 #include "ospfd/ospfd.h"
 #include "ospfd/ospf_interface.h"
@@ -44,6 +45,17 @@
 #include "ospfd/ospf_errors.h"
 #include "ospfd/ospf_ldp_sync.h"
 #include "ospfd/ospf_routemap_nb.h"
+#include "ospfd/ospf_apiserver.h"
+
+#define OSPFD_STATE_NAME	 "%s/ospfd.json", frr_libstatedir
+#define OSPFD_INST_STATE_NAME(i) "%s/ospfd-%d.json", frr_runstatedir, i
+
+/* this one includes the path... because the instance number was in the path
+ * before :( ... which totally didn't have a mkdir anywhere.
+ */
+#define OSPFD_COMPAT_STATE_NAME "%s/ospfd-gr.json", frr_libstatedir
+#define OSPFD_COMPAT_INST_STATE_NAME(i)                                        \
+	"%s-%d/ospfd-gr.json", frr_runstatedir, i
 
 /* ospfd privileges */
 zebra_capabilities_t _caps_p[] = {ZCAP_NET_RAW, ZCAP_BIND, ZCAP_NET_ADMIN,
@@ -65,6 +77,7 @@ struct zebra_privs_t ospfd_privs = {
 const struct option longopts[] = {
 	{"instance", required_argument, NULL, 'n'},
 	{"apiserver", no_argument, NULL, 'a'},
+	{"apiserver_addr", required_argument, NULL, 'l'},
 	{0}
 };
 
@@ -73,10 +86,6 @@ const struct option longopts[] = {
 /* Master of threads. */
 struct event_loop *master;
 
-#ifdef SUPPORT_OSPF_API
-extern int ospf_apiserver_enable;
-#endif /* SUPPORT_OSPF_API */
-
 /* SIGHUP handler. */
 static void sighup(void)
 {
@@ -89,6 +98,7 @@ static void sigint(void)
 	zlog_notice("Terminating on signal");
 	bfd_protocol_integration_set_shutdown(true);
 	ospf_terminate();
+
 	exit(0);
 }
 
@@ -123,17 +133,35 @@ static const struct frr_yang_module_info *const ospfd_yang_modules[] = {
 	&frr_route_map_info,
 	&frr_vrf_info,
 	&frr_ospf_route_map_info,
+	&ietf_key_chain_info,
+	&ietf_key_chain_deviation_info,
 };
 
-FRR_DAEMON_INFO(ospfd, OSPF, .vty_port = OSPF_VTY_PORT,
+/* actual paths filled in main() */
+static char state_path[512];
+static char state_compat_path[512];
+static char *state_paths[] = {
+	state_path,
+	state_compat_path,
+	NULL,
+};
 
-		.proghelp = "Implementation of the OSPFv2 routing protocol.",
+/* clang-format off */
+FRR_DAEMON_INFO(ospfd, OSPF,
+	.vty_port = OSPF_VTY_PORT,
+	.proghelp = "Implementation of the OSPFv2 routing protocol.",
 
-		.signals = ospf_signals, .n_signals = array_size(ospf_signals),
+	.signals = ospf_signals,
+	.n_signals = array_size(ospf_signals),
 
-		.privs = &ospfd_privs, .yang_modules = ospfd_yang_modules,
-		.n_yang_modules = array_size(ospfd_yang_modules),
+	.privs = &ospfd_privs,
+
+	.yang_modules = ospfd_yang_modules,
+	.n_yang_modules = array_size(ospfd_yang_modules),
+
+	.state_paths = state_paths,
 );
+/* clang-format on */
 
 /** Max wait time for config to load before accepting hellos */
 #define OSPF_PRE_CONFIG_MAX_WAIT_SECONDS 600
@@ -164,15 +192,11 @@ static void ospf_config_end(void)
 /* OSPFd main routine. */
 int main(int argc, char **argv)
 {
-#ifdef SUPPORT_OSPF_API
-	/* OSPF apiserver is disabled by default. */
-	ospf_apiserver_enable = 0;
-#endif /* SUPPORT_OSPF_API */
-
 	frr_preinit(&ospfd_di, argc, argv);
-	frr_opt_add("n:a", longopts,
+	frr_opt_add("n:al:", longopts,
 		    "  -n, --instance     Set the instance id\n"
-		    "  -a, --apiserver    Enable OSPF apiserver\n");
+		    "  -a, --apiserver    Enable OSPF apiserver\n"
+		    "  -l, --apiserver_addr     Set OSPF apiserver bind address\n");
 
 	while (1) {
 		int opt;
@@ -194,6 +218,14 @@ int main(int argc, char **argv)
 		case 'a':
 			ospf_apiserver_enable = 1;
 			break;
+		case 'l':
+			if (inet_pton(AF_INET, optarg, &ospf_apiserver_addr) <=
+			    0) {
+				zlog_err("OSPF: Invalid API Server IPv4 address %s specified",
+					 optarg);
+				exit(0);
+			}
+			break;
 #endif /* SUPPORT_OSPF_API */
 		default:
 			frr_help_exit(1);
@@ -207,6 +239,17 @@ int main(int argc, char **argv)
 		exit(1);
 	}
 
+	if (ospf_instance) {
+		snprintf(state_path, sizeof(state_path),
+			 OSPFD_INST_STATE_NAME(ospf_instance));
+		snprintf(state_compat_path, sizeof(state_compat_path),
+			 OSPFD_COMPAT_INST_STATE_NAME(ospf_instance));
+	} else {
+		snprintf(state_path, sizeof(state_path), OSPFD_STATE_NAME);
+		snprintf(state_compat_path, sizeof(state_compat_path),
+			 OSPFD_COMPAT_STATE_NAME);
+	}
+
 	/* OSPF master init. */
 	ospf_master_init(frr_init());
 
@@ -214,6 +257,7 @@ int main(int argc, char **argv)
 	master = om->master;
 
 	/* Library inits. */
+	libagentx_init();
 	ospf_debug_init();
 	ospf_vrf_init();
 
