@@ -1487,30 +1487,20 @@ void ospf_zebra_import_default_route(struct ospf *ospf, bool unreg)
 			 __func__);
 }
 
-static int ospf_zebra_import_check_update(ZAPI_CALLBACK_ARGS)
+static void ospf_zebra_import_check_update(struct vrf *vrf, struct prefix *match,
+					   struct zapi_route *nhr)
 {
-	struct ospf *ospf;
-	struct zapi_route nhr;
-	struct prefix matched;
+	struct ospf *ospf = vrf->info;
 
-	ospf = ospf_lookup_by_vrf_id(vrf_id);
 	if (ospf == NULL || !IS_OSPF_ASBR(ospf))
-		return 0;
+		return;
 
-	if (!zapi_nexthop_update_decode(zclient->ibuf, &matched, &nhr)) {
-		zlog_err("%s[%u]: Failure to decode route", __func__,
-			 ospf->vrf_id);
-		return -1;
-	}
+	if (match->family != AF_INET || match->prefixlen != 0 ||
+	    nhr->type == ZEBRA_ROUTE_OSPF)
+		return;
 
-	if (matched.family != AF_INET || matched.prefixlen != 0 ||
-	    nhr.type == ZEBRA_ROUTE_OSPF)
-		return 0;
-
-	ospf->nssa_default_import_check.status = !!nhr.nexthop_num;
+	ospf->nssa_default_import_check.status = !!nhr->nexthop_num;
 	ospf_abr_nssa_type7_defaults(ospf);
-
-	return 0;
 }
 
 int ospf_distribute_list_out_set(struct ospf *ospf, int type, const char *name)
@@ -1779,6 +1769,7 @@ static void ospf_prefix_list_update(struct prefix_list *plist)
 	int type;
 	int abr_inv = 0;
 	struct ospf_area *area;
+	struct ospf_interface *oi;
 	struct listnode *node, *n1;
 
 	/* If OSPF instatnce does not exist, return right now. */
@@ -1834,6 +1825,19 @@ static void ospf_prefix_list_update(struct prefix_list *plist)
 			}
 		}
 
+		/* Update interface neighbor-filter lists. */
+		for (ALL_LIST_ELEMENTS_RO(ospf->oiflist, node, oi)) {
+			if (OSPF_IF_PARAM(oi, nbr_filter_name) &&
+			    strcmp(OSPF_IF_PARAM(oi, nbr_filter_name),
+				   prefix_list_name(plist)) == 0) {
+				oi->nbr_filter = prefix_list_lookup(
+					AFI_IP,
+					OSPF_IF_PARAM(oi, nbr_filter_name));
+				if (oi->nbr_filter)
+					ospf_intf_neighbor_filter_apply(oi);
+			}
+		}
+
 		/* Schedule ABR task. */
 		if (IS_OSPF_ABR(ospf) && abr_inv)
 			ospf_schedule_abr_task(ospf);
@@ -2183,7 +2187,6 @@ static zclient_handler *const ospf_handlers[] = {
 
 	[ZEBRA_REDISTRIBUTE_ROUTE_ADD] = ospf_zebra_read_route,
 	[ZEBRA_REDISTRIBUTE_ROUTE_DEL] = ospf_zebra_read_route,
-	[ZEBRA_NEXTHOP_UPDATE] = ospf_zebra_import_check_update,
 
 	[ZEBRA_OPAQUE_MESSAGE] = ospf_opaque_msg_handler,
 
@@ -2197,11 +2200,10 @@ void ospf_zebra_init(struct event_loop *master, unsigned short instance)
 			      array_size(ospf_handlers));
 	zclient_init(zclient, ZEBRA_ROUTE_OSPF, instance, &ospfd_privs);
 	zclient->zebra_connected = ospf_zebra_connected;
+	zclient->nexthop_update = ospf_zebra_import_check_update;
 
 	/* Initialize special zclient for synchronous message exchanges. */
-	struct zclient_options options = zclient_options_default;
-	options.synchronous = true;
-	zclient_sync = zclient_new(master, &options, NULL, 0);
+	zclient_sync = zclient_new(master, &zclient_options_sync, NULL, 0);
 	zclient_sync->sock = -1;
 	zclient_sync->redist_default = ZEBRA_ROUTE_OSPF;
 	zclient_sync->instance = instance;
