@@ -77,7 +77,7 @@ DEFPY (ipv6_pim_spt_switchover_infinity,
 
 DEFPY (ipv6_pim_spt_switchover_infinity_plist,
        ipv6_pim_spt_switchover_infinity_plist_cmd,
-       "ipv6 pim spt-switchover infinity-and-beyond prefix-list WORD$plist",
+       "ipv6 pim spt-switchover infinity-and-beyond prefix-list PREFIXLIST6_NAME$plist",
        IPV6_STR
        PIM_STR
        "SPT-Switchover\n"
@@ -102,7 +102,7 @@ DEFPY (no_ipv6_pim_spt_switchover_infinity,
 
 DEFPY (no_ipv6_pim_spt_switchover_infinity_plist,
        no_ipv6_pim_spt_switchover_infinity_plist_cmd,
-       "no ipv6 pim spt-switchover infinity-and-beyond prefix-list WORD",
+       "no ipv6 pim spt-switchover infinity-and-beyond prefix-list PREFIXLIST6_NAME",
        NO_STR
        IPV6_STR
        PIM_STR
@@ -244,7 +244,7 @@ DEFPY (interface_no_ipv6_pim,
 
 DEFPY (interface_ipv6_pim_drprio,
        interface_ipv6_pim_drprio_cmd,
-       "ipv6 pim drpriority (1-4294967295)",
+       "ipv6 pim drpriority (0-4294967295)",
        IPV6_STR
        PIM_STR
        "Set the Designated Router Election Priority\n"
@@ -255,7 +255,7 @@ DEFPY (interface_ipv6_pim_drprio,
 
 DEFPY (interface_no_ipv6_pim_drprio,
        interface_no_ipv6_pim_drprio_cmd,
-       "no ipv6 pim drpriority [(1-4294967295)]",
+       "no ipv6 pim drpriority [(0-4294967295)]",
        NO_STR
        IPV6_STR
        PIM_STR
@@ -436,7 +436,7 @@ DEFPY (no_ipv6_pim_rp,
 
 DEFPY (ipv6_pim_rp_prefix_list,
        ipv6_pim_rp_prefix_list_cmd,
-       "ipv6 pim rp X:X::X:X$rp prefix-list WORD$plist",
+       "ipv6 pim rp X:X::X:X$rp prefix-list PREFIXLIST6_NAME$plist",
        IPV6_STR
        PIM_STR
        "Rendezvous Point\n"
@@ -449,7 +449,7 @@ DEFPY (ipv6_pim_rp_prefix_list,
 
 DEFPY (no_ipv6_pim_rp_prefix_list,
        no_ipv6_pim_rp_prefix_list_cmd,
-       "no ipv6 pim rp X:X::X:X$rp prefix-list WORD$plist",
+       "no ipv6 pim rp X:X::X:X$rp prefix-list PREFIXLIST6_NAME$plist",
        NO_STR
        IPV6_STR
        PIM_STR
