@@ -17,6 +17,7 @@
 #include "jhash.h"
 #include "wheel.h"
 #include "network.h"
+#include "frrdistance.h"
 
 #include "pimd.h"
 #include "pim_pim.h"
@@ -303,7 +304,7 @@ static void on_join_timer(struct event *t)
 	}
 
 	/*
-	 * In the case of a HFR we will not ahve anyone to send this to.
+	 * In the case of a FHR we will not ahve anyone to send this to.
 	 */
 	if (PIM_UPSTREAM_FLAG_TEST_FHR(up->flags))
 		return;
@@ -634,6 +635,12 @@ void pim_upstream_update_use_rpt(struct pim_upstream *up,
 	if (pim_addr_is_any(up->sg.src))
 		return;
 
+	/* Ignore RP mapping when the upsteam state
+	 * is NOT Joined on a FHR
+	 */
+	if (up->join_state == PIM_UPSTREAM_NOTJOINED && PIM_UPSTREAM_FLAG_TEST_FHR(up->flags))
+		return;
+
 	old_use_rpt = !!PIM_UPSTREAM_FLAG_TEST_USE_RPT(up->flags);
 
 	/* We will use the SPT (IIF=RPF_interface(S) if -
@@ -908,7 +915,7 @@ static struct pim_upstream *pim_upstream_new(struct pim_instance *pim,
 			 * Set the right RPF so that future changes will
 			 * be right
 			 */
-			rpf_result = pim_rpf_update(pim, up, NULL, __func__);
+			(void)pim_rpf_update(pim, up, NULL, __func__);
 			pim_upstream_keep_alive_timer_start(
 				up, pim->keep_alive_time);
 		}
@@ -1686,6 +1693,8 @@ const char *pim_reg_state2str(enum pim_reg_state reg_state, char *state_str,
 	return state_str;
 }
 
+static void pim_upstream_start_register_stop_timer(struct pim_upstream *up);
+
 static void pim_upstream_register_stop_timer(struct event *t)
 {
 	struct pim_interface *pim_ifp;
@@ -1733,7 +1742,7 @@ static void pim_upstream_register_stop_timer(struct event *t)
 			return;
 		}
 		up->reg_state = PIM_REG_JOIN_PENDING;
-		pim_upstream_start_register_stop_timer(up, 1);
+		pim_upstream_start_register_stop_timer(up);
 
 		if (((up->channel_oil->cc.lastused / 100)
 		     > pim->keep_alive_time)
@@ -1751,34 +1760,59 @@ static void pim_upstream_register_stop_timer(struct event *t)
 	}
 }
 
-void pim_upstream_start_register_stop_timer(struct pim_upstream *up,
-					    int null_register)
+static void pim_upstream_start_register_stop_timer(struct pim_upstream *up)
 {
 	uint32_t time;
 
 	EVENT_OFF(up->t_rs_timer);
 
-	if (!null_register) {
-		uint32_t lower = (0.5 * router->register_suppress_time);
-		uint32_t upper = (1.5 * router->register_suppress_time);
-		time = lower + (frr_weak_random() % (upper - lower + 1));
-		/* Make sure we don't wrap around */
-		if (time >= router->register_probe_time)
-			time -= router->register_probe_time;
-		else
-			time = 0;
-	} else
-		time = router->register_probe_time;
+	time = router->register_probe_time;
 
-	if (PIM_DEBUG_PIM_TRACE) {
-		zlog_debug(
-			"%s: (S,G)=%s Starting upstream register stop timer %d",
-			__func__, up->sg_str, time);
-	}
+	if (PIM_DEBUG_PIM_TRACE)
+		zlog_debug("%s: (S,G)=%s Starting upstream register stop timer %d",
+			   __func__, up->sg_str, time);
 	event_add_timer(router->master, pim_upstream_register_stop_timer, up,
 			time, &up->t_rs_timer);
 }
 
+static void pim_upstream_register_probe_timer(struct event *t)
+{
+	struct pim_upstream *up = EVENT_ARG(t);
+
+	if (!up->rpf.source_nexthop.interface ||
+	    !up->rpf.source_nexthop.interface->info) {
+		if (PIM_DEBUG_PIM_REG)
+			zlog_debug("cannot send Null register for %pSG, no path to RP",
+				   &up->sg);
+	} else
+		pim_null_register_send(up);
+
+	pim_upstream_start_register_stop_timer(up);
+}
+
+void pim_upstream_start_register_probe_timer(struct pim_upstream *up)
+{
+	uint32_t time;
+
+	EVENT_OFF(up->t_rs_timer);
+
+	uint32_t lower = (0.5 * router->register_suppress_time);
+	uint32_t upper = (1.5 * router->register_suppress_time);
+	time = lower + (frr_weak_random() % (upper - lower + 1));
+	/* Make sure we don't wrap around */
+	if (time >= router->register_probe_time)
+		time -= router->register_probe_time;
+	else
+		time = 0;
+
+	if (PIM_DEBUG_PIM_TRACE)
+		zlog_debug("%s: (S,G)=%s Starting upstream register stop null probe timer %d",
+			   __func__, up->sg_str, time);
+
+	event_add_timer(router->master, pim_upstream_register_probe_timer, up,
+			time, &up->t_rs_timer);
+}
+
 int pim_upstream_inherited_olist_decide(struct pim_instance *pim,
 					struct pim_upstream *up)
 {
@@ -1943,6 +1977,40 @@ void pim_upstream_terminate(struct pim_instance *pim)
 		wheel_delete(pim->upstream_sg_wheel);
 	pim->upstream_sg_wheel = NULL;
 }
+bool pim_sg_is_reevaluate_oil_req(struct pim_instance *pim,
+				  struct pim_upstream *up)
+{
+	struct pim_interface *pim_ifp = NULL;
+
+	/*
+	 * Attempt to retrieve the PIM interface information if the RPF
+	 * interface is present
+	 */
+	if (up->rpf.source_nexthop.interface) {
+		pim_ifp = up->rpf.source_nexthop.interface->info;
+	} else {
+		if (PIM_DEBUG_PIM_TRACE) {
+			zlog_debug("%s: up %s RPF is not present", __func__,
+				   up->sg_str);
+		}
+	}
+
+	/*
+	 * Determine if a reevaluation of the outgoing interface list (OIL) is
+	 * required. This may be necessary in scenarios such as MSDP where the
+	 * RP role for a group changes from secondary to primary. In such cases,
+	 * SGRpt may receive a prune, resulting in an S,G entry with a NULL OIL.
+	 * The S,G upstream should then inherit the OIL from *,G, which is
+	 * particularly important for VXLAN setups.
+	 */
+	if (up->channel_oil->oil_inherited_rescan ||
+	    (pim_ifp && I_am_RP(pim_ifp->pim, up->sg.grp)) ||
+	    pim_upstream_empty_inherited_olist(up)) {
+		return true;
+	}
+
+	return false;
+}
 
 bool pim_upstream_equal(const void *arg1, const void *arg2)
 {
@@ -2078,7 +2146,7 @@ static void pim_upstream_sg_running(void *arg)
 	 * only doing this at this point in time
 	 * to get us up and working for the moment
 	 */
-	if (up->channel_oil->oil_inherited_rescan) {
+	if (pim_sg_is_reevaluate_oil_req(pim, up)) {
 		if (PIM_DEBUG_TRACE)
 			zlog_debug(
 				"%s: Handling unscanned inherited_olist for %s[%s]",
