@@ -19,6 +19,7 @@
 #include "libfrr.h"
 #include "routing_nb.h"
 #include "northbound_cli.h"
+#include "frrdistance.h"
 
 #include "static_vrf.h"
 #include "static_vty.h"
@@ -59,8 +60,6 @@ struct static_route_args {
 	bool bfd_multi_hop;
 	const char *bfd_source;
 	const char *bfd_profile;
-
-	const char *input;
 };
 
 static int static_route_nb_run(struct vty *vty, struct static_route_args *args)
@@ -100,7 +99,7 @@ static int static_route_nb_run(struct vty *vty, struct static_route_args *args)
 			return CMD_WARNING_CONFIG_FAILED;
 		}
 
-		args->vrf = yang_dnode_get_string(vrf_dnode, "./name");
+		args->vrf = yang_dnode_get_string(vrf_dnode, "name");
 	} else {
 		if (args->vrf == NULL)
 			args->vrf = VRF_DEFAULT_NAME;
@@ -152,20 +151,9 @@ static int static_route_nb_run(struct vty *vty, struct static_route_args *args)
 	else
 		buf_gate_str = "";
 
-	if (args->gateway == NULL && args->interface_name == NULL) {
+	if (args->gateway == NULL && args->interface_name == NULL)
 		type = STATIC_BLACKHOLE;
-		/* If this is blackhole/reject flagged route, then
-		 * specify interface_name with the value of what was really
-		 * entered.
-		 * interface_name will be validated later in NB functions
-		 * to check if we don't create blackhole/reject routes that
-		 * match the real interface names.
-		 * E.g.: `ip route 10.0.0.1/32 bla` will create a blackhole
-		 * route despite the real interface named `bla` exists.
-		 */
-		if (args->input)
-			args->interface_name = args->input;
-	} else if (args->gateway && args->interface_name) {
+	else if (args->gateway && args->interface_name) {
 		if (args->afi == AFI_IP)
 			type = STATIC_IPV4_GATEWAY_IFNAME;
 		else
@@ -552,8 +540,6 @@ DEFPY_YANG(ip_route_blackhole,
       "Table to configure\n"
       "The table number to configure\n")
 {
-	int idx_flag = 0;
-
 	struct static_route_args args = {
 		.delete = !!no,
 		.afi = AFI_IP,
@@ -568,9 +554,6 @@ DEFPY_YANG(ip_route_blackhole,
 		.vrf = vrf,
 	};
 
-	if (flag && argv_find(argv, argc, flag, &idx_flag))
-		args.input = argv[idx_flag]->arg;
-
 	return static_route_nb_run(vty, &args);
 }
 
@@ -599,8 +582,6 @@ DEFPY_YANG(ip_route_blackhole_vrf,
       "Table to configure\n"
       "The table number to configure\n")
 {
-	int idx_flag = 0;
-
 	struct static_route_args args = {
 		.delete = !!no,
 		.afi = AFI_IP,
@@ -622,9 +603,6 @@ DEFPY_YANG(ip_route_blackhole_vrf,
 	 */
 	assert(args.prefix);
 
-	if (flag && argv_find(argv, argc, flag, &idx_flag))
-		args.input = argv[idx_flag]->arg;
-
 	return static_route_nb_run(vty, &args);
 }
 
@@ -915,8 +893,6 @@ DEFPY_YANG(ipv6_route_blackhole,
       "Table to configure\n"
       "The table number to configure\n")
 {
-	int idx_flag = 0;
-
 	struct static_route_args args = {
 		.delete = !!no,
 		.afi = AFI_IP6,
@@ -931,9 +907,6 @@ DEFPY_YANG(ipv6_route_blackhole,
 		.vrf = vrf,
 	};
 
-	if (flag && argv_find(argv, argc, flag, &idx_flag))
-		args.input = argv[idx_flag]->arg;
-
 	return static_route_nb_run(vty, &args);
 }
 
@@ -962,8 +935,6 @@ DEFPY_YANG(ipv6_route_blackhole_vrf,
       "Table to configure\n"
       "The table number to configure\n")
 {
-	int idx_flag = 0;
-
 	struct static_route_args args = {
 		.delete = !!no,
 		.afi = AFI_IP6,
@@ -985,9 +956,6 @@ DEFPY_YANG(ipv6_route_blackhole_vrf,
 	 */
 	assert(args.prefix);
 
-	if (flag && argv_find(argv, argc, flag, &idx_flag))
-		args.input = argv[idx_flag]->arg;
-
 	return static_route_nb_run(vty, &args);
 }
 
@@ -1233,8 +1201,10 @@ DEFPY_YANG(ipv6_route_vrf, ipv6_route_vrf_cmd,
 	return static_route_nb_run(vty, &args);
 }
 
-void static_cli_show(struct vty *vty, const struct lyd_node *dnode,
-		     bool show_defaults)
+#ifdef INCLUDE_MGMTD_CMDDEFS_ONLY
+
+static void static_cli_show(struct vty *vty, const struct lyd_node *dnode,
+			    bool show_defaults)
 {
 	const char *vrf;
 
@@ -1243,7 +1213,7 @@ void static_cli_show(struct vty *vty, const struct lyd_node *dnode,
 		vty_out(vty, "vrf %s\n", vrf);
 }
 
-void static_cli_show_end(struct vty *vty, const struct lyd_node *dnode)
+static void static_cli_show_end(struct vty *vty, const struct lyd_node *dnode)
 {
 	const char *vrf;
 
@@ -1261,13 +1231,13 @@ static int mpls_label_iter_cb(const struct lyd_node *dnode, void *arg)
 {
 	struct mpls_label_iter *iter = arg;
 
-	if (yang_dnode_exists(dnode, "./label")) {
+	if (yang_dnode_exists(dnode, "label")) {
 		if (iter->first)
 			vty_out(iter->vty, " label %s",
-				yang_dnode_get_string(dnode, "./label"));
+				yang_dnode_get_string(dnode, "label"));
 		else
 			vty_out(iter->vty, "/%s",
-				yang_dnode_get_string(dnode, "./label"));
+				yang_dnode_get_string(dnode, "label"));
 		iter->first = false;
 	}
 
@@ -1285,16 +1255,16 @@ static int srv6_seg_iter_cb(const struct lyd_node *dnode, void *arg)
 	char buffer[INET6_ADDRSTRLEN];
 	struct in6_addr cli_seg;
 
-	if (yang_dnode_exists(dnode, "./seg")) {
+	if (yang_dnode_exists(dnode, "seg")) {
 		if (iter->first) {
-			yang_dnode_get_ipv6(&cli_seg, dnode, "./seg");
+			yang_dnode_get_ipv6(&cli_seg, dnode, "seg");
 			if (inet_ntop(AF_INET6, &cli_seg, buffer,
 				      INET6_ADDRSTRLEN) == NULL) {
 				return 1;
 			}
 			vty_out(iter->vty, " segments %s", buffer);
 		} else {
-			yang_dnode_get_ipv6(&cli_seg, dnode, "./seg");
+			yang_dnode_get_ipv6(&cli_seg, dnode, "seg");
 			if (inet_ntop(AF_INET6, &cli_seg, buffer,
 				      INET6_ADDRSTRLEN) == NULL) {
 				return 1;
@@ -1328,7 +1298,7 @@ static void nexthop_cli_show(struct vty *vty, const struct lyd_node *route,
 
 	vrf = yang_dnode_get_string(route, "../../vrf");
 
-	afi_safi = yang_dnode_get_string(route, "./afi-safi");
+	afi_safi = yang_dnode_get_string(route, "afi-safi");
 	yang_afi_safi_identity2value(afi_safi, &afi, &safi);
 
 	if (afi == AFI_IP)
@@ -1343,32 +1313,32 @@ static void nexthop_cli_show(struct vty *vty, const struct lyd_node *route,
 	else
 		vty_out(vty, " mroute");
 
-	vty_out(vty, " %s", yang_dnode_get_string(route, "./prefix"));
+	vty_out(vty, " %s", yang_dnode_get_string(route, "prefix"));
 
 	if (src)
 		vty_out(vty, " from %s",
-			yang_dnode_get_string(src, "./src-prefix"));
+			yang_dnode_get_string(src, "src-prefix"));
 
-	nh_type = yang_dnode_get_enum(nexthop, "./nh-type");
+	nh_type = yang_dnode_get_enum(nexthop, "nh-type");
 	switch (nh_type) {
 	case STATIC_IFNAME:
 		vty_out(vty, " %s",
-			yang_dnode_get_string(nexthop, "./interface"));
+			yang_dnode_get_string(nexthop, "interface"));
 		break;
 	case STATIC_IPV4_GATEWAY:
 	case STATIC_IPV6_GATEWAY:
 		vty_out(vty, " %s",
-			yang_dnode_get_string(nexthop, "./gateway"));
+			yang_dnode_get_string(nexthop, "gateway"));
 		break;
 	case STATIC_IPV4_GATEWAY_IFNAME:
 	case STATIC_IPV6_GATEWAY_IFNAME:
 		vty_out(vty, " %s",
-			yang_dnode_get_string(nexthop, "./gateway"));
+			yang_dnode_get_string(nexthop, "gateway"));
 		vty_out(vty, " %s",
-			yang_dnode_get_string(nexthop, "./interface"));
+			yang_dnode_get_string(nexthop, "interface"));
 		break;
 	case STATIC_BLACKHOLE:
-		bh_type = yang_dnode_get_enum(nexthop, "./bh-type");
+		bh_type = yang_dnode_get_enum(nexthop, "bh-type");
 		switch (bh_type) {
 		case STATIC_BLACKHOLE_DROP:
 			vty_out(vty, " blackhole");
@@ -1383,13 +1353,13 @@ static void nexthop_cli_show(struct vty *vty, const struct lyd_node *route,
 		break;
 	}
 
-	if (yang_dnode_exists(path, "./tag")) {
-		tag = yang_dnode_get_uint32(path, "./tag");
+	if (yang_dnode_exists(path, "tag")) {
+		tag = yang_dnode_get_uint32(path, "tag");
 		if (tag != 0 || show_defaults)
 			vty_out(vty, " tag %" PRIu32, tag);
 	}
 
-	distance = yang_dnode_get_uint8(path, "./distance");
+	distance = yang_dnode_get_uint8(path, "distance");
 	if (distance != ZEBRA_STATIC_DISTANCE_DEFAULT || show_defaults)
 		vty_out(vty, " %" PRIu8, distance);
 
@@ -1403,48 +1373,49 @@ static void nexthop_cli_show(struct vty *vty, const struct lyd_node *route,
 	yang_dnode_iterate(srv6_seg_iter_cb, &seg_iter, nexthop,
 			   "./srv6-segs-stack/entry");
 
-	nexthop_vrf = yang_dnode_get_string(nexthop, "./vrf");
+	nexthop_vrf = yang_dnode_get_string(nexthop, "vrf");
 	if (strcmp(vrf, nexthop_vrf))
 		vty_out(vty, " nexthop-vrf %s", nexthop_vrf);
 
-	table_id = yang_dnode_get_uint32(path, "./table-id");
+	table_id = yang_dnode_get_uint32(path, "table-id");
 	if (table_id || show_defaults)
 		vty_out(vty, " table %" PRIu32, table_id);
 
-	if (yang_dnode_exists(nexthop, "./onlink")) {
-		onlink = yang_dnode_get_bool(nexthop, "./onlink");
+	if (yang_dnode_exists(nexthop, "onlink")) {
+		onlink = yang_dnode_get_bool(nexthop, "onlink");
 		if (onlink)
 			vty_out(vty, " onlink");
 	}
 
-	if (yang_dnode_exists(nexthop, "./srte-color"))
+	if (yang_dnode_exists(nexthop, "srte-color"))
 		vty_out(vty, " color %s",
-			yang_dnode_get_string(nexthop, "./srte-color"));
+			yang_dnode_get_string(nexthop, "srte-color"));
 
-	if (yang_dnode_exists(nexthop, "./bfd-monitoring")) {
+	if (yang_dnode_exists(nexthop, "bfd-monitoring")) {
 		const struct lyd_node *bfd_dnode =
-			yang_dnode_get(nexthop, "./bfd-monitoring");
+			yang_dnode_get(nexthop, "bfd-monitoring");
 
-		if (yang_dnode_get_bool(bfd_dnode, "./multi-hop")) {
+		if (yang_dnode_get_bool(bfd_dnode, "multi-hop")) {
 			vty_out(vty, " bfd multi-hop");
 
-			if (yang_dnode_exists(bfd_dnode, "./source"))
+			if (yang_dnode_exists(bfd_dnode, "source"))
 				vty_out(vty, " source %s",
 					yang_dnode_get_string(bfd_dnode,
 							      "./source"));
 		} else
 			vty_out(vty, " bfd");
 
-		if (yang_dnode_exists(bfd_dnode, "./profile"))
+		if (yang_dnode_exists(bfd_dnode, "profile"))
 			vty_out(vty, " profile %s",
-				yang_dnode_get_string(bfd_dnode, "./profile"));
+				yang_dnode_get_string(bfd_dnode, "profile"));
 	}
 
 	vty_out(vty, "\n");
 }
 
-void static_nexthop_cli_show(struct vty *vty, const struct lyd_node *dnode,
-			     bool show_defaults)
+static void static_nexthop_cli_show(struct vty *vty,
+				    const struct lyd_node *dnode,
+				    bool show_defaults)
 {
 	const struct lyd_node *path = yang_dnode_get_parent(dnode, "path-list");
 	const struct lyd_node *route =
@@ -1453,8 +1424,9 @@ void static_nexthop_cli_show(struct vty *vty, const struct lyd_node *dnode,
 	nexthop_cli_show(vty, route, NULL, path, dnode, show_defaults);
 }
 
-void static_src_nexthop_cli_show(struct vty *vty, const struct lyd_node *dnode,
-				 bool show_defaults)
+static void static_src_nexthop_cli_show(struct vty *vty,
+					const struct lyd_node *dnode,
+					bool show_defaults)
 {
 	const struct lyd_node *path = yang_dnode_get_parent(dnode, "path-list");
 	const struct lyd_node *src = yang_dnode_get_parent(path, "src-list");
@@ -1463,16 +1435,16 @@ void static_src_nexthop_cli_show(struct vty *vty, const struct lyd_node *dnode,
 	nexthop_cli_show(vty, route, src, path, dnode, show_defaults);
 }
 
-int static_nexthop_cli_cmp(const struct lyd_node *dnode1,
-			   const struct lyd_node *dnode2)
+static int static_nexthop_cli_cmp(const struct lyd_node *dnode1,
+				  const struct lyd_node *dnode2)
 {
 	enum static_nh_type nh_type1, nh_type2;
 	struct prefix prefix1, prefix2;
 	const char *vrf1, *vrf2;
 	int ret = 0;
 
-	nh_type1 = yang_dnode_get_enum(dnode1, "./nh-type");
-	nh_type2 = yang_dnode_get_enum(dnode2, "./nh-type");
+	nh_type1 = yang_dnode_get_enum(dnode1, "nh-type");
+	nh_type2 = yang_dnode_get_enum(dnode2, "nh-type");
 
 	if (nh_type1 != nh_type2)
 		return (int)nh_type1 - (int)nh_type2;
@@ -1480,24 +1452,24 @@ int static_nexthop_cli_cmp(const struct lyd_node *dnode1,
 	switch (nh_type1) {
 	case STATIC_IFNAME:
 		ret = if_cmp_name_func(
-			yang_dnode_get_string(dnode1, "./interface"),
-			yang_dnode_get_string(dnode2, "./interface"));
+			yang_dnode_get_string(dnode1, "interface"),
+			yang_dnode_get_string(dnode2, "interface"));
 		break;
 	case STATIC_IPV4_GATEWAY:
 	case STATIC_IPV6_GATEWAY:
-		yang_dnode_get_prefix(&prefix1, dnode1, "./gateway");
-		yang_dnode_get_prefix(&prefix2, dnode2, "./gateway");
+		yang_dnode_get_prefix(&prefix1, dnode1, "gateway");
+		yang_dnode_get_prefix(&prefix2, dnode2, "gateway");
 		ret = prefix_cmp(&prefix1, &prefix2);
 		break;
 	case STATIC_IPV4_GATEWAY_IFNAME:
 	case STATIC_IPV6_GATEWAY_IFNAME:
-		yang_dnode_get_prefix(&prefix1, dnode1, "./gateway");
-		yang_dnode_get_prefix(&prefix2, dnode2, "./gateway");
+		yang_dnode_get_prefix(&prefix1, dnode1, "gateway");
+		yang_dnode_get_prefix(&prefix2, dnode2, "gateway");
 		ret = prefix_cmp(&prefix1, &prefix2);
 		if (!ret)
 			ret = if_cmp_name_func(
-				yang_dnode_get_string(dnode1, "./interface"),
-				yang_dnode_get_string(dnode2, "./interface"));
+				yang_dnode_get_string(dnode1, "interface"),
+				yang_dnode_get_string(dnode2, "interface"));
 		break;
 	case STATIC_BLACKHOLE:
 		/* There's only one blackhole nexthop per route */
@@ -1508,28 +1480,28 @@ int static_nexthop_cli_cmp(const struct lyd_node *dnode1,
 	if (ret)
 		return ret;
 
-	vrf1 = yang_dnode_get_string(dnode1, "./vrf");
+	vrf1 = yang_dnode_get_string(dnode1, "vrf");
 	if (strmatch(vrf1, "default"))
 		vrf1 = "";
-	vrf2 = yang_dnode_get_string(dnode2, "./vrf");
+	vrf2 = yang_dnode_get_string(dnode2, "vrf");
 	if (strmatch(vrf2, "default"))
 		vrf2 = "";
 
 	return if_cmp_name_func(vrf1, vrf2);
 }
 
-int static_route_list_cli_cmp(const struct lyd_node *dnode1,
-			      const struct lyd_node *dnode2)
+static int static_route_list_cli_cmp(const struct lyd_node *dnode1,
+				     const struct lyd_node *dnode2)
 {
 	const char *afi_safi1, *afi_safi2;
 	afi_t afi1, afi2;
 	safi_t safi1, safi2;
 	struct prefix prefix1, prefix2;
 
-	afi_safi1 = yang_dnode_get_string(dnode1, "./afi-safi");
+	afi_safi1 = yang_dnode_get_string(dnode1, "afi-safi");
 	yang_afi_safi_identity2value(afi_safi1, &afi1, &safi1);
 
-	afi_safi2 = yang_dnode_get_string(dnode2, "./afi-safi");
+	afi_safi2 = yang_dnode_get_string(dnode2, "afi-safi");
 	yang_afi_safi_identity2value(afi_safi2, &afi2, &safi2);
 
 	if (afi1 != afi2)
@@ -1538,42 +1510,97 @@ int static_route_list_cli_cmp(const struct lyd_node *dnode1,
 	if (safi1 != safi2)
 		return (int)safi1 - (int)safi2;
 
-	yang_dnode_get_prefix(&prefix1, dnode1, "./prefix");
-	yang_dnode_get_prefix(&prefix2, dnode2, "./prefix");
+	yang_dnode_get_prefix(&prefix1, dnode1, "prefix");
+	yang_dnode_get_prefix(&prefix2, dnode2, "prefix");
 
 	return prefix_cmp(&prefix1, &prefix2);
 }
 
-int static_src_list_cli_cmp(const struct lyd_node *dnode1,
-			    const struct lyd_node *dnode2)
+static int static_src_list_cli_cmp(const struct lyd_node *dnode1,
+				   const struct lyd_node *dnode2)
 {
 	struct prefix prefix1, prefix2;
 
-	yang_dnode_get_prefix(&prefix1, dnode1, "./src-prefix");
-	yang_dnode_get_prefix(&prefix2, dnode2, "./src-prefix");
+	yang_dnode_get_prefix(&prefix1, dnode1, "src-prefix");
+	yang_dnode_get_prefix(&prefix2, dnode2, "src-prefix");
 
 	return prefix_cmp(&prefix1, &prefix2);
 }
 
-int static_path_list_cli_cmp(const struct lyd_node *dnode1,
-			     const struct lyd_node *dnode2)
+static int static_path_list_cli_cmp(const struct lyd_node *dnode1,
+				    const struct lyd_node *dnode2)
 {
 	uint32_t table_id1, table_id2;
 	uint8_t distance1, distance2;
 
-	table_id1 = yang_dnode_get_uint32(dnode1, "./table-id");
-	table_id2 = yang_dnode_get_uint32(dnode2, "./table-id");
+	table_id1 = yang_dnode_get_uint32(dnode1, "table-id");
+	table_id2 = yang_dnode_get_uint32(dnode2, "table-id");
 
 	if (table_id1 != table_id2)
 		return (int)table_id1 - (int)table_id2;
 
-	distance1 = yang_dnode_get_uint8(dnode1, "./distance");
-	distance2 = yang_dnode_get_uint8(dnode2, "./distance");
+	distance1 = yang_dnode_get_uint8(dnode1, "distance");
+	distance2 = yang_dnode_get_uint8(dnode2, "distance");
 
 	return (int)distance1 - (int)distance2;
 }
 
-#ifndef INCLUDE_MGMTD_CMDDEFS_ONLY
+const struct frr_yang_module_info frr_staticd_cli_info = {
+	.name = "frr-staticd",
+	.ignore_cfg_cbs = true,
+	.nodes = {
+		{
+			.xpath = "/frr-routing:routing/control-plane-protocols/control-plane-protocol/frr-staticd:staticd",
+			.cbs = {
+				.cli_show = static_cli_show,
+				.cli_show_end = static_cli_show_end,
+			}
+		},
+		{
+			.xpath = "/frr-routing:routing/control-plane-protocols/control-plane-protocol/frr-staticd:staticd/route-list",
+			.cbs = {
+				.cli_cmp = static_route_list_cli_cmp,
+			}
+		},
+		{
+			.xpath = "/frr-routing:routing/control-plane-protocols/control-plane-protocol/frr-staticd:staticd/route-list/path-list",
+			.cbs = {
+				.cli_cmp = static_path_list_cli_cmp,
+			}
+		},
+		{
+			.xpath = "/frr-routing:routing/control-plane-protocols/control-plane-protocol/frr-staticd:staticd/route-list/path-list/frr-nexthops/nexthop",
+			.cbs = {
+				.cli_show = static_nexthop_cli_show,
+				.cli_cmp = static_nexthop_cli_cmp,
+			}
+		},
+		{
+			.xpath = "/frr-routing:routing/control-plane-protocols/control-plane-protocol/frr-staticd:staticd/route-list/src-list",
+			.cbs = {
+				.cli_cmp = static_src_list_cli_cmp,
+			}
+		},
+		{
+			.xpath = "/frr-routing:routing/control-plane-protocols/control-plane-protocol/frr-staticd:staticd/route-list/src-list/path-list",
+			.cbs = {
+				.cli_cmp = static_path_list_cli_cmp,
+			}
+		},
+		{
+			.xpath = "/frr-routing:routing/control-plane-protocols/control-plane-protocol/frr-staticd:staticd/route-list/src-list/path-list/frr-nexthops/nexthop",
+			.cbs = {
+				.cli_show = static_src_nexthop_cli_show,
+				.cli_cmp = static_nexthop_cli_cmp,
+			}
+		},
+		{
+			.xpath = NULL,
+		},
+	}
+};
+
+#else /* ifdef INCLUDE_MGMTD_CMDDEFS_ONLY */
 
 DEFPY_YANG(debug_staticd, debug_staticd_cmd,
 	   "[no] debug static [{events$events|route$route|bfd$bfd}]",
@@ -1636,8 +1663,7 @@ void static_vty_init(void)
 	install_element(CONFIG_NODE, &debug_staticd_cmd);
 	install_element(ENABLE_NODE, &show_debugging_static_cmd);
 	install_element(ENABLE_NODE, &staticd_show_bfd_routes_cmd);
-#endif /* ifndef INCLUDE_MGMTD_CMDDEFS_ONLY */
-
+#else /* else INCLUDE_MGMTD_CMDDEFS_ONLY */
 	install_element(CONFIG_NODE, &ip_mroute_dist_cmd);
 
 	install_element(CONFIG_NODE, &ip_route_blackhole_cmd);
@@ -1653,6 +1679,9 @@ void static_vty_init(void)
 	install_element(VRF_NODE, &ipv6_route_address_interface_vrf_cmd);
 	install_element(CONFIG_NODE, &ipv6_route_cmd);
 	install_element(VRF_NODE, &ipv6_route_vrf_cmd);
+#endif /* ifndef INCLUDE_MGMTD_CMDDEFS_ONLY */
 
+#ifndef INCLUDE_MGMTD_CMDDEFS_ONLY
 	mgmt_be_client_lib_vty_init();
+#endif
 }
