@@ -18,6 +18,7 @@
 #include "bgpd/bgp_vty.h"
 #include "bgpd/bgp_zebra.h"
 #include "bgpd/bgp_network.h"
+#include "bgpd/bgp_label.h"
 
 #ifdef ENABLE_BGP_VNC
 #include "bgpd/rfapi/rfapi_backend.h"
@@ -261,25 +262,11 @@ static struct test_peer_attr test_peer_attrs[] = {
 		.u.flag = PEER_FLAG_CAPABILITY_ENHE,
 		.type = PEER_AT_GLOBAL_FLAG,
 	},
-	{
-		.cmd = "capability extended-nexthop",
-		.u.flag = PEER_FLAG_CAPABILITY_ENHE,
-		.type = PEER_AT_GLOBAL_FLAG,
-		.o.invert_peer = true,
-		.o.use_iface_peer = true,
-	},
 	{
 		.cmd = "capability software-version",
 		.u.flag = PEER_FLAG_CAPABILITY_SOFT_VERSION,
 		.type = PEER_AT_GLOBAL_FLAG,
 	},
-	{
-		.cmd = "capability software-version",
-		.u.flag = PEER_FLAG_CAPABILITY_SOFT_VERSION,
-		.type = PEER_AT_GLOBAL_FLAG,
-		.o.invert_peer = true,
-		.o.use_iface_peer = true,
-	},
 	{
 		.cmd = "description",
 		.peer_cmd = "description FRR Peer",
@@ -297,9 +284,11 @@ static struct test_peer_attr test_peer_attrs[] = {
 		.type = PEER_AT_GLOBAL_FLAG,
 	},
 	{
-		.cmd = "enforce-first-as",
-		.u.flag = PEER_FLAG_ENFORCE_FIRST_AS,
+		.cmd = "capability fqdn",
+		.u.flag = PEER_FLAG_CAPABILITY_FQDN,
 		.type = PEER_AT_GLOBAL_FLAG,
+		.o.invert_peer = true,
+		.o.invert_group = true,
 	},
 	{
 		.cmd = "local-as",
@@ -1386,6 +1375,7 @@ static void bgp_shutdown(void)
 	bgp_route_finish();
 	bgp_route_map_terminate();
 	bgp_attr_finish();
+	bgp_labels_finish();
 	bgp_pthreads_finish();
 	access_list_add_hook(NULL);
 	access_list_delete_hook(NULL);
