@@ -5,6 +5,7 @@
  */
 
 #include <zebra.h>
+#include <sys/stat.h>
 
 #include "frrevent.h"
 #include "vty.h"
@@ -13,6 +14,7 @@
 #include "lib_vty.h"
 #include "log.h"
 #include "northbound.h"
+#include "northbound_cli.h"
 
 static struct event_loop *master;
 
@@ -199,6 +201,38 @@ static struct yang_data *frr_test_module_vrfs_vrf_routes_route_active_get_elem(
 	return NULL;
 }
 
+/*
+ * XPath: /frr-test-module:frr-test-module/vrfs/vrf/ping
+ */
+static int frr_test_module_vrfs_vrf_ping(struct nb_cb_rpc_args *args)
+{
+	const char *vrf = yang_dnode_get_string(args->input, "../name");
+	const char *data = yang_dnode_get_string(args->input, "data");
+
+	yang_dnode_rpc_output_add(args->output, "vrf", vrf);
+	yang_dnode_rpc_output_add(args->output, "data-out", data);
+
+	return NB_OK;
+}
+
+/*
+ * XPath: /frr-test-module:frr-test-module/c1value
+ */
+static struct yang_data *
+frr_test_module_c1value_get_elem(struct nb_cb_get_elem_args *args)
+{
+	return yang_data_new_uint8(args->xpath, 21);
+}
+
+/*
+ * XPath: /frr-test-module:frr-test-module/c2cont/c2value
+ */
+static struct yang_data *
+frr_test_module_c2cont_c2value_get_elem(struct nb_cb_get_elem_args *args)
+{
+	return yang_data_new_uint32(args->xpath, 0xAB010203);
+}
+
 /* clang-format off */
 const struct frr_yang_module_info frr_test_module_info = {
 	.name = "frr-test-module",
@@ -242,6 +276,18 @@ const struct frr_yang_module_info frr_test_module_info = {
 			.xpath = "/frr-test-module:frr-test-module/vrfs/vrf/routes/route/active",
 			.cbs.get_elem = frr_test_module_vrfs_vrf_routes_route_active_get_elem,
 		},
+		{
+			.xpath = "/frr-test-module:frr-test-module/vrfs/vrf/ping",
+			.cbs.rpc = frr_test_module_vrfs_vrf_ping,
+		},
+		{
+			.xpath = "/frr-test-module:frr-test-module/c1value",
+			.cbs.get_elem = frr_test_module_c1value_get_elem,
+		},
+		{
+			.xpath = "/frr-test-module:frr-test-module/c2cont/c2value",
+			.cbs.get_elem = frr_test_module_c2cont_c2value_get_elem,
+		},
 		{
 			.xpath = NULL,
 		},
@@ -249,6 +295,33 @@ const struct frr_yang_module_info frr_test_module_info = {
 };
 /* clang-format on */
 
+DEFUN(test_rpc, test_rpc_cmd, "test rpc",
+      "Test\n"
+      "RPC\n")
+{
+	struct lyd_node *output = NULL;
+	char xpath[XPATH_MAXLEN];
+	int ret;
+
+	snprintf(xpath, sizeof(xpath),
+		 "/frr-test-module:frr-test-module/vrfs/vrf[name='testname']/ping");
+
+	nb_cli_rpc_enqueue(vty, "data", "testdata");
+
+	ret = nb_cli_rpc(vty, xpath, &output);
+	if (ret != CMD_SUCCESS) {
+		vty_out(vty, "RPC failed\n");
+		return ret;
+	}
+
+	vty_out(vty, "vrf %s data %s\n", yang_dnode_get_string(output, "vrf"),
+		yang_dnode_get_string(output, "data-out"));
+
+	yang_dnode_free(output);
+
+	return CMD_SUCCESS;
+}
+
 static const struct frr_yang_module_info *const modules[] = {
 	&frr_test_module_info,
 };
@@ -388,6 +461,8 @@ int main(int argc, char **argv)
 	lib_cmd_init();
 	nb_init(master, modules, array_size(modules), false);
 
+	install_element(ENABLE_NODE, &test_rpc_cmd);
+
 	/* Create artificial data. */
 	create_data(num_vrfs, num_interfaces, num_routes);
 
