@@ -1,4 +1,5 @@
 #!/usr/bin/env python
+# SPDX-License-Identifier: ISC
 
 #
 # Part of NetDEF Topology Tests
@@ -6,20 +7,6 @@
 # Copyright (c) 2018, LabN Consulting, L.L.C.
 # Authored by Lou Berger <lberger@labn.net>
 #
-# Permission to use, copy, modify, and/or distribute this software
-# for any purpose with or without fee is hereby granted, provided
-# that the above copyright notice and this permission notice appear
-# in all copies.
-#
-# THE SOFTWARE IS PROVIDED "AS IS" AND NETDEF DISCLAIMS ALL WARRANTIES
-# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
-# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL NETDEF BE LIABLE FOR
-# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY
-# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
-# WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
-# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
-# OF THIS SOFTWARE.
-#
 
 import os
 import sys
@@ -36,6 +23,7 @@ from lib import topotest
 from lib.topogen import Topogen, TopoRouter, get_topogen
 from lib.topolog import logger
 from lib.common_config import required_linux_kernel_version
+from lib.checkping import check_ping
 
 pytestmark = [pytest.mark.bgpd]
 
@@ -147,22 +135,6 @@ def open_json_file(filename):
         assert False, "Could not read file {}".format(filename)
 
 
-def check_ping(name, dest_addr, expect_connected):
-    def _check(name, dest_addr, match):
-        tgen = get_topogen()
-        output = tgen.gears[name].run("ping6 {} -c 1 -w 1".format(dest_addr))
-        logger.info(output)
-        if match not in output:
-            return "ping fail"
-
-    match = ", {} packet loss".format("0%" if expect_connected else "100%")
-    logger.info("[+] check {} {} {}".format(name, dest_addr, match))
-    tgen = get_topogen()
-    func = functools.partial(_check, name, dest_addr, match)
-    success, result = topotest.run_and_expect(func, None, count=10, wait=1)
-    assert result is None, "Failed"
-
-
 def check_rib(name, cmd, expected_file):
     def _check(name, cmd, expected_file):
         logger.info("polling")
@@ -195,20 +167,20 @@ def test_rib():
 
 
 def test_ping():
-    check_ping("ce1", "2001:2::2", True)
-    check_ping("ce1", "2001:3::2", True)
-    check_ping("ce1", "2001:4::2", False)
-    check_ping("ce1", "2001:5::2", False)
-    check_ping("ce1", "2001:6::2", False)
-    check_ping("ce4", "2001:1::2", False)
-    check_ping("ce4", "2001:2::2", False)
-    check_ping("ce4", "2001:3::2", False)
-    check_ping("ce4", "2001:5::2", True)
-    check_ping("ce4", "2001:6::2", True)
+    check_ping("ce1", "2001:2::2", True, 10, 1)
+    check_ping("ce1", "2001:3::2", True, 10, 1)
+    check_ping("ce1", "2001:4::2", False, 10, 1)
+    check_ping("ce1", "2001:5::2", False, 10, 1)
+    check_ping("ce1", "2001:6::2", False, 10, 1)
+    check_ping("ce4", "2001:1::2", False, 10, 1)
+    check_ping("ce4", "2001:2::2", False, 10, 1)
+    check_ping("ce4", "2001:3::2", False, 10, 1)
+    check_ping("ce4", "2001:5::2", True, 10, 1)
+    check_ping("ce4", "2001:6::2", True, 10, 1)
 
 
 def test_locator_delete():
-    check_ping("ce1", "2001:2::2", True)
+    check_ping("ce1", "2001:2::2", True, 10, 1)
     get_topogen().gears["r1"].vtysh_cmd(
         """
         configure terminal
@@ -220,11 +192,11 @@ def test_locator_delete():
     )
     check_rib("r1", "show bgp ipv6 vpn json", "r1/vpnv6_rib_locator_deleted.json")
     check_rib("r2", "show bgp ipv6 vpn json", "r2/vpnv6_rib_locator_deleted.json")
-    check_ping("ce1", "2001:2::2", False)
+    check_ping("ce1", "2001:2::2", False, 10, 1)
 
 
 def test_locator_recreate():
-    check_ping("ce1", "2001:2::2", False)
+    check_ping("ce1", "2001:2::2", False, 10, 1)
     get_topogen().gears["r1"].vtysh_cmd(
         """
         configure terminal
@@ -237,11 +209,11 @@ def test_locator_recreate():
     )
     check_rib("r1", "show bgp ipv6 vpn json", "r1/vpnv6_rib_locator_recreated.json")
     check_rib("r2", "show bgp ipv6 vpn json", "r2/vpnv6_rib_locator_recreated.json")
-    check_ping("ce1", "2001:2::2", True)
+    check_ping("ce1", "2001:2::2", True, 10, 1)
 
 
 def test_bgp_locator_unset():
-    check_ping("ce1", "2001:2::2", True)
+    check_ping("ce1", "2001:2::2", True, 10, 1)
     get_topogen().gears["r1"].vtysh_cmd(
         """
         configure terminal
@@ -252,11 +224,11 @@ def test_bgp_locator_unset():
     )
     check_rib("r1", "show bgp ipv6 vpn json", "r1/vpnv6_rib_locator_deleted.json")
     check_rib("r2", "show bgp ipv6 vpn json", "r2/vpnv6_rib_locator_deleted.json")
-    check_ping("ce1", "2001:2::2", False)
+    check_ping("ce1", "2001:2::2", False, 10, 1)
 
 
 def test_bgp_locator_reset():
-    check_ping("ce1", "2001:2::2", False)
+    check_ping("ce1", "2001:2::2", False, 10, 1)
     get_topogen().gears["r1"].vtysh_cmd(
         """
         configure terminal
@@ -267,11 +239,11 @@ def test_bgp_locator_reset():
     )
     check_rib("r1", "show bgp ipv6 vpn json", "r1/vpnv6_rib_locator_recreated.json")
     check_rib("r2", "show bgp ipv6 vpn json", "r2/vpnv6_rib_locator_recreated.json")
-    check_ping("ce1", "2001:2::2", True)
+    check_ping("ce1", "2001:2::2", True, 10, 1)
 
 
 def test_bgp_srv6_unset():
-    check_ping("ce1", "2001:2::2", True)
+    check_ping("ce1", "2001:2::2", True, 10, 1)
     get_topogen().gears["r1"].vtysh_cmd(
         """
         configure terminal
@@ -281,11 +253,11 @@ def test_bgp_srv6_unset():
     )
     check_rib("r1", "show bgp ipv6 vpn json", "r1/vpnv6_rib_locator_deleted.json")
     check_rib("r2", "show bgp ipv6 vpn json", "r2/vpnv6_rib_locator_deleted.json")
-    check_ping("ce1", "2001:2::2", False)
+    check_ping("ce1", "2001:2::2", False, 10, 1)
 
 
 def test_bgp_srv6_reset():
-    check_ping("ce1", "2001:2::2", False)
+    check_ping("ce1", "2001:2::2", False, 10, 1)
     get_topogen().gears["r1"].vtysh_cmd(
         """
         configure terminal
@@ -296,7 +268,7 @@ def test_bgp_srv6_reset():
     )
     check_rib("r1", "show bgp ipv6 vpn json", "r1/vpnv6_rib_locator_recreated.json")
     check_rib("r2", "show bgp ipv6 vpn json", "r2/vpnv6_rib_locator_recreated.json")
-    check_ping("ce1", "2001:2::2", True)
+    check_ping("ce1", "2001:2::2", True, 10, 1)
 
 
 if __name__ == "__main__":
