@@ -851,23 +851,25 @@ def test_dynamic_imported_matching_prefix_based_on_community_list_p0(request):
                 result = verify_bgp_rib(
                     tgen, addr_type, "r3", static_routes, expected=False
                 )
-                assert (
-                    result is not True
-                ), "Testcase {} : Failed \nError {}\n" "Routes {} still in BGP table".format(
-                    tc_name,
-                    result,
-                    static_routes["r3"]["static_routes"][0]["network"],
+                assert result is not True, (
+                    "Testcase {} : Failed \nError {}\n"
+                    "Routes {} still in BGP table".format(
+                        tc_name,
+                        result,
+                        static_routes["r3"]["static_routes"][0]["network"],
+                    )
                 )
 
                 result = verify_rib(
                     tgen, addr_type, "r3", static_routes, expected=False
                 )
-                assert (
-                    result is not True
-                ), "Testcase {} : Failed Error {}" "Routes {} still in Route table".format(
-                    tc_name,
-                    result,
-                    static_routes["r3"]["static_routes"][0]["network"],
+                assert result is not True, (
+                    "Testcase {} : Failed Error {}"
+                    "Routes {} still in Route table".format(
+                        tc_name,
+                        result,
+                        static_routes["r3"]["static_routes"][0]["network"],
+                    )
                 )
             else:
                 result = verify_bgp_rib(tgen, addr_type, "r3", static_routes)
@@ -924,23 +926,25 @@ def test_dynamic_imported_matching_prefix_based_on_community_list_p0(request):
                 result = verify_bgp_rib(
                     tgen, addr_type, "r3", static_routes, expected=False
                 )
-                assert (
-                    result is not True
-                ), "Testcase {} : Failed \nError {}\n" "Routes {} still in BGP table".format(
-                    tc_name,
-                    result,
-                    static_routes["r3"]["static_routes"][0]["network"],
+                assert result is not True, (
+                    "Testcase {} : Failed \nError {}\n"
+                    "Routes {} still in BGP table".format(
+                        tc_name,
+                        result,
+                        static_routes["r3"]["static_routes"][0]["network"],
+                    )
                 )
 
                 result = verify_rib(
                     tgen, addr_type, "r3", static_routes, expected=False
                 )
-                assert (
-                    result is not True
-                ), "Testcase {} : Failed Error {}" "Routes {} still in Route table".format(
-                    tc_name,
-                    result,
-                    static_routes["r3"]["static_routes"][0]["network"],
+                assert result is not True, (
+                    "Testcase {} : Failed Error {}"
+                    "Routes {} still in Route table".format(
+                        tc_name,
+                        result,
+                        static_routes["r3"]["static_routes"][0]["network"],
+                    )
                 )
             else:
                 result = verify_bgp_rib(tgen, addr_type, "r3", static_routes)
@@ -1153,23 +1157,25 @@ def test_dynamic_import_routes_delete_static_route_p1(request):
                 result = verify_bgp_rib(
                     tgen, addr_type, "r2", static_routes, expected=False
                 )
-                assert (
-                    result is not True
-                ), "Testcase {} : Failed \nError {}\n" "Routes {} still in BGP table".format(
-                    tc_name,
-                    result,
-                    static_routes["r2"]["static_routes"][0]["network"],
+                assert result is not True, (
+                    "Testcase {} : Failed \nError {}\n"
+                    "Routes {} still in BGP table".format(
+                        tc_name,
+                        result,
+                        static_routes["r2"]["static_routes"][0]["network"],
+                    )
                 )
 
                 result = verify_rib(
                     tgen, addr_type, "r2", static_routes, expected=False
                 )
-                assert (
-                    result is not True
-                ), "Testcase {} : Failed Error {}" "Routes {} still in Route table".format(
-                    tc_name,
-                    result,
-                    static_routes[dut]["static_routes"][0]["network"],
+                assert result is not True, (
+                    "Testcase {} : Failed Error {}"
+                    "Routes {} still in Route table".format(
+                        tc_name,
+                        result,
+                        static_routes[dut]["static_routes"][0]["network"],
+                    )
                 )
 
     step("Delete static routes from vrf BLUE")
@@ -1209,21 +1215,23 @@ def test_dynamic_import_routes_delete_static_route_p1(request):
                 result = verify_bgp_rib(
                     tgen, addr_type, dut, static_routes, expected=False
                 )
-                assert (
-                    result is not True
-                ), "Testcase {} : Failed \nError {}\n" "Routes {} still in BGP table".format(
-                    tc_name,
-                    result,
-                    static_routes[dut]["static_routes"][0]["network"],
+                assert result is not True, (
+                    "Testcase {} : Failed \nError {}\n"
+                    "Routes {} still in BGP table".format(
+                        tc_name,
+                        result,
+                        static_routes[dut]["static_routes"][0]["network"],
+                    )
                 )
 
                 result = verify_rib(tgen, addr_type, dut, static_routes, expected=False)
-                assert (
-                    result is not True
-                ), "Testcase {} : Failed Error {}" "Routes {} still in Route table".format(
-                    tc_name,
-                    result,
-                    static_routes[dut]["static_routes"][0]["network"],
+                assert result is not True, (
+                    "Testcase {} : Failed Error {}"
+                    "Routes {} still in Route table".format(
+                        tc_name,
+                        result,
+                        static_routes[dut]["static_routes"][0]["network"],
+                    )
                 )
 
     step("Delete static routes from vrf default")
