@@ -1,24 +1,11 @@
 #!/usr/bin/python
+# SPDX-License-Identifier: ISC
 
 #
 # Copyright (c) 2020 by VMware, Inc. ("VMware")
 # Used Copyright (c) 2018 by Network Device Education Foundation, Inc.
 # ("NetDEF") in this file.
 #
-# Permission to use, copy, modify, and/or distribute this software
-# for any purpose with or without fee is hereby granted, provided
-# that the above copyright notice and this permission notice appear
-# in all copies.
-#
-# THE SOFTWARE IS PROVIDED "AS IS" AND VMWARE DISCLAIMS ALL WARRANTIES
-# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
-# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL VMWARE BE LIABLE FOR
-# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY
-# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
-# WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
-# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
-# OF THIS SOFTWARE.
-#
 
 
 """OSPF Basic Functionality Automation."""
@@ -77,7 +64,9 @@ TOPOOLOGY =
 TESTCASES =
 1. Verify ospf authentication with Simple password authentication.
 2. Verify ospf authentication with MD5 authentication.
-3. Verify ospf authentication with different authentication methods.
+3. Verify ospf authentication with MD5 keychain authentication.
+4. Verify ospf authentication with SHA256 keychain authentication.
+5. Verify ospf authentication with different authentication methods.
 
  """
 
@@ -113,7 +102,7 @@ def setup_module(mod):
         pytest.skip(tgen.errors)
 
     ospf_covergence = verify_ospf_neighbor(tgen, topo)
-    assert ospf_covergence is True, "setup_module :Failed \n Error:" " {}".format(
+    assert ospf_covergence is True, "setup_module :Failed \n Error  {}".format(
         ospf_covergence
     )
 
@@ -179,7 +168,7 @@ def test_ospf_authentication_simple_pass_tc28_p1(request):
     step("Verify that the neighbour is not FULL between R1 and R2.")
     dut = "r1"
     ospf_covergence = verify_ospf_neighbor(tgen, topo, dut=dut, expected=False)
-    assert ospf_covergence is not True, "setup_module :Failed \n Error:" " {}".format(
+    assert ospf_covergence is not True, "Testcase Failed \n Error  {}".format(
         ospf_covergence
     )
 
@@ -205,7 +194,7 @@ def test_ospf_authentication_simple_pass_tc28_p1(request):
 
     dut = "r2"
     ospf_covergence = verify_ospf_neighbor(tgen, topo, dut=dut)
-    assert ospf_covergence is True, "setup_module :Failed \n Error:" " {}".format(
+    assert ospf_covergence is True, "Testcase Failed \n Error  {}".format(
         ospf_covergence
     )
 
@@ -236,7 +225,7 @@ def test_ospf_authentication_simple_pass_tc28_p1(request):
     ospf_covergence = verify_ospf_neighbor(
         tgen, topo, dut=dut, expected=False, retry_timeout=10
     )
-    assert ospf_covergence is not True, "setup_module :Failed \n Error:" " {}".format(
+    assert ospf_covergence is not True, "Testcase Failed \n Error  {}".format(
         ospf_covergence
     )
 
@@ -258,7 +247,7 @@ def test_ospf_authentication_simple_pass_tc28_p1(request):
 
     dut = "r2"
     ospf_covergence = verify_ospf_neighbor(tgen, topo, dut=dut)
-    assert ospf_covergence is True, "setup_module :Failed \n Error:" " {}".format(
+    assert ospf_covergence is True, "Testcase Failed \n Error  {}".format(
         ospf_covergence
     )
 
@@ -273,7 +262,7 @@ def test_ospf_authentication_simple_pass_tc28_p1(request):
         "show ip ospf neighbor cmd."
     )
     ospf_covergence = verify_ospf_neighbor(tgen, topo, dut=dut, expected=False)
-    assert ospf_covergence is not True, "setup_module :Failed \n Error:" " {}".format(
+    assert ospf_covergence is not True, "Testcase Failed \n Error  {}".format(
         ospf_covergence
     )
 
@@ -287,7 +276,7 @@ def test_ospf_authentication_simple_pass_tc28_p1(request):
 
     dut = "r2"
     ospf_covergence = verify_ospf_neighbor(tgen, topo, dut=dut)
-    assert ospf_covergence is True, "setup_module :Failed \n Error:" " {}".format(
+    assert ospf_covergence is True, "Testcase Failed \n Error  {}".format(
         ospf_covergence
     )
 
@@ -327,7 +316,7 @@ def test_ospf_authentication_simple_pass_tc28_p1(request):
 
     dut = "r1"
     ospf_covergence = verify_ospf_neighbor(tgen, topo, dut=dut)
-    assert ospf_covergence is True, "setup_module :Failed \n Error:" " {}".format(
+    assert ospf_covergence is True, "Testcase Failed \n Error  {}".format(
         ospf_covergence
     )
 
@@ -374,7 +363,7 @@ def test_ospf_authentication_md5_tc29_p1(request):
     ospf_covergence = verify_ospf_neighbor(
         tgen, topo, dut=dut, expected=False, retry_timeout=6
     )
-    assert ospf_covergence is not True, "setup_module :Failed \n Error:" " {}".format(
+    assert ospf_covergence is not True, "Testcase Failed \n Error  {}".format(
         ospf_covergence
     )
 
@@ -406,7 +395,7 @@ def test_ospf_authentication_md5_tc29_p1(request):
 
     dut = "r2"
     ospf_covergence = verify_ospf_neighbor(tgen, topo, dut=dut)
-    assert ospf_covergence is True, "setup_module :Failed \n Error:" " {}".format(
+    assert ospf_covergence is True, "Testcase Failed \n Error  {}".format(
         ospf_covergence
     )
 
@@ -439,7 +428,7 @@ def test_ospf_authentication_md5_tc29_p1(request):
     ospf_covergence = verify_ospf_neighbor(
         tgen, topo, dut=dut, expected=False, retry_timeout=10
     )
-    assert ospf_covergence is not True, "setup_module :Failed \n Error:" " {}".format(
+    assert ospf_covergence is not True, "Testcase Failed \n Error  {}".format(
         ospf_covergence
     )
 
@@ -467,7 +456,7 @@ def test_ospf_authentication_md5_tc29_p1(request):
 
     dut = "r2"
     ospf_covergence = verify_ospf_neighbor(tgen, topo, dut=dut)
-    assert ospf_covergence is True, "setup_module :Failed \n Error:" " {}".format(
+    assert ospf_covergence is True, "Testcase Failed \n Error  {}".format(
         ospf_covergence
     )
 
@@ -482,7 +471,7 @@ def test_ospf_authentication_md5_tc29_p1(request):
         "show ip ospf neighbor cmd."
     )
     ospf_covergence = verify_ospf_neighbor(tgen, topo, dut=dut, expected=False)
-    assert ospf_covergence is not True, "setup_module :Failed \n Error:" " {}".format(
+    assert ospf_covergence is not True, "Testcase Failed \n Error  {}".format(
         ospf_covergence
     )
 
@@ -496,7 +485,7 @@ def test_ospf_authentication_md5_tc29_p1(request):
 
     dut = "r2"
     ospf_covergence = verify_ospf_neighbor(tgen, topo, dut=dut)
-    assert ospf_covergence is True, "setup_module :Failed \n Error:" " {}".format(
+    assert ospf_covergence is True, "Testcase Failed \n Error  {}".format(
         ospf_covergence
     )
 
@@ -541,14 +530,484 @@ def test_ospf_authentication_md5_tc29_p1(request):
 
     dut = "r1"
     ospf_covergence = verify_ospf_neighbor(tgen, topo, dut=dut)
-    assert ospf_covergence is True, "setup_module :Failed \n Error:" " {}".format(
+    assert ospf_covergence is True, "Testcase Failed \n Error  {}".format(
         ospf_covergence
     )
 
     write_test_footer(tc_name)
 
 
-def test_ospf_authentication_different_auths_tc30_p1(request):
+def test_ospf_authentication_md5_keychain_tc30_p1(request):
+    """
+    OSPF Authentication - Verify ospf authentication with MD5 authentication.
+
+    """
+    tc_name = request.node.name
+    write_test_header(tc_name)
+    tgen = get_topogen()
+    global topo
+    step("Bring up the base config.")
+    reset_config_on_routers(tgen)
+    step(
+        "Configure ospf with on R1 and R2, enable ospf on R1 interface "
+        "connected to R2 with message-digest authentication using  ip "
+        "ospf authentication key-chain cmd."
+    )
+
+    router1 = tgen.gears["r1"]
+    router2 = tgen.gears["r2"]
+
+    router1.vtysh_cmd(
+        """configure terminal
+           key chain auth
+             key 10
+               key-string ospf
+               cryptographic-algorithm md5"""
+    )
+
+    r1_ospf_auth = {
+        "r1": {
+            "links": {
+                "r2": {
+                    "ospf": {
+                        "authentication": "key-chain",
+                        "keychain": "auth",
+                    }
+                }
+            }
+        }
+    }
+    result = config_ospf_interface(tgen, topo, r1_ospf_auth)
+    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
+
+    step("Verify that the neighbour is not FULL between R1 and R2.")
+    # wait for dead time expiry.
+    sleep(6)
+    dut = "r1"
+    ospf_covergence = verify_ospf_neighbor(
+        tgen, topo, dut=dut, expected=False, retry_timeout=6
+    )
+    assert ospf_covergence is not True, "Testcase Failed \n Error  {}".format(
+        ospf_covergence
+    )
+
+    step(
+        "On R2 enable ospf on interface with message-digest authentication"
+        "  using  ip ospf authentication  message-digest password cmd."
+    )
+
+    router2.vtysh_cmd(
+        """configure terminal
+           key chain auth
+             key 10
+               key-string ospf
+               cryptographic-algorithm md5"""
+    )
+
+    r2_ospf_auth = {
+        "r2": {
+            "links": {
+                "r1": {
+                    "ospf": {
+                        "authentication": "key-chain",
+                        "keychain": "auth",
+                    }
+                }
+            }
+        }
+    }
+    result = config_ospf_interface(tgen, topo, r2_ospf_auth)
+    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
+
+    step(
+        "Verify that the neighbour is FULL between R1 and R2  "
+        "using show ip ospf neighbor cmd."
+    )
+
+    dut = "r2"
+    ospf_covergence = verify_ospf_neighbor(tgen, topo, dut=dut)
+    assert ospf_covergence is True, "Testcase Failed \n Error  {}".format(
+        ospf_covergence
+    )
+
+    step(
+        "Disable message-digest authentication on R2  using no ip ospf "
+        "authentication  key-chain cmd."
+    )
+
+    r2_ospf_auth = {
+        "r2": {
+            "links": {
+                "r1": {
+                    "ospf": {
+                        "authentication": "key-chain",
+                        "keychain": "auth",
+                        "del_action": True,
+                    }
+                }
+            }
+        }
+    }
+    result = config_ospf_interface(tgen, topo, r2_ospf_auth)
+    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
+
+    step("Verify on R1 ,nbr is deleted for R2 after dead interval expiry")
+    #  wait till the dead timer expiry
+    sleep(6)
+    dut = "r2"
+    ospf_covergence = verify_ospf_neighbor(
+        tgen, topo, dut=dut, expected=False, retry_timeout=10
+    )
+    assert ospf_covergence is not True, "Testcase Failed \n Error  {}".format(
+        ospf_covergence
+    )
+
+    step("Again On R2 enable ospf on interface with key-chain auth")
+    r2_ospf_auth = {
+        "r2": {
+            "links": {
+                "r1": {
+                    "ospf": {
+                        "authentication": "key-chain",
+                        "keychain": "auth",
+                    }
+                }
+            }
+        }
+    }
+    result = config_ospf_interface(tgen, topo, r2_ospf_auth)
+    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
+
+    step(
+        "Verify that the neighbour is FULL between R1 and R2 using"
+        " show ip ospf neighbor cmd."
+    )
+
+    dut = "r2"
+    ospf_covergence = verify_ospf_neighbor(tgen, topo, dut=dut)
+    assert ospf_covergence is True, "Testcase Failed \n Error  {}".format(
+        ospf_covergence
+    )
+
+    step("Shut no shut interface on R1")
+    dut = "r1"
+    intf = topo["routers"]["r1"]["links"]["r2"]["interface"]
+    shutdown_bringup_interface(tgen, dut, intf, False)
+
+    dut = "r2"
+    step(
+        "Verify that the neighbour is not FULL between R1 and R2 using "
+        "show ip ospf neighbor cmd."
+    )
+    ospf_covergence = verify_ospf_neighbor(tgen, topo, dut=dut, expected=False)
+    assert ospf_covergence is not True, "Testcase Failed \n Error  {}".format(
+        ospf_covergence
+    )
+
+    dut = "r1"
+    shutdown_bringup_interface(tgen, dut, intf, True)
+
+    step(
+        "Verify that the neighbour is FULL between R1 and R2 using "
+        "show ip ospf neighbor cmd."
+    )
+
+    dut = "r2"
+    ospf_covergence = verify_ospf_neighbor(tgen, topo, dut=dut)
+    assert ospf_covergence is True, "Testcase Failed \n Error  {}".format(
+        ospf_covergence
+    )
+
+    step("Change Ip address on R1 and R2")
+
+    topo_modify_change_ip = deepcopy(topo)
+
+    intf_ip = topo_modify_change_ip["routers"]["r1"]["links"]["r2"]["ipv4"]
+
+    topo_modify_change_ip["routers"]["r1"]["links"]["r2"]["ipv4"] = str(
+        IPv4Address(frr_unicode(intf_ip.split("/")[0])) + 3
+    ) + "/{}".format(intf_ip.split("/")[1])
+
+    build_config_from_json(tgen, topo_modify_change_ip, save_bkup=False)
+
+    reset_config_on_routers(tgen, routerName="r1")
+    dut = "r1"
+    intf = topo["routers"]["r1"]["links"]["r2"]["interface"]
+    shutdown_bringup_interface(tgen, dut, intf, False)
+    shutdown_bringup_interface(tgen, dut, intf, True)
+    clear_ospf(tgen, "r1")
+    router1.vtysh_cmd(
+        """configure terminal
+           key chain auth
+             key 10
+               key-string ospf
+               cryptographic-algorithm md5"""
+    )
+    r1_ospf_auth = {
+        "r1": {
+            "links": {
+                "r2": {
+                    "ospf": {
+                        "authentication": "key-chain",
+                        "keychain": "auth",
+                    }
+                }
+            }
+        }
+    }
+    result = config_ospf_interface(tgen, topo, r1_ospf_auth)
+    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
+
+    step(
+        "Verify that the neighbour is FULL between R1 and R2 with new "
+        "ip address using show ip ospf "
+    )
+
+    dut = "r1"
+    ospf_covergence = verify_ospf_neighbor(tgen, topo, dut=dut)
+    assert ospf_covergence is True, "Testcase Failed \n Error  {}".format(
+        ospf_covergence
+    )
+
+    write_test_footer(tc_name)
+
+
+def test_ospf_authentication_sha256_keychain_tc32_p1(request):
+    """
+    OSPF Authentication - Verify ospf authentication with MD5 authentication.
+
+    """
+    tc_name = request.node.name
+    write_test_header(tc_name)
+    tgen = get_topogen()
+    global topo
+    step("Bring up the base config.")
+    reset_config_on_routers(tgen)
+    step(
+        "Configure ospf with on R1 and R2, enable ospf on R1 interface "
+        "connected to R2 with message-digest authentication using  ip "
+        "ospf authentication key-chain cmd."
+    )
+
+    router1 = tgen.gears["r1"]
+    router2 = tgen.gears["r2"]
+
+    router1.vtysh_cmd(
+        """configure terminal
+           key chain auth
+             key 10
+               key-string ospf
+               cryptographic-algorithm hmac-sha-256"""
+    )
+
+    r1_ospf_auth = {
+        "r1": {
+            "links": {
+                "r2": {
+                    "ospf": {
+                        "authentication": "key-chain",
+                        "keychain": "auth",
+                    }
+                }
+            }
+        }
+    }
+    result = config_ospf_interface(tgen, topo, r1_ospf_auth)
+    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
+
+    step("Verify that the neighbour is not FULL between R1 and R2.")
+    # wait for dead time expiry.
+    sleep(6)
+    dut = "r1"
+    ospf_covergence = verify_ospf_neighbor(
+        tgen, topo, dut=dut, expected=False, retry_timeout=6
+    )
+    assert ospf_covergence is not True, "Testcase Failed \n Error  {}".format(
+        ospf_covergence
+    )
+
+    step(
+        "On R2 enable ospf on interface with message-digest authentication"
+        "  using  ip ospf authentication  message-digest password cmd."
+    )
+
+    router2.vtysh_cmd(
+        """configure terminal
+           key chain auth
+             key 10
+               key-string ospf
+               cryptographic-algorithm hmac-sha-256"""
+    )
+
+    r2_ospf_auth = {
+        "r2": {
+            "links": {
+                "r1": {
+                    "ospf": {
+                        "authentication": "key-chain",
+                        "keychain": "auth",
+                    }
+                }
+            }
+        }
+    }
+    result = config_ospf_interface(tgen, topo, r2_ospf_auth)
+    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
+
+    step(
+        "Verify that the neighbour is FULL between R1 and R2  "
+        "using show ip ospf neighbor cmd."
+    )
+
+    dut = "r2"
+    ospf_covergence = verify_ospf_neighbor(tgen, topo, dut=dut)
+    assert ospf_covergence is True, "Testcase Failed \n Error  {}".format(
+        ospf_covergence
+    )
+
+    step(
+        "Disable message-digest authentication on R2  using no ip ospf "
+        "authentication  key-chain cmd."
+    )
+
+    r2_ospf_auth = {
+        "r2": {
+            "links": {
+                "r1": {
+                    "ospf": {
+                        "authentication": "key-chain",
+                        "keychain": "auth",
+                        "del_action": True,
+                    }
+                }
+            }
+        }
+    }
+    result = config_ospf_interface(tgen, topo, r2_ospf_auth)
+    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
+
+    step("Verify on R1 ,nbr is deleted for R2 after dead interval expiry")
+    #  wait till the dead timer expiry
+    sleep(6)
+    dut = "r2"
+    ospf_covergence = verify_ospf_neighbor(
+        tgen, topo, dut=dut, expected=False, retry_timeout=10
+    )
+    assert ospf_covergence is not True, "Testcase Failed \n Error  {}".format(
+        ospf_covergence
+    )
+
+    step("Again On R2 enable ospf on interface with key-chain auth")
+    r2_ospf_auth = {
+        "r2": {
+            "links": {
+                "r1": {
+                    "ospf": {
+                        "authentication": "key-chain",
+                        "keychain": "auth",
+                    }
+                }
+            }
+        }
+    }
+    result = config_ospf_interface(tgen, topo, r2_ospf_auth)
+    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
+
+    step(
+        "Verify that the neighbour is FULL between R1 and R2 using"
+        " show ip ospf neighbor cmd."
+    )
+
+    dut = "r2"
+    ospf_covergence = verify_ospf_neighbor(tgen, topo, dut=dut)
+    assert ospf_covergence is True, "Testcase Failed \n Error  {}".format(
+        ospf_covergence
+    )
+
+    step("Shut no shut interface on R1")
+    dut = "r1"
+    intf = topo["routers"]["r1"]["links"]["r2"]["interface"]
+    shutdown_bringup_interface(tgen, dut, intf, False)
+
+    dut = "r2"
+    step(
+        "Verify that the neighbour is not FULL between R1 and R2 using "
+        "show ip ospf neighbor cmd."
+    )
+    ospf_covergence = verify_ospf_neighbor(tgen, topo, dut=dut, expected=False)
+    assert ospf_covergence is not True, "Testcase Failed \n Error  {}".format(
+        ospf_covergence
+    )
+
+    dut = "r1"
+    shutdown_bringup_interface(tgen, dut, intf, True)
+
+    step(
+        "Verify that the neighbour is FULL between R1 and R2 using "
+        "show ip ospf neighbor cmd."
+    )
+
+    dut = "r2"
+    ospf_covergence = verify_ospf_neighbor(tgen, topo, dut=dut)
+    assert ospf_covergence is True, "Testcase Failed \n Error  {}".format(
+        ospf_covergence
+    )
+
+    step("Change Ip address on R1 and R2")
+
+    topo_modify_change_ip = deepcopy(topo)
+
+    intf_ip = topo_modify_change_ip["routers"]["r1"]["links"]["r2"]["ipv4"]
+
+    topo_modify_change_ip["routers"]["r1"]["links"]["r2"]["ipv4"] = str(
+        IPv4Address(frr_unicode(intf_ip.split("/")[0])) + 3
+    ) + "/{}".format(intf_ip.split("/")[1])
+
+    build_config_from_json(tgen, topo_modify_change_ip, save_bkup=False)
+
+    reset_config_on_routers(tgen, routerName="r1")
+    dut = "r1"
+    intf = topo["routers"]["r1"]["links"]["r2"]["interface"]
+    shutdown_bringup_interface(tgen, dut, intf, False)
+    shutdown_bringup_interface(tgen, dut, intf, True)
+    clear_ospf(tgen, "r1")
+    router1.vtysh_cmd(
+        """configure terminal
+           key chain auth
+             key 10
+               key-string ospf
+               cryptographic-algorithm hmac-sha-256"""
+    )
+    r1_ospf_auth = {
+        "r1": {
+            "links": {
+                "r2": {
+                    "ospf": {
+                        "authentication": "key-chain",
+                        "keychain": "auth",
+                    }
+                }
+            }
+        }
+    }
+    result = config_ospf_interface(tgen, topo, r1_ospf_auth)
+    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
+
+    step(
+        "Verify that the neighbour is FULL between R1 and R2 with new "
+        "ip address using show ip ospf "
+    )
+
+    dut = "r1"
+    ospf_covergence = verify_ospf_neighbor(tgen, topo, dut=dut)
+    assert ospf_covergence is True, "Testcase Failed \n Error  {}".format(
+        ospf_covergence
+    )
+
+    write_test_footer(tc_name)
+
+
+def test_ospf_authentication_different_auths_tc35_p1(request):
     """
     OSPF Authentication - Verify ospf authentication with different
     authentication methods.
@@ -566,6 +1025,9 @@ def test_ospf_authentication_different_auths_tc30_p1(request):
         "ospf authentication  message-digest cmd."
     )
 
+    router1 = tgen.gears["r1"]
+    router2 = tgen.gears["r2"]
+
     r1_ospf_auth = {
         "r1": {
             "links": {
@@ -589,7 +1051,7 @@ def test_ospf_authentication_different_auths_tc30_p1(request):
     ospf_covergence = verify_ospf_neighbor(
         tgen, topo, dut=dut, expected=False, retry_timeout=10
     )
-    assert ospf_covergence is not True, "setup_module :Failed \n Error:" " {}".format(
+    assert ospf_covergence is not True, "Testcase Failed \n Error  {}".format(
         ospf_covergence
     )
 
@@ -621,7 +1083,7 @@ def test_ospf_authentication_different_auths_tc30_p1(request):
 
     dut = "r2"
     ospf_covergence = verify_ospf_neighbor(tgen, topo, dut=dut)
-    assert ospf_covergence is True, "setup_module :Failed \n Error:" " {}".format(
+    assert ospf_covergence is True, "Testcase Failed \n Error  {}".format(
         ospf_covergence
     )
 
@@ -668,7 +1130,7 @@ def test_ospf_authentication_different_auths_tc30_p1(request):
 
     dut = "r2"
     ospf_covergence = verify_ospf_neighbor(tgen, topo, dut=dut)
-    assert ospf_covergence is True, "setup_module :Failed \n Error:" " {}".format(
+    assert ospf_covergence is True, "Testcase Failed \n Error  {}".format(
         ospf_covergence
     )
 
@@ -700,7 +1162,7 @@ def test_ospf_authentication_different_auths_tc30_p1(request):
 
     dut = "r2"
     ospf_covergence = verify_ospf_neighbor(tgen, topo, dut=dut)
-    assert ospf_covergence is True, "setup_module :Failed \n Error:" " {}".format(
+    assert ospf_covergence is True, "Testcase Failed \n Error  {}".format(
         ospf_covergence
     )
 
@@ -733,7 +1195,7 @@ def test_ospf_authentication_different_auths_tc30_p1(request):
 
     dut = "r2"
     ospf_covergence = verify_ospf_neighbor(tgen, topo, dut=dut)
-    assert ospf_covergence is True, "setup_module :Failed \n Error:" " {}".format(
+    assert ospf_covergence is True, "Testcase Failed \n Error  {}".format(
         ospf_covergence
     )
 
@@ -778,20 +1240,27 @@ def test_ospf_authentication_different_auths_tc30_p1(request):
 
     dut = "r2"
     ospf_covergence = verify_ospf_neighbor(tgen, topo, dut=dut)
-    assert ospf_covergence is True, "setup_module :Failed \n Error:" " {}".format(
+    assert ospf_covergence is True, "Testcase Failed \n Error  {}".format(
         ospf_covergence
     )
 
-    step("Enable Md5 authentication on the interface")
+    step("Enable SHA-256 authentication on the interface")
+
+    router1.vtysh_cmd(
+        """configure terminal
+           key chain auth
+             key 10
+               key-string ospf
+               cryptographic-algorithm hmac-sha-256"""
+    )
 
     r1_ospf_auth = {
         "r1": {
             "links": {
                 "r2": {
                     "ospf": {
-                        "authentication": "message-digest",
-                        "authentication-key": "ospf",
-                        "message-digest-key": "10",
+                        "authentication": "key-chain",
+                        "keychain": "auth",
                     }
                 }
             }
@@ -800,14 +1269,21 @@ def test_ospf_authentication_different_auths_tc30_p1(request):
     result = config_ospf_interface(tgen, topo, r1_ospf_auth)
     assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
 
+    router2.vtysh_cmd(
+        """configure terminal
+           key chain auth
+             key 10
+               key-string ospf
+               cryptographic-algorithm hmac-sha-256"""
+    )
+
     r2_ospf_auth = {
         "r2": {
             "links": {
                 "r1": {
                     "ospf": {
-                        "authentication": "message-digest",
-                        "authentication-key": "ospf",
-                        "message-digest-key": "10",
+                        "authentication": "key-chain",
+                        "keychain": "auth",
                     }
                 }
             }
@@ -823,43 +1299,31 @@ def test_ospf_authentication_different_auths_tc30_p1(request):
 
     dut = "r2"
     ospf_covergence = verify_ospf_neighbor(tgen, topo, dut=dut)
-    assert ospf_covergence is True, "setup_module :Failed \n Error:" " {}".format(
+    assert ospf_covergence is True, "Testcase Failed \n Error  {}".format(
         ospf_covergence
     )
 
-    step("Change the MD5 authentication password")
+    step("Change the SHA-256 authentication password")
 
-    r1_ospf_auth = {
-        "r1": {
-            "links": {
-                "r2": {
-                    "ospf": {
-                        "authentication": "message-digest",
-                        "authentication-key": "OSPFv4",
-                        "message-digest-key": "10",
-                    }
-                }
-            }
-        }
-    }
-    result = config_ospf_interface(tgen, topo, r1_ospf_auth)
-    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
+    router1.vtysh_cmd(
+        """configure terminal
+           key chain auth
+             key 10
+               key-string OSPFv4
+               cryptographic-algorithm hmac-sha-512"""
+    )
 
-    r2_ospf_auth = {
-        "r2": {
-            "links": {
-                "r1": {
-                    "ospf": {
-                        "authentication": "message-digest",
-                        "authentication-key": "OSPFv4",
-                        "message-digest-key": "10",
-                    }
-                }
-            }
-        }
-    }
-    result = config_ospf_interface(tgen, topo, r2_ospf_auth)
-    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
+    router2.vtysh_cmd(
+        """configure terminal
+           key chain auth
+             key 10
+               key-string OSPFv4
+               cryptographic-algorithm hmac-sha-512"""
+    )
+    ospf_covergence = verify_ospf_neighbor(tgen, topo, dut=dut)
+    assert ospf_covergence is True, "Testcase Failed \n Error  {}".format(
+        ospf_covergence
+    )
 
     write_test_footer(tc_name)
 
