@@ -1,24 +1,11 @@
 #!/usr/bin/python
+# SPDX-License-Identifier: ISC
 
 #
 # Copyright (c) 2021 by VMware, Inc. ("VMware")
 # Used Copyright (c) 2018 by Network Device Education Foundation, Inc.
 # ("NetDEF") in this file.
 #
-# Permission to use, copy, modify, and/or distribute this software
-# for any purpose with or without fee is hereby granted, provided
-# that the above copyright notice and this permission notice appear
-# in all copies.
-#
-# THE SOFTWARE IS PROVIDED "AS IS" AND VMWARE DISCLAIMS ALL WARRANTIES
-# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
-# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL VMWARE BE LIABLE FOR
-# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY
-# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
-# WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
-# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
-# OF THIS SOFTWARE.
-#
 
 
 """OSPF Basic Functionality Automation."""
@@ -124,7 +111,7 @@ def setup_module(mod):
         pytest.skip(tgen.errors)
 
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo)
-    assert ospf6_covergence is True, "setup_module :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is True, "setup_module :Failed \n Error:  {}".format(
         ospf6_covergence
     )
 
@@ -188,14 +175,14 @@ def test_ospf6_auth_trailer_tc1_md5(request):
     result = config_ospf6_interface(tgen, topo, r1_ospf6_auth)
     assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
 
-    step("Verify that the neighbour is not FULL between R1 and R2.")
+    step("Verify that the neighbor is not FULL between R1 and R2.")
     # wait for dead time expiry.
     sleep(6)
     dut = "r1"
     ospf6_covergence = verify_ospf6_neighbor(
         tgen, topo, dut=dut, expected=False, retry_timeout=3
     )
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -221,13 +208,13 @@ def test_ospf6_auth_trailer_tc1_md5(request):
     assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
 
     step(
-        "Verify that the neighbour is FULL between R1 and R2  "
+        "Verify that the neighbor is FULL between R1 and R2  "
         "using show ipv6 ospf6 neighbor cmd."
     )
 
     dut = "r2"
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
-    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -257,7 +244,7 @@ def test_ospf6_auth_trailer_tc1_md5(request):
     ospf6_covergence = verify_ospf6_neighbor(
         tgen, topo, dut=dut, expected=False, retry_timeout=5
     )
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -279,13 +266,13 @@ def test_ospf6_auth_trailer_tc1_md5(request):
     assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
 
     step(
-        "Verify that the neighbour is FULL between R1 and R2 using"
+        "Verify that the neighbor is FULL between R1 and R2 using"
         " show ip ospf6 neighbor cmd."
     )
 
     dut = "r2"
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
-    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -296,11 +283,11 @@ def test_ospf6_auth_trailer_tc1_md5(request):
 
     dut = "r2"
     step(
-        "Verify that the neighbour is not FULL between R1 and R2 using "
+        "Verify that the neighbor is not FULL between R1 and R2 using "
         "show ip ospf6 neighbor cmd."
     )
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut, expected=False)
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -308,13 +295,13 @@ def test_ospf6_auth_trailer_tc1_md5(request):
     shutdown_bringup_interface(tgen, dut, intf, True)
 
     step(
-        "Verify that the neighbour is FULL between R1 and R2 using "
+        "Verify that the neighbor is FULL between R1 and R2 using "
         "show ip ospf6 neighbor cmd."
     )
 
     dut = "r2"
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
-    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -354,14 +341,14 @@ def test_ospf6_auth_trailer_tc2_sha256(request):
     result = config_ospf6_interface(tgen, topo, r1_ospf6_auth)
     assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
 
-    step("Verify that the neighbour is not FULL between R1 and R2.")
+    step("Verify that the neighbor is not FULL between R1 and R2.")
     # wait for dead time expiry.
     sleep(6)
     dut = "r1"
     ospf6_covergence = verify_ospf6_neighbor(
         tgen, topo, dut=dut, expected=False, retry_timeout=3
     )
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -387,13 +374,13 @@ def test_ospf6_auth_trailer_tc2_sha256(request):
     assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
 
     step(
-        "Verify that the neighbour is FULL between R1 and R2  "
+        "Verify that the neighbor is FULL between R1 and R2  "
         "using show ipv6 ospf6 neighbor cmd."
     )
 
     dut = "r2"
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
-    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -423,7 +410,7 @@ def test_ospf6_auth_trailer_tc2_sha256(request):
     ospf6_covergence = verify_ospf6_neighbor(
         tgen, topo, dut=dut, expected=False, retry_timeout=5
     )
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -445,13 +432,13 @@ def test_ospf6_auth_trailer_tc2_sha256(request):
     assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
 
     step(
-        "Verify that the neighbour is FULL between R1 and R2 using"
+        "Verify that the neighbor is FULL between R1 and R2 using"
         " show ip ospf6 neighbor cmd."
     )
 
     dut = "r2"
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
-    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -462,11 +449,11 @@ def test_ospf6_auth_trailer_tc2_sha256(request):
 
     dut = "r2"
     step(
-        "Verify that the neighbour is not FULL between R1 and R2 using "
+        "Verify that the neighbor is not FULL between R1 and R2 using "
         "show ip ospf6 neighbor cmd."
     )
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut, expected=False)
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -474,13 +461,72 @@ def test_ospf6_auth_trailer_tc2_sha256(request):
     shutdown_bringup_interface(tgen, dut, intf, True)
 
     step(
-        "Verify that the neighbour is FULL between R1 and R2 using "
+        "Verify that the neighbor is FULL between R1 and R2 using "
         "show ip ospf6 neighbor cmd."
     )
 
     dut = "r2"
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
-    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
+        tc_name, ospf6_covergence
+    )
+
+    step("Change the key ID on R2 to not match R1")
+    r2_ospf6_auth = {
+        "r2": {
+            "links": {
+                "r1": {
+                    "ospf6": {
+                        "hash-algo": "hmac-sha-256",
+                        "key": "ospf6",
+                        "key-id": "30",
+                    }
+                }
+            }
+        }
+    }
+    result = config_ospf6_interface(tgen, topo, r2_ospf6_auth)
+    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
+
+    step(
+        "Verify on R1 that R2 nbr is deleted due to key-id mismatch "
+        "after dead interval expiry"
+    )
+    #  wait till the dead timer expiry
+    sleep(6)
+    dut = "r2"
+    ospf6_covergence = verify_ospf6_neighbor(
+        tgen, topo, dut=dut, expected=False, retry_timeout=5
+    )
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
+        tc_name, ospf6_covergence
+    )
+
+    step("Correct the key ID on R2 so that it matches R1")
+    r2_ospf6_auth = {
+        "r2": {
+            "links": {
+                "r1": {
+                    "ospf6": {
+                        "hash-algo": "hmac-sha-256",
+                        "key": "ospf6",
+                        "key-id": "10",
+                    }
+                }
+            }
+        }
+    }
+    result = config_ospf6_interface(tgen, topo, r2_ospf6_auth)
+    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
+
+    step(
+        "Verify that the neighbor is FULL between R1 and R2 using "
+        "show ip ospf6 neighbor cmd."
+    )
+
+    dut = "r2"
+    ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
+    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -537,14 +583,14 @@ def test_ospf6_auth_trailer_tc3_keychain_md5(request):
     result = config_ospf6_interface(tgen, topo, r1_ospf6_auth)
     assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
 
-    step("Verify that the neighbour is not FULL between R1 and R2.")
+    step("Verify that the neighbor is not FULL between R1 and R2.")
     # wait for dead time expiry.
     sleep(6)
     dut = "r1"
     ospf6_covergence = verify_ospf6_neighbor(
         tgen, topo, dut=dut, expected=False, retry_timeout=3
     )
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -568,13 +614,13 @@ def test_ospf6_auth_trailer_tc3_keychain_md5(request):
     assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
 
     step(
-        "Verify that the neighbour is FULL between R1 and R2  "
+        "Verify that the neighbor is FULL between R1 and R2  "
         "using show ipv6 ospf6 neighbor cmd."
     )
 
     dut = "r2"
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
-    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -593,7 +639,7 @@ def test_ospf6_auth_trailer_tc3_keychain_md5(request):
     ospf6_covergence = verify_ospf6_neighbor(
         tgen, topo, dut=dut, expected=False, retry_timeout=5
     )
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -613,13 +659,13 @@ def test_ospf6_auth_trailer_tc3_keychain_md5(request):
     assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
 
     step(
-        "Verify that the neighbour is FULL between R1 and R2 using"
+        "Verify that the neighbor is FULL between R1 and R2 using"
         " show ip ospf6 neighbor cmd."
     )
 
     dut = "r2"
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
-    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -630,11 +676,11 @@ def test_ospf6_auth_trailer_tc3_keychain_md5(request):
 
     dut = "r2"
     step(
-        "Verify that the neighbour is not FULL between R1 and R2 using "
+        "Verify that the neighbor is not FULL between R1 and R2 using "
         "show ip ospf6 neighbor cmd."
     )
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut, expected=False)
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -642,13 +688,13 @@ def test_ospf6_auth_trailer_tc3_keychain_md5(request):
     shutdown_bringup_interface(tgen, dut, intf, True)
 
     step(
-        "Verify that the neighbour is FULL between R1 and R2 using "
+        "Verify that the neighbor is FULL between R1 and R2 using "
         "show ip ospf6 neighbor cmd."
     )
 
     dut = "r2"
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
-    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -705,14 +751,14 @@ def test_ospf6_auth_trailer_tc4_keychain_sha256(request):
     result = config_ospf6_interface(tgen, topo, r1_ospf6_auth)
     assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
 
-    step("Verify that the neighbour is not FULL between R1 and R2.")
+    step("Verify that the neighbor is not FULL between R1 and R2.")
     # wait for dead time expiry.
     sleep(6)
     dut = "r1"
     ospf6_covergence = verify_ospf6_neighbor(
         tgen, topo, dut=dut, expected=False, retry_timeout=3
     )
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -736,13 +782,13 @@ def test_ospf6_auth_trailer_tc4_keychain_sha256(request):
     assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
 
     step(
-        "Verify that the neighbour is FULL between R1 and R2  "
+        "Verify that the neighbor is FULL between R1 and R2  "
         "using show ipv6 ospf6 neighbor cmd."
     )
 
     dut = "r2"
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
-    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -761,7 +807,7 @@ def test_ospf6_auth_trailer_tc4_keychain_sha256(request):
     ospf6_covergence = verify_ospf6_neighbor(
         tgen, topo, dut=dut, expected=False, retry_timeout=5
     )
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -781,13 +827,13 @@ def test_ospf6_auth_trailer_tc4_keychain_sha256(request):
     assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
 
     step(
-        "Verify that the neighbour is FULL between R1 and R2 using"
+        "Verify that the neighbor is FULL between R1 and R2 using"
         " show ip ospf6 neighbor cmd."
     )
 
     dut = "r2"
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
-    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -798,11 +844,11 @@ def test_ospf6_auth_trailer_tc4_keychain_sha256(request):
 
     dut = "r2"
     step(
-        "Verify that the neighbour is not FULL between R1 and R2 using "
+        "Verify that the neighbor is not FULL between R1 and R2 using "
         "show ip ospf6 neighbor cmd."
     )
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut, expected=False)
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -810,13 +856,13 @@ def test_ospf6_auth_trailer_tc4_keychain_sha256(request):
     shutdown_bringup_interface(tgen, dut, intf, True)
 
     step(
-        "Verify that the neighbour is FULL between R1 and R2 using "
+        "Verify that the neighbor is FULL between R1 and R2 using "
         "show ip ospf6 neighbor cmd."
     )
 
     dut = "r2"
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
-    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -856,14 +902,14 @@ def test_ospf6_auth_trailer_tc5_md5_keymissmatch(request):
     result = config_ospf6_interface(tgen, topo, r1_ospf6_auth)
     assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
 
-    step("Verify that the neighbour is not FULL between R1 and R2.")
+    step("Verify that the neighbor is not FULL between R1 and R2.")
     # wait for dead time expiry.
     sleep(6)
     dut = "r1"
     ospf6_covergence = verify_ospf6_neighbor(
         tgen, topo, dut=dut, expected=False, retry_timeout=3
     )
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -889,18 +935,18 @@ def test_ospf6_auth_trailer_tc5_md5_keymissmatch(request):
     assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
 
     step(
-        "Verify that the neighbour is not FULL between R1 and R2  "
+        "Verify that the neighbor is not FULL between R1 and R2  "
         "using show ipv6 ospf6 neighbor cmd."
     )
 
-    step("Verify that the neighbour is FULL between R1 and R2.")
+    step("Verify that the neighbor is FULL between R1 and R2.")
     # wait for dead time expiry.
     sleep(6)
     dut = "r2"
     ospf6_covergence = verify_ospf6_neighbor(
         tgen, topo, dut=dut, expected=False, retry_timeout=3
     )
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -926,13 +972,13 @@ def test_ospf6_auth_trailer_tc5_md5_keymissmatch(request):
     assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
 
     step(
-        "Verify that the neighbour is FULL between R1 and R2  "
+        "Verify that the neighbor is FULL between R1 and R2  "
         "using show ipv6 ospf6 neighbor cmd."
     )
 
     dut = "r2"
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
-    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -972,14 +1018,14 @@ def test_ospf6_auth_trailer_tc6_sha256_mismatch(request):
     result = config_ospf6_interface(tgen, topo, r1_ospf6_auth)
     assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
 
-    step("Verify that the neighbour is not FULL between R1 and R2.")
+    step("Verify that the neighbor is not FULL between R1 and R2.")
     # wait for dead time expiry.
     sleep(6)
     dut = "r1"
     ospf6_covergence = verify_ospf6_neighbor(
         tgen, topo, dut=dut, expected=False, retry_timeout=3
     )
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -1004,14 +1050,14 @@ def test_ospf6_auth_trailer_tc6_sha256_mismatch(request):
     result = config_ospf6_interface(tgen, topo, r2_ospf6_auth)
     assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
 
-    step("Verify that the neighbour is not FULL between R1 and R2.")
+    step("Verify that the neighbor is not FULL between R1 and R2.")
     # wait for dead time expiry.
     sleep(6)
     dut = "r2"
     ospf6_covergence = verify_ospf6_neighbor(
         tgen, topo, dut=dut, expected=False, retry_timeout=3
     )
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -1037,13 +1083,13 @@ def test_ospf6_auth_trailer_tc6_sha256_mismatch(request):
     assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
 
     step(
-        "Verify that the neighbour is FULL between R1 and R2  "
+        "Verify that the neighbor is FULL between R1 and R2  "
         "using show ipv6 ospf6 neighbor cmd."
     )
 
     dut = "r2"
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
-    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -1108,14 +1154,14 @@ def test_ospf6_auth_trailer_tc7_keychain_md5_missmatch(request):
     result = config_ospf6_interface(tgen, topo, r1_ospf6_auth)
     assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
 
-    step("Verify that the neighbour is not FULL between R1 and R2.")
+    step("Verify that the neighbor is not FULL between R1 and R2.")
     # wait for dead time expiry.
     sleep(6)
     dut = "r1"
     ospf6_covergence = verify_ospf6_neighbor(
         tgen, topo, dut=dut, expected=False, retry_timeout=3
     )
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -1138,14 +1184,14 @@ def test_ospf6_auth_trailer_tc7_keychain_md5_missmatch(request):
     result = config_ospf6_interface(tgen, topo, r2_ospf6_auth)
     assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
 
-    step("Verify that the neighbour is not FULL between R1 and R2.")
+    step("Verify that the neighbor is not FULL between R1 and R2.")
     # wait for dead time expiry.
     sleep(6)
     dut = "r2"
     ospf6_covergence = verify_ospf6_neighbor(
         tgen, topo, dut=dut, expected=False, retry_timeout=3
     )
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -1169,13 +1215,13 @@ def test_ospf6_auth_trailer_tc7_keychain_md5_missmatch(request):
     assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
 
     step(
-        "Verify that the neighbour is FULL between R1 and R2  "
+        "Verify that the neighbor is FULL between R1 and R2  "
         "using show ipv6 ospf6 neighbor cmd."
     )
 
     dut = "r2"
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
-    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -1240,14 +1286,14 @@ def test_ospf6_auth_trailer_tc8_keychain_sha256_missmatch(request):
     result = config_ospf6_interface(tgen, topo, r1_ospf6_auth)
     assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
 
-    step("Verify that the neighbour is not FULL between R1 and R2.")
+    step("Verify that the neighbor is not FULL between R1 and R2.")
     # wait for dead time expiry.
     sleep(6)
     dut = "r1"
     ospf6_covergence = verify_ospf6_neighbor(
         tgen, topo, dut=dut, expected=False, retry_timeout=3
     )
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -1270,14 +1316,14 @@ def test_ospf6_auth_trailer_tc8_keychain_sha256_missmatch(request):
     result = config_ospf6_interface(tgen, topo, r2_ospf6_auth)
     assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
 
-    step("Verify that the neighbour is not FULL between R1 and R2.")
+    step("Verify that the neighbor is not FULL between R1 and R2.")
     # wait for dead time expiry.
     sleep(6)
     dut = "r2"
     ospf6_covergence = verify_ospf6_neighbor(
         tgen, topo, dut=dut, expected=False, retry_timeout=3
     )
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -1301,13 +1347,13 @@ def test_ospf6_auth_trailer_tc8_keychain_sha256_missmatch(request):
     assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
 
     step(
-        "Verify that the neighbour is FULL between R1 and R2  "
+        "Verify that the neighbor is FULL between R1 and R2  "
         "using show ipv6 ospf6 neighbor cmd."
     )
 
     dut = "r2"
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
-    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -1348,14 +1394,14 @@ def test_ospf6_auth_trailer_tc9_keychain_not_configured(request):
     result = config_ospf6_interface(tgen, topo, r1_ospf6_auth)
     assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
 
-    step("Verify that the neighbour is not FULL between R1 and R2.")
+    step("Verify that the neighbor is not FULL between R1 and R2.")
     # wait for dead time expiry.
     sleep(6)
     dut = "r1"
     ospf6_covergence = verify_ospf6_neighbor(
         tgen, topo, dut=dut, expected=False, retry_timeout=3
     )
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -1378,14 +1424,14 @@ def test_ospf6_auth_trailer_tc9_keychain_not_configured(request):
     result = config_ospf6_interface(tgen, topo, r2_ospf6_auth)
     assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
 
-    step("Verify that the neighbour is not FULL between R1 and R2.")
+    step("Verify that the neighbor is not FULL between R1 and R2.")
     # wait for dead time expiry.
     sleep(6)
     dut = "r2"
     ospf6_covergence = verify_ospf6_neighbor(
         tgen, topo, dut=dut, expected=False, retry_timeout=3
     )
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -1409,13 +1455,13 @@ def test_ospf6_auth_trailer_tc10_no_auth_trailer(request):
     router2 = tgen.gears["r2"]
 
     step(
-        "Verify that the neighbour is FULL between R1 and R2  "
+        "Verify that the neighbor is FULL between R1 and R2  "
         "using show ipv6 ospf6 neighbor cmd."
     )
 
     dut = "r2"
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
-    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
