@@ -1,24 +1,11 @@
 #!/usr/bin/python
+# SPDX-License-Identifier: ISC
 
 #
 # Copyright (c) 2021 by VMware, Inc. ("VMware")
 # Used Copyright (c) 2018 by Network Device Education Foundation, Inc.
 # ("NetDEF") in this file.
 #
-# Permission to use, copy, modify, and/or distribute this software
-# for any purpose with or without fee is hereby granted, provided
-# that the above copyright notice and this permission notice appear
-# in all copies.
-#
-# THE SOFTWARE IS PROVIDED "AS IS" AND VMWARE DISCLAIMS ALL WARRANTIES
-# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
-# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL VMWARE BE LIABLE FOR
-# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY
-# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
-# WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
-# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
-# OF THIS SOFTWARE.
-#
 
 
 """OSPF Basic Functionality Automation."""
@@ -151,7 +138,7 @@ def setup_module(mod):
 
     # Api call verify whether OSPF is converged
     ospf_covergence = verify_ospf6_neighbor(tgen, topo)
-    assert ospf_covergence is True, "setup_module :Failed \n Error:" " {}".format(
+    assert ospf_covergence is True, "setup_module :Failed \n Error:  {}".format(
         ospf_covergence
     )
 
@@ -290,7 +277,7 @@ def test_ospfv3_nssa_tc26_p0(request):
     result = verify_ospf6_neighbor(tgen, topo, dut="r2", expected=False)
     assert (
         result is not True
-    ), "Testcase {} : Failed \n Nbrs are not down" "Error: {}".format(tc_name, result)
+    ), "Testcase {} : Failed \n Nbrs are not down Error: {}".format(tc_name, result)
 
     step("Now configure area 0 on interface of r1 connecting to r2.")
 
@@ -362,7 +349,7 @@ def test_ospfv3_nssa_tc26_p0(request):
     result = verify_ospf6_neighbor(tgen, topo, dut="r2", expected=False)
     assert (
         result is not True
-    ), "Testcase {} : Failed \n Nbrs are not down" "Error: {}".format(tc_name, result)
+    ), "Testcase {} : Failed \n Nbrs are not down Error: {}".format(tc_name, result)
 
     step("Now configure area 2 on interface of r1 connecting to r2.")
 
@@ -423,7 +410,7 @@ def test_ospfv3_learning_tc15_p0(request):
 
     step("Verify that Type 3 summary LSA is originated for the same Area 0")
     ip = topo["routers"]["r1"]["links"]["r3-link0"]["ipv6"]
-    ip_net = str(ipaddress.ip_interface(u"{}".format(ip)).network)
+    ip_net = str(ipaddress.ip_interface("{}".format(ip)).network)
 
     input_dict = {
         "r1": {
@@ -484,7 +471,7 @@ def test_ospfv3_learning_tc15_p0(request):
     result = verify_ospf6_neighbor(tgen, topo)
     assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)
 
-    step("Change area 1 as non nssa area (on the fly changing area" " type on DUT).")
+    step("Change area 1 as non nssa area (on the fly changing area  type on DUT).")
 
     for rtr in ["r1", "r2", "r3"]:
         input_dict = {
