@@ -1,4 +1,5 @@
 #!/usr/bin/env python
+# SPDX-License-Identifier: ISC
 
 #
 # test_route_scale1.py
@@ -7,20 +8,6 @@
 # Nvidia, Inc.
 # Donald Sharp
 #
-# Permission to use, copy, modify, and/or distribute this software
-# for any purpose with or without fee is hereby granted, provided
-# that the above copyright notice and this permission notice appear
-# in all copies.
-#
-# THE SOFTWARE IS PROVIDED "AS IS" AND NETDEF DISCLAIMS ALL WARRANTIES
-# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
-# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL NETDEF BE LIABLE FOR
-# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY
-# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
-# WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
-# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
-# OF THIS SOFTWARE.
-#
 
 """
 test_route_scale1.py: Testing route scale
@@ -43,35 +30,51 @@ from lib import topotest
 from lib.topogen import Topogen, TopoRouter, get_topogen
 from lib.topolog import logger
 
-from scale_test_common import scale_build_common, scale_setup_module, route_install_helper, scale_test_memory_leak, scale_converge_protocols, scale_teardown_module
+from scale_test_common import (
+    scale_build_common,
+    scale_setup_module,
+    route_install_helper,
+    scale_test_memory_leak,
+    scale_converge_protocols,
+    scale_teardown_module,
+)
 
 
 pytestmark = [pytest.mark.sharpd]
 
+
 def build(tgen):
     scale_build_common(tgen)
 
+
 def setup_module(module):
     scale_setup_module(module)
 
+
 def teardown_module(_mod):
     scale_teardown_module(_mod)
 
+
 def test_converge_protocols():
     scale_converge_protocols()
 
+
 def test_route_install_2nh():
     route_install_helper(1)
 
+
 def test_route_install_4nh():
     route_install_helper(2)
 
+
 def test_route_install_16nh():
     route_install_helper(4)
 
+
 def test_memory_leak():
     scale_test_memory_leak()
 
+
 if __name__ == "__main__":
     args = ["-s"] + sys.argv[1:]
     sys.exit(pytest.main(args))
