@@ -55,7 +55,7 @@ int main(int argc, char *argv[])
 	yang_init(false, false);
 
 	/* Load YANG module. */
-	module = yang_module_load(argv[0]);
+	module = yang_module_load(argv[0], NULL);
 
 	/* Generate deviations. */
 	yang_snodes_iterate(module->info, generate_yang_deviation, 0, NULL);
