@@ -5,6 +5,8 @@
 
 #include <zebra.h>
 
+#include <net/route.h>
+
 #ifndef HAVE_NETLINK
 
 #include <net/if_types.h>
@@ -48,11 +50,7 @@ extern struct zebra_privs_t zserv_privs;
  * 0).  We follow this practice without questioning it, but it is a
  * bug if frr calls ROUNDUP with 0.
  */
-#ifdef __APPLE__
-#define ROUNDUP_TYPE	int
-#else
-#define ROUNDUP_TYPE	long
-#endif
+#define ROUNDUP_TYPE long
 
 /*
  * Because of these varying conventions, the only sane approach is for
@@ -834,12 +832,12 @@ int ifam_read(struct ifa_msghdr *ifam)
 	struct interface *ifp = NULL;
 	union sockunion addr, mask, brd;
 	bool dest_same = false;
-	char ifname[INTERFACE_NAMSIZ];
+	char ifname[IFNAMSIZ];
 	short ifnlen = 0;
 	bool isalias = false;
 	uint32_t flags = 0;
 
-	ifname[0] = ifname[INTERFACE_NAMSIZ - 1] = '\0';
+	ifname[0] = ifname[IFNAMSIZ - 1] = '\0';
 
 	/* Allocate and read address information. */
 	ifam_read_mesg(ifam, &addr, &mask, &brd, ifname, &ifnlen);
@@ -851,7 +849,7 @@ int ifam_read(struct ifa_msghdr *ifam)
 		return -1;
 	}
 
-	if (ifnlen && strncmp(ifp->name, ifname, INTERFACE_NAMSIZ))
+	if (ifnlen && strncmp(ifp->name, ifname, IFNAMSIZ))
 		isalias = true;
 
 	/*
@@ -995,7 +993,7 @@ void rtm_read(struct rt_msghdr *rtm)
 	int flags;
 	uint32_t zebra_flags;
 	union sockunion dest, mask, gate;
-	char ifname[INTERFACE_NAMSIZ + 1];
+	char ifname[IFNAMSIZ + 1];
 	short ifnlen = 0;
 	struct nexthop nh;
 	struct prefix p;
@@ -1626,6 +1624,7 @@ void kernel_update_multi(struct dplane_ctx_list_head *ctx_list)
 		case DPLANE_OP_INTF_ADDR_ADD:
 		case DPLANE_OP_INTF_ADDR_DEL:
 		case DPLANE_OP_STARTUP_STAGE:
+		case DPLANE_OP_SRV6_ENCAP_SRCADDR_SET:
 			zlog_err("Unhandled dplane data for %s",
 				 dplane_op2str(dplane_ctx_get_op(ctx)));
 			res = ZEBRA_DPLANE_REQUEST_FAILURE;
