@@ -31,9 +31,6 @@ extern "C" {
 
 struct zebra_vrf;
 
-/* Default port information. */
-#define ZEBRA_VTY_PORT                2601
-
 /* Default configuration filename. */
 #define DEFAULT_CONFIG_FILE "zebra.conf"
 
@@ -121,7 +118,7 @@ struct zserv {
 	vrf_bitmap_t ridinfo[AFI_MAX];
 
 	/* Router-id information. */
-	vrf_bitmap_t nhrp_neighinfo[AFI_MAX];
+	vrf_bitmap_t neighinfo[AFI_MAX];
 
 	bool notify_owner;
 
@@ -185,6 +182,9 @@ struct zserv {
 	uint32_t local_es_evi_add_cnt;
 	uint32_t local_es_evi_del_cnt;
 	uint32_t error_cnt;
+	uint32_t nhg_add_cnt;
+	uint32_t nhg_upd8_cnt;
+	uint32_t nhg_del_cnt;
 
 	time_t nh_reg_time;
 	time_t nh_dereg_time;
@@ -237,8 +237,7 @@ DECLARE_HOOK(zserv_client_connect, (struct zserv *client), (client));
 DECLARE_KOOH(zserv_client_close, (struct zserv *client), (client));
 
 #define DYNAMIC_CLIENT_GR_DISABLED(_client)                                    \
-	((_client->proto <= ZEBRA_ROUTE_CONNECT)                               \
-	 || !(_client->gr_instance_count))
+	((_client->proto <= ZEBRA_ROUTE_LOCAL) || !(_client->gr_instance_count))
 
 /*
  * Initialize Zebra API server.
@@ -255,15 +254,24 @@ extern void zserv_init(void);
 extern void zserv_close(void);
 
 /*
- * Start Zebra API server.
+ * Open Zebra API server socket.
  *
- * Allocates resources, creates the server socket and begins listening on the
- * socket.
+ * Create and open the server socket.
  *
  * path
  *    where to place the Unix domain socket
  */
-extern void zserv_start(char *path);
+extern void zserv_open(const char *path);
+
+/*
+ * Start Zebra API server.
+ *
+ * Allocates resources and begins listening on the server socket.
+ *
+ * path
+ *    where to place the Unix domain socket
+ */
+extern void zserv_start(const char *path);
 
 /*
  * Send a message to a connected Zebra API client.
@@ -379,6 +387,7 @@ __attribute__((__noreturn__)) void zebra_finalize(struct event *event);
 /*
  * Graceful restart functions.
  */
+extern void zebra_gr_client_final_shutdown(struct zserv *client);
 extern int zebra_gr_client_disconnect(struct zserv *client);
 extern void zebra_gr_client_reconnect(struct zserv *client);
 extern void zebra_gr_stale_client_cleanup(struct list *client_list);
